/*	@(#)ARR.c	1.12 2/23/93
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991-93, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 *  .00 02-08-91 rac	initial version
 *  .01 07-31-91 rac	installed in SCCS
 *  .02 09-10-91 rac	added icon; put working directory on title bar
 *  .03 10-15-91 rac	get default printer from EPICS_PRINTER;
 *  .04 11-18-91 rac	use guiInit;
 *  .05 03-01-92 rac	allow specifying some defaults from cmd line; use
 *			PR.o rather than ARR_PR.o; rearrange layout; add
 *			restricted time range capability; add data
 *			`filtering' test
 *  .05 08-31-92 rac	avoid a crash when user `quits' from title bar
 *  .06 01-25-93 rac	add `means' to `work with'
 * ARR.c,v
 * Revision 1.2  1994/05/06 19:05:17  mrk
 * conversion from sccs to cvs
 *
 *
 */
/*+/mod***********************************************************************
* TITLE	ARR.c - AR retrieval utility
*
* DESCRIPTION
*	Retrieve information from archive files or Channel Access for display
*	or export.  Display can be in textual or graphical form.
*
* % ARR [options]
*
*   options:		(options starting with -D allow changing defaults;
*			the "canned" default is shown in ()'s )
*	-Dssf dir	default directory for sample set files (arSet)
*	-Dbcf dir	default directory for by channel files (arChan)
*	-Dret dir	default directory for retrieval files (arReq)
*	-Dexp dir	default directory for export files (data)
*
* WISH LIST
* o	be more flexible in dealing with operator input of time stamp and
*	delta time
* o	allow wild card selection of channels to retrieve
* o	change axis labeling so horizontal is labeled only once, in larger
*	letters
* o	allow `editing' retrieved samples--comment out undesired ones,
*	change individual values, etc.
* o	allow configuring channel for plotting--label, scaling, nTicks, etc.
*-***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define XVIEW
#if defined XVIEW
#   include <xview/xview.h>
#   include <xview/frame.h>
#   include <xview/openmenu.h>
#   include <xview/panel.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <guiSubr.h>
#include <sydDefs.h>
#include <cadef.h>
#include <enumShared.h>
#include <pprPlotDefs.h>
#include <sydPlotDefs.h>
#include "EX.h"
#include "PL.h"
#include "PR.h"
#include "ARR.h"

#include "ARR_RET.h"

void quit_pb();
void restrictTime_xvo();
void useStats_xvo();


ARR_CTX	arrCtx;


/*+/subr**********************************************************************
* NAME	main
*
*-*/
void timerFunc(), monFunc();

main(argc,argv)
int	argc;
char	*argv[];
{
    Panel	panel;
    char	title[120], *pwd;
    int		pwdLen;
    static unsigned short arrIcon_bits[]={
#include "ARR.icon"
    };
    int		y=10, x=5, ht=0;
    ARR_CTX	*pCtx;
    int		i;

    bzero((char *)&arrCtx, sizeof(arrCtx));
    pCtx = &arrCtx;
    pCtx->pChanHead = NULL;
    pCtx->pChanTail = NULL;
    pCtx->pSspec = NULL;
    pCtx->restrictBegTs.secPastEpoch = 0;
    pCtx->restrictEndTs.secPastEpoch = 0;

    strcpy(title, "ARR");

    panel = guiInit(&pCtx->guiCtx, &argc, argv, title, 100, 100, 0, 0);
    guiIconCreate(pCtx->guiCtx.baseFrame, arrIcon_bits);
    pCtx->pGuiPrt = guiPrinter(&pCtx->guiCtx,
		pCtx->guiCtx.baseFrame, "ARR: printer setup", 1);

    pCtx->dfltSSF_dir = "arSet";
    pCtx->dfltBCF_dir = "arChan";
    pCtx->dfltRet_dir = "arReq";
    pCtx->dfltExp_dir = "data";
    for (i=1; i<argc; i++) {
	if (strcmp(argv[i], "-Dssf") == 0)
	    pCtx->dfltSSF_dir = argv[++i];
	else if (strcmp(argv[i], "-Dbcf") == 0)
	    pCtx->dfltBCF_dir = argv[++i];
	else if (strcmp(argv[i], "-Dret") == 0)
	    pCtx->dfltRet_dir = argv[++i];
	else if (strcmp(argv[i], "-Dexp") == 0)
	    pCtx->dfltExp_dir = argv[++i];
    }

    RETinit(pCtx, panel, &x, &y, &ht);

    y += 5 + ht; x = 5; ht = 0;
    pCtx->useStats_PC = guiChoice("work with:",
		"data values", "mean", "mean and stdDev", NULL,
		panel, 1, 1, 0, &x,&y,&ht, useStats_xvo, ARRCTX,pCtx,0,NULL);
    y += 5 + ht; x = 5; ht = 0;
    pCtx->restrict_PC = guiChoice("time range:", "full", "restricted", NULL,
		panel, 1, 1, 0, &x,&y,&ht, restrictTime_xvo,ARRCTX,pCtx,0,NULL);
    y += 5 + ht; x = 5; ht = 0;

    PLinitButton(&pCtx->pPlCtx, &pCtx->guiCtx, pCtx->pGuiPrt,
		"ARR: plotting", panel, &x,&y,&ht, &pCtx->pSspec);
    pCtx->pPlCtx->pRestrict_PC = &pCtx->restrict_PC;
    pCtx->pPlCtx->pRestrictBegTs = &pCtx->restrictBegTs;
    pCtx->pPlCtx->pRestrictEndTs = &pCtx->restrictEndTs;
    PRinitButton(&pCtx->pPrCtx, &pCtx->guiCtx, pCtx->pGuiPrt,
		"ARR: printing", panel, &x,&y,&ht, &pCtx->pSspec);
    EXinitButton(&pCtx->pExCtx, &pCtx->guiCtx, "ARR",
		"ARR: exporting", pCtx->dfltExp_dir,
		panel, &x,&y,&ht, &pCtx->pSspec);

    y += 5 + ht; x = 5; ht = 0;
    guiButton("QUIT", panel, &x,&y,&ht, quit_pb, NULL, ARRCTX,pCtx,0,NULL);

    window_fit(panel);
    window_fit(pCtx->guiCtx.baseFrame);

    guiTimer(.1, timerFunc, pCtx);

    xv_main_loop(pCtx->guiCtx.baseFrame);
    arrQuit(pCtx, 0);
    return 0;
}
/*+/subr**********************************************************************
* NAME	restrictTime_xvo
*
* DESCRIPTION
*	Apply or remove restricted time.
*
*	The restriction range is established in PL.c.  When the plotting
*	software applies the restriction, it puts the time stamps in
*	the main ARR context block.  This allows the choice button on
*	the main window to know what the time range is.
*
*-*/
void
restrictTime_xvo(item)
Panel_item item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);

    if (pCtx->pSspec == NULL)
	return;
    if ((int)xv_get(item, PANEL_VALUE) == 0)
	sydSampleSetRestrict(pCtx->pSspec, NULL, NULL);
    else {
	sydSampleSetRestrict(pCtx->pSspec,
				pCtx->restrictBegTs, pCtx->restrictEndTs);
    }
}
/*+/subr**********************************************************************
* NAME	useStats_xvo
*
*-*/
void
useStats_xvo(item)
Panel_item item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    if ((int)xv_get(item, PANEL_VALUE) == 0) {
	pCtx->pExCtx->useStats = pCtx->pExCtx->useMeans = 0;
	pCtx->pPlCtx->useStats = pCtx->pPlCtx->useMeans = 0;
	pCtx->pPrCtx->useStats = pCtx->pPrCtx->useMeans = 0;
    }
    else if ((int)xv_get(item, PANEL_VALUE) == 1) {
	pCtx->pExCtx->useStats = 0; pCtx->pExCtx->useMeans = 1;
	pCtx->pPlCtx->useStats = 0; pCtx->pPlCtx->useMeans = 1;
	pCtx->pPrCtx->useStats = 0; pCtx->pPrCtx->useMeans = 1;
    }
    else {
	pCtx->pExCtx->useStats = 1; pCtx->pExCtx->useMeans = 0;
	pCtx->pPlCtx->useStats = 1; pCtx->pPlCtx->useMeans = 0;
	pCtx->pPrCtx->useStats = 1; pCtx->pPrCtx->useMeans = 0;
    }
}

/*+/subr**********************************************************************
* NAME	quit_pb
*
*-*/
void
quit_pb(item)
Panel_item item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    arrQuit(pCtx, 1);
}

arrQuit(pCtx, destroyFlag)
ARR_CTX	*pCtx;
int	destroyFlag;
{
    if (destroyFlag) {
	if (RETquitCheck(pCtx) != 0)
	    return;
    }
    if (pCtx->pExCtx != NULL) {
	EXquit(pCtx->pExCtx, destroyFlag);
	pCtx->pExCtx = NULL;
    }
    if (pCtx->pPlCtx != NULL) {
	PLquit(pCtx->pPlCtx, destroyFlag);
	pCtx->pPlCtx = NULL;
    }
    if (pCtx->pPrCtx != NULL) {
	PRquit(pCtx->pPrCtx, destroyFlag);
	pCtx->pPrCtx = NULL;
    }
    if (pCtx->pRetCtx != NULL) {
	RETquit(pCtx->pRetCtx, destroyFlag);
	pCtx->pRetCtx = NULL;
    }
    if (destroyFlag)
	xv_destroy_safe(pCtx->guiCtx.baseFrame);
}

/*+/subr**********************************************************************
* NAME	timerFunc
*
*-*/
void
timerFunc(pCtx, dummy)
ARR_CTX	*pCtx;
void	*dummy;
{
    void	*RETbutton_timerFunc();
    PL_CTX	*pPlCtx;
    SYD_PL_MSTR	*pMstr;
    SYD_PL_SLAVE *pSlave;
    PPR_AREA	*pArea;

    if (pCtx->pSspec != NULL && pCtx->pSspec->type == SYD_TY_CA)
	ca_pend_event(.001);

    if ((pPlCtx = pCtx->pPlCtx) != NULL) {
	if ((int)xv_get(pPlCtx->PLwin_CF, XV_SHOW) != 0) {
	    if (pPlCtx->PLplotting &&
		    (pPlCtx->PLmaster.plotAxis == SYD_PLAX_STRIP_Y ||
		    pPlCtx->PLmaster.plotAxis == SYD_PLAX_STRIP_YY) &&
		    (int)xv_get(pPlCtx->stripMode_PCS, PANEL_VALUE) == 0) {
		int	shiftIt=0;
		double	elapsed;
		TS_STAMP now;

		tsLocalTime(&now);
		if (pPlCtx->stripLastTs.secPastEpoch == 0) {
		    shiftIt = 1;
		    pPlCtx->stripLastTs = now;
		    elapsed = pPlCtx->stripDelay;
		}
		else {
		    TsDiffAsDouble(&elapsed, &now, &pPlCtx->stripLastTs);
		    if (elapsed >= pPlCtx->stripDelay) {
			elapsed -= fmod(elapsed, pPlCtx->stripDelay);
			shiftIt = 1;
		    }
		}
		if (shiftIt) {
		    pMstr = &pPlCtx->PLmaster;
		    for (pSlave=pMstr->pHead;pSlave!=NULL;pSlave=pSlave->pNext){
			pArea = pSlave->pArea;
			pprAreaShiftLeft(pArea, elapsed);
			if (pPlCtx->PLmaster.plotAxis == SYD_PLAX_STRIP_YY)
			    break;
		    }
		    tsAddDouble(&pPlCtx->stripLastTs, &pPlCtx->stripLastTs,
					elapsed);
		}
	    }
	}
    }

    RETbutton_timerFunc(pCtx);
}
#if 0
void
monFunc(arg)
struct event_handler_args arg;
{
    printf("%f\n", *(float *)arg.dbr);
}
#endif
