/*	@(#)ARR_EX.c	1.7 3/1/92
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991-92, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	02-08-91	rac	initial version
 * .02	07-31-91	rac	installed in SCCS
 * .03	08-21-91	rac	use new sydXxx routines
 * .04	11-25-91	rac	consolidate setup functions in export
 *				window; allow specifying directory and
 *				whether to append or over-write;
 * .05	12-02-91	rac	add `sample set' file as an export option
 * .06	02-26-92	rac	add "for arrays" as an export option
 * .07	03-01-92	rac	use default directories from context
 * ARR_EX.c,v
 * Revision 1.2  1994/05/06 19:05:21  mrk
 * conversion from sccs to cvs
 *
 *
 * make options
 *	-DXWINDOWS	to use xview/X Window System
 */
/*+/mod***********************************************************************
* TITLE	ARR_EX.c - export routines for ARR
*
* DESCRIPTION
*
*-***************************************************************************/
#include <stdio.h>

#if defined XWINDOWS
#   include <xview/xview.h>
#   include <xview/frame.h>
#   include <xview/panel.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <sydDefs.h>

#include "ARR.h"

#define EX_FNAME_DIM 80

#define EX_NAME_DIM 80
typedef struct arrExCtx {
    Panel_item	dir_PT;		/* PANEL_TEXT item for directory */
    char	dir[EX_FNAME_DIM];	/* directory */
    Panel_item	file_PT;	/* PANEL_TEXT item for file name */
    char	file[EX_FNAME_DIM];	/* file name */
    Panel_item	fileMode_PCS;	/* PANEL_CHOICE_STACK item for file mode */
    Frame	EXwin_CF;	/* command frame for exporting */
    Panel_item	EXtype_PCS;	/* export type */
    Panel_item	EXstat_PCS;	/* value status--suppress or show */
} ARR_EX_CTX;


/*+/subr**********************************************************************
* NAME	EXport_xvo - handle the "Export" menu item
*
*	o   decode the export option
*	o   do the export
*-*/
static void EXport_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, PCTX);
    ARR_EX_CTX	*pExCtx=pCtx->pExCtx;
    ARR_OT_TYPE cmdType=(ARR_OT_TYPE)xv_get(item, XV_KEY_DATA, OUT_TYPE);
    SYD_SPEC    *pSspec=pCtx->pSspec;
    int		option, show;
    FILE	*f;
    char	path[2*EX_FNAME_DIM];
    int		modeCode;

    if ((int)xv_get(pExCtx->EXwin_CF, XV_SHOW) == FALSE)
	xv_set(pExCtx->EXwin_CF, XV_SHOW, TRUE, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
    if (pSspec == NULL || pSspec->chanCount <= 0) {
	if (cmdType == ARR_OT_NULL)
	    return;
        guiNotice(&pCtx->guiCtx, "no channels selected; use RETRIEVAL Setup...");
        return;
    }
    if (pSspec->sampleCount <= 0) {
	if (cmdType == ARR_OT_NULL)
	    return;
        guiNotice(&pCtx->guiCtx, "no samples to export; use RETRIEVE");
        return;
    }

    strcpy(pExCtx->dir, (char *)xv_get(pExCtx->dir_PT, PANEL_VALUE));
    strcpy(pExCtx->file, (char *)xv_get(pExCtx->file_PT, PANEL_VALUE));
    modeCode = (int)xv_get(pExCtx->fileMode_PCS, PANEL_VALUE);
    if (strlen(pExCtx->file) == 0) {
	guiNotice(&pCtx->guiCtx, "no file selected");
	return;
    }
    sprintf(path, "%s/%s", pExCtx->dir, pExCtx->file);
    if (modeCode == 0) {	/* over-write mode */
	if ((f = fopen(path, "w")) == NULL) {
	    guiNoticeFile(&pCtx->guiCtx, "Unable to open for output", path);
	    return;
	}
    }
    else {			/* append mode */
	if ((f = fopen(path, "a")) == NULL) {
	    guiNoticeFile(&pCtx->guiCtx, "Unable to open for output", path);
	    return;
	}
    }

    if ((int)xv_get(pExCtx->EXtype_PCS, PANEL_VALUE) == 0) {
	if ((show = (int)xv_get(pExCtx->EXstat_PCS, PANEL_VALUE)) == 0)
	    option = 1;
	else
	    option = 2;
	sydSampleSetExport(pSspec, f, option);
    }
    else if ((int)xv_get(pExCtx->EXtype_PCS, PANEL_VALUE) == 1) {
	if ((show = (int)xv_get(pExCtx->EXstat_PCS, PANEL_VALUE)) == 0)
	    option = 3;
	else
	    option = 4;
	sydSampleSetExport(pSspec, f, option);
    }
    else {
	sydSampleSetWriteSSF(pSspec, f, "PROGRAM: ARR", "");
    }
    fclose(f);
}

/*+/subr**********************************************************************
* NAME	EXwinInitCF - initialize the export command frame
*
*-*/
static void EXwinInitCF(pCtx)
ARR_CTX	*pCtx;
{
    ARR_EX_CTX	*pExCtx=pCtx->pExCtx;
    FILE	*f;
    Panel	panel;
    Xv_opaque	item;
    int		y=5, x=5, ht=0;

/*-----------------------------------------------------------------------------
* Dismiss  Export to: file
* Directory: _______________________
* File: _______________________  V over-write
*                                  append
* convert for               value status
* V spreadsheet             V suppress
*   sample set file           show
*   spreadsheet for arrays
*----------------------------------------------------------------------------*/

    pExCtx->EXwin_CF =
	    guiFrameCmd("ARR: EXPORT", pCtx->guiCtx.baseFrame, &panel, NULL);
    guiButton("Dismiss", panel, &x, &y, &ht, guiCFdismiss_xvo, NULL,
				    GUI_CF, pExCtx->EXwin_CF, 0, NULL);
    guiMessage("Export to:", panel, &x, &y, &ht);
    guiButton("file", panel, &x, &y, &ht, EXport_xvo, NULL,
	    PCTX, pCtx, OUT_TYPE, ARR_OT_DO);
    y += ht + 5; x = 5; ht = 0;

    pExCtx->dir_PT = guiTextField("Directory:", panel, &x,&y,&ht, NULL,
		EX_FNAME_DIM-1, 40, 0, NULL, 0, NULL);
    strcpy(pExCtx->dir, pCtx->dfltExp_dir);
    xv_set(pExCtx->dir_PT, PANEL_VALUE, pExCtx->dir, NULL);
    y += ht + 5; x = 5; ht = 0;

    pExCtx->file_PT = guiTextField("File:", panel, &x,&y,&ht, NULL,
		EX_FNAME_DIM-1, 40, 0, NULL, 0, NULL);
    strcpy(pExCtx->file, "");
    xv_set(pExCtx->file_PT, PANEL_VALUE, pExCtx->file, NULL);
    pExCtx->fileMode_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_CHOICE_STRINGS,	"over-write","append",NULL,
	PANEL_VALUE,	0,
	XV_X, x, XV_Y, y-5, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;

    y += ht + 5; x = 5; ht = 0;

    pExCtx->EXtype_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_LABEL_STRING,	"convert for",
	PANEL_CHOICE_STRINGS,	"spreadsheet", "spreadsheet for arrays",
				"sample set file", NULL,
	PANEL_VALUE,		0,
	XV_X, x, XV_Y, y, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    ht = ArrMax(ht, (int)xv_get(item, XV_HEIGHT));

    pExCtx->EXstat_PCS = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_LABEL_STRING,	"value status",
	PANEL_CHOICE_STRINGS,	"suppress", "show", NULL,
	PANEL_VALUE,		0,
	XV_X, x, XV_Y, y, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    ht = ArrMax(ht, (int)xv_get(item, XV_HEIGHT));

    window_fit(panel);
    window_fit(pExCtx->EXwin_CF);
}

/*+/subr**********************************************************************
* NAME	EXinit - initialize for exporting
*
* DESCRIPTION
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
EXinit(pCtx, panel, pX, pY, pHt)
ARR_CTX	*pCtx;
Panel	panel;
int	*pX, *pY, *pHt;
{
    ARR_EX_CTX *pExCtx;

    if ((pExCtx = (ARR_EX_CTX *)GenMalloc(sizeof(ARR_EX_CTX))) == NULL)
	assertAlways(0);
    pCtx->pExCtx = pExCtx;
    EXwinInitCF(pCtx);
    guiButton("exporting...", panel, pX, pY, pHt, guiCFshowPin_xvo, NULL,
			GUI_PCTX, &pCtx->guiCtx, GUI_CF, pExCtx->EXwin_CF);
}

/*+/subr**********************************************************************
* NAME	EXquit
*
*-*/
void
EXquit(pCtx)
ARR_CTX	*pCtx;
{
    xv_destroy_safe(pCtx->pExCtx->EXwin_CF);
    GenFree(pCtx->pExCtx);
}
