
/*	@(#)ARR_PL.c	1.6 1/28/92
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	02-08-91	rac	initial version
 * .02	07-31-91	rac	installed in SCCS
 * .03	09-11-91	rac	set to use plot icon
 * .04	11-18-91	rac	use modified call to guiShellCmd; consolidate
 *				setup functions into setup command frame
 * .05	12-02-91	rac	add plot auto-range capability
 * .06	01-21-92	rac	honor back/foreground from .Xdefaults/cmdLine;
 *				put temp files in /tmp
 * ARR_PL.c,v
 * Revision 1.2  1994/05/06 19:05:24  mrk
 * conversion from sccs to cvs
 *
 *
 * make options
 *	-DXWINDOWS	to use xview/X Window System
 */
/*+/mod***********************************************************************
* TITLE	ARR_PL.c - plotting routines for ARR
*
* DESCRIPTION
*
*-***************************************************************************/
#include <stdio.h>

#if defined XWINDOWS
#   include <xview/xview.h>
#   include <xview/cms.h>
#   include <xview/frame.h>
#   include <xview/icon.h>
#   include <xview/notice.h>
#   include <xview/panel.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <arAccessLib.h>
#include <sydDefs.h>
#include <sydPlotDefs.h>
#include <db_access.h>
#include <pprPlotDefs.h>

#include "ARR.h"

#define EPS_FNAME_DIM 80
typedef struct arrPlCtx {
    Menu_item	plotHC_MI;	/* menu item for plotHC */
    Menu_item	plotEPS_MI;	/* menu item for plotEPS */
    int		PLplotting;	/* plotting on screen is in progress */
    Panel_item	PLoverlay_PCS;	/* overlay type--grid */
    Panel_item	PLhoriz_PCS;	/* horiz axis type--time, value, bin */
    Panel_item	PLscale_PCS;	/* axis auto scaling--no or yes */
    Panel_item	PLseparate_PCS;	/* separate or combined */
    Panel_item	PLstat_PCS;	/* value status--suppress or show */
    Panel_item	PLwith_PCS;	/* plot with--lines, marks, or points */
    Panel_item	dir_PT;		/* PANEL_TEXT item for EPS directory */
    char	dir[EPS_FNAME_DIM];	/* EPS directory */
    Panel_item	file_PT;	/* PANEL_TEXT item for EPS file name */
    char	file[EPS_FNAME_DIM];	/* EPS file name */
    Panel_item	fileMode_PCS;	/* PANEL_CHOICE_STACK item for file mode */
    Frame	PLsetup_CF;	/* command frame for setting up plots */
    Frame	PLwin_F;	/* frame containing plot canvas */
    Canvas	PLwinCanvas;	/* the plot canvas */
    unsigned long PLwinBg;	/* background pixel value */
    unsigned long PLwinFg;	/* foreground pixel value */
    int		PLwinWidth;	/* width of plot window frame */
    int		PLwinHeight;	/* height of plot window frame */
    PPR_WIN	*pWin;		/* pprPlot window structure pointer */
    SYD_PL_MSTR	PLmaster;	/* sync data plot master struct */
} ARR_PL_CTX;


/*+/subr**********************************************************************
* NAME	PLwinDone_xvo
*
*-*/
static void
PLwinDone_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, PCTX);
    long	stat;

    if (pCtx->pPlCtx->PLplotting) {
	pCtx->pPlCtx->PLplotting = 0;
	stat = sydPlotDone(&pCtx->pPlCtx->PLmaster, 1);
	assertAlways(stat == S_syd_OK);
    }
    xv_set(pCtx->pPlCtx->PLwin_F, XV_SHOW, FALSE, NULL);
}

/*+/subr**********************************************************************
* NAME	PLwinEvent - handle events for plot window
*
*	WIN_REPAINT
*	o   repaint the plot
*
*-*/
static void
PLwinEvent(canvas,event)
Canvas	canvas;
Event *event;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(canvas, XV_KEY_DATA, PCTX);
    int		x,y;	/* dummy for position */
    long	stat;

    if (event_id(event) == WIN_REPAINT) {
	if (pCtx->pPlCtx->PLplotting) {
	    stat = sydPlotWinReplot(&pCtx->pPlCtx->PLmaster);
	    assertAlways(stat == S_syd_OK);
	}
    }
}

/*+/subr**********************************************************************
* NAME	PLwinPlot_xvo - do a plot
*
* NOTES
* 1.	Under XView, the plot frame already exists and a repaint event has
*	already occurred, even as the frame was created.  Because of this,
*	the first repaint is done here, manually.  Succeeding repaints for
*	the plot are triggered by the event handler.
*
*-*/
static void PLwinPlot_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, PCTX);
    ARR_PL_CTX	*pPlCtx = pCtx->pPlCtx;
    ARR_OT_TYPE	cmdType=(ARR_OT_TYPE)xv_get(item, XV_KEY_DATA, OUT_TYPE);
    SYD_SPEC	*pSspec=pCtx->pSspec;
    Window	plotWindow;
    GC		plotGC;
    SYD_PL_SLAVE	*pSlave;
    long	stat;
    SYD_CHAN	*pSChan;
    TS_STAMP	now;
    char	nowText[32], begText[28], endText[28];
    char	title[120];
    char	command[160];
    char	resultBuf[80];
    int		type, point, mark, show, with, line, autoScale;
    SYD_PLAX	plotType;
    SYD_PL_MSTR master, *pMstr=&master;

    if ((int)xv_get(pPlCtx->PLwin_F, XV_SHOW) == FALSE)
	xv_set(pPlCtx->PLwin_F, XV_SHOW, TRUE, NULL);
    if (pPlCtx->PLplotting && cmdType == ARR_OT_PLAIN) {
	pCtx->pPlCtx->PLplotting = 0;
	stat = sydPlotDone(&pCtx->pPlCtx->PLmaster, 1);
	assertAlways(stat == S_syd_OK);
    }
    if (pSspec == NULL || pSspec->chanCount <= 0) {
	if (cmdType == ARR_OT_NULL)
	    return;
	guiNotice(&pCtx->guiCtx, "no channels selected; use RETRIEVE setup...");
	return;
    }
    strcpy(pCtx->HCname, (char *)xv_get(pCtx->PL_HCname_PT, PANEL_VALUE));
    xv_set(pCtx->PR_HCname_PT, PANEL_VALUE, pCtx->HCname, NULL);

    type = 2 * (int)xv_get(pPlCtx->PLhoriz_PCS, PANEL_VALUE);
    if ((int)xv_get(pPlCtx->PLseparate_PCS, PANEL_VALUE) == 1)
	type++;
    if (type == 0)	plotType = SYD_PLAX_TY;
    else if (type == 1)	plotType = SYD_PLAX_TYY;
    else if (type == 2)	plotType = SYD_PLAX_XY;
    else if (type == 3)	plotType = SYD_PLAX_XYY;
    else if (type == 4)	plotType = SYD_PLAX_Y;
    else if (type == 5)	plotType = SYD_PLAX_YY;
    if (plotType == SYD_PLAX_XY || plotType == SYD_PLAX_XYY) {
	if (pSspec->chanCount < 2) {
	    if (cmdType == ARR_OT_NULL)
		return;
	    guiNotice(&pCtx->guiCtx, "no channel to plot on X axis");
	    return;
	}
    }
    if (pSspec->sampleCount <= 0) {
	if (cmdType == ARR_OT_NULL)
	    return;
	guiNotice(&pCtx->guiCtx, "no samples to plot; use RETRIEVE");
	return;
    }
    line = 1;
    point = mark = 0;
    with = (int)xv_get(pPlCtx->PLwith_PCS, PANEL_VALUE);
    if (with == 1)	mark = 1, line = 0;
    else if (with == 2)	point = 1, line = 0;
    show = (int)xv_get(pPlCtx->PLstat_PCS, PANEL_VALUE);
    autoScale = (int)xv_get(pPlCtx->PLscale_PCS, PANEL_VALUE);

    (void)tsLocalTime(&now);
    (void)tsStampToText(&now, TS_TEXT_MONDDYYYY, nowText);
    (void)tsStampToText(&pSspec->pTimeStamp[pSspec->firstData], TS_TEXT_MMDDYY,
								begText);
    (void)tsStampToText(&pSspec->pTimeStamp[pSspec->lastData], TS_TEXT_MMDDYY,
								endText);
    (void)sprintf(title, "samples for %s through %s", begText, endText);

    if (cmdType == ARR_OT_PLAIN || cmdType == ARR_OT_NULL) {
	pPlCtx->PLplotting = 1;
	plotWindow =
	    (Window)xv_get(canvas_paint_window(pPlCtx->PLwinCanvas), XV_XID);
	plotGC =
	    DefaultGC(pCtx->guiCtx.pDisp, DefaultScreen(pCtx->guiCtx.pDisp));
	XSetBackground(pCtx->guiCtx.pDisp, plotGC, pPlCtx->PLwinBg);
	XSetForeground(pCtx->guiCtx.pDisp, plotGC, pPlCtx->PLwinFg);

	stat = sydPlotInitUW(&pPlCtx->PLmaster, pSspec,
		    		pCtx->guiCtx.pDisp, plotWindow, plotGC);
	assertAlways(stat == S_syd_OK);
	stat = sydPlotSetTitles(&pPlCtx->PLmaster, title, NULL, NULL, NULL);
	assertAlways(stat == S_syd_OK);
	pPlCtx->PLmaster.plotAxis = plotType;
	pPlCtx->PLmaster.linePlot = line;
	pPlCtx->PLmaster.markPlot = mark;
	pPlCtx->PLmaster.pointPlot = point;
	pPlCtx->PLmaster.showStat = show;
	for (pSChan=pSspec->pChanHead; pSChan!=NULL; pSChan = pSChan->pNext) {
	    pSlave = sydPlotChanAdd(&pPlCtx->PLmaster, pSChan);
	    assert(pSlave != NULL);
	    if (autoScale)
		sydPlotAxisAutoRange(pSlave);
	}
	stat = sydPlotWinReplot(&pPlCtx->PLmaster);
	assertAlways(stat == S_syd_OK);
    }
    else if (cmdType == ARR_OT_EPS) {
	FILE	*f;
	int	answer;
	char	path[2*EPS_FNAME_DIM];
	int	modeCode;

	strcpy(pPlCtx->dir, (char *)xv_get(pPlCtx->dir_PT, PANEL_VALUE));
	strcpy(pPlCtx->file, (char *)xv_get(pPlCtx->file_PT, PANEL_VALUE));
	modeCode = (int)xv_get(pPlCtx->fileMode_PCS, PANEL_VALUE);
	if (strlen(pPlCtx->file) == 0) {
	    guiNotice(&pCtx->guiCtx,
		"pick an EPS file using the \"plotting setup\" window");
	    return;
	}
	sprintf(path, "%s/%s", pPlCtx->dir, pPlCtx->file);
        if (modeCode == 0) {	/* append mode */
	    if ((f = fopen(path, "a")) == NULL) {
		guiNoticeFile(&pCtx->guiCtx, "Unable to open for output", path);
		return;
	    }
	}
	else {			/* over-write mode */
	    if ((f = fopen(path, "w")) == NULL) {
		guiNoticeFile(&pCtx->guiCtx, "Unable to open for output", path);
		return;
	    }
	}

	if (pPlCtx->PLplotting) {
	    master.width = pPlCtx->PLmaster.width;
	    master.height = pPlCtx->PLmaster.height;
	}
	else {
	    master.width = 512;
	    master.height = 512;
	}
	stat = sydPlotInit(&master, pSspec, PPR_WIN_EPS, path, NULL, 0);
	assertAlways(stat == S_syd_OK);
	stat = sydPlotSetTitles(&master, title, NULL, NULL, NULL);
	assertAlways(stat == S_syd_OK);
	master.plotAxis = plotType;
	master.linePlot = line;
	master.markPlot = mark;
	master.pointPlot = point;
	master.showStat = show;
	for (pSChan=pSspec->pChanHead; pSChan!=NULL; pSChan = pSChan->pNext) {
	    pSlave = sydPlotChanAdd(&master, pSChan);
	    assert(pSlave != NULL);
	    if (autoScale)
		sydPlotAxisAutoRange(pSlave);
	}

	stat = sydPlotWinLoop(&master);
	assert(stat == S_syd_OK);

	stat = sydPlotDone(&master, 1);
	assert(stat == S_syd_OK);
    }
    else if (cmdType == ARR_OT_HC) {
	char	command[120];

	if (strlen(pCtx->HCname) == 0) {
	    guiNotice(&pCtx->guiCtx,
		"pick a printer using the \"plotting setup\" window");
	    return;
	}
	sprintf(pCtx->HCfName, "/tmp/ARR%d.HC.tmp", getpid());
	if (pPlCtx->PLplotting) {
	    master.width = pPlCtx->PLmaster.width;
	    master.height = pPlCtx->PLmaster.height;
	}
	else {
	    master.width = 512;
	    master.height = 512;
	}
	stat = sydPlotInit(&master, pSspec,
				PPR_WIN_POSTSCRIPT, pCtx->HCfName, NULL, 0);
	assertAlways(stat == S_syd_OK);
	stat = sydPlotSetTitles(&master, title, NULL, NULL, NULL);
	assertAlways(stat == S_syd_OK);
	master.plotAxis = plotType;
	master.linePlot = line;
	master.markPlot = mark;
	master.pointPlot = point;
	master.showStat = show;
	for (pSChan=pSspec->pChanHead; pSChan!=NULL; pSChan = pSChan->pNext) {
	    pSlave = sydPlotChanAdd(&master, pSChan);
	    assert(pSlave != NULL);
	    if (autoScale)
		sydPlotAxisAutoRange(pSlave);
	}

	stat = sydPlotWinLoop(&master);
	assert(stat == S_syd_OK);

	stat = sydPlotDone(&master, 1);
	assert(stat == S_syd_OK);
	if (cmdType == ARR_OT_HC) {
	    sprintf(command, "lpr -P%s %s", pCtx->HCname, pCtx->HCfName);
	    guiShellCmd(pCtx->guiCtx, command, lprClient, NULL, NULL);
	}
    }
}

/*+/subr**********************************************************************
* NAME	PLsetupInitCF
*
*	o   "plot with" choice stack, to choose line, point, or mark plots
*	o   "horiz vs vert" choice stack, to choose layout and use of axes
*	o   "value status" choice stack, to choose whether to suppress or
*	    show the status of individual points on plots
*
*-*/
static void
PLsetupInitCF(pCtx)
ARR_CTX	*pCtx;
{   ARR_PL_CTX	*pPlCtx=pCtx->pPlCtx;
    Panel	panel;
    int		x=5, y=5, ht=0;
    Xv_opaque	item;

    pCtx->pPlCtx->PLsetup_CF =
	guiFrameCmd("ARR: plotting setup", pCtx->guiCtx.baseFrame,&panel,NULL);

/*-----------------------------------------------------------------------------
* Dismiss
* EPS directory: _______________________
* EPS file: _______________________  V append
*                                      over-write
* Printer: ______
* plot with: V lines    share? V separate   overlay: V grid
*              marks             shared
*              points
*----------------------------------------------------------------------------*/

    guiButton("Dismiss", panel, &x, &y, &ht, guiCFdismiss_xvo, NULL,
		GUI_CF, pCtx->pPlCtx->PLsetup_CF, 0, NULL);
    y += ht + 10; x = 5; ht = 0;

    pPlCtx->dir_PT = guiTextField("EPS directory:", panel, &x,&y,&ht, NULL,
		EPS_FNAME_DIM-1, 40, 0, NULL, 0, NULL);
    strcpy(pPlCtx->dir, "./data");
    xv_set(pPlCtx->dir_PT, PANEL_VALUE, pPlCtx->dir, NULL);
    y += ht + 5; x = 5; ht = 0;

    pPlCtx->file_PT = guiTextField("EPS file:", panel, &x,&y,&ht, NULL,
		EPS_FNAME_DIM-1, 40, 0, NULL, 0, NULL);
    strcpy(pPlCtx->file, "");
    xv_set(pPlCtx->file_PT, PANEL_VALUE, pPlCtx->file, NULL);
    pPlCtx->fileMode_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_CHOICE_STRINGS,	"append","over-write",NULL,
	PANEL_VALUE,	1,
	XV_X, x, XV_Y, y-5, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    y += ht + 5; x = 5; ht = 0;

    pCtx->PL_HCname_PT = guiTextField("Printer:", panel, &x,&y,&ht,
		NULL, 19, 8, 0, NULL, 0, NULL);
    xv_set(pCtx->PL_HCname_PT, PANEL_VALUE, pCtx->dfltPrinter, NULL);
    y += ht + 5; x = 5; ht = 0;

    pCtx->pPlCtx->PLwith_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_LABEL_STRING,	"plot with",
	PANEL_CHOICE_STRINGS,	"lines","marks","points",NULL,
	PANEL_VALUE,		0,
	XV_X, x, XV_Y, y, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    ht = ArrMax(ht, (int)xv_get(item, XV_HEIGHT));

    pCtx->pPlCtx->PLseparate_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_LABEL_STRING,	"share?",
	PANEL_CHOICE_STRINGS,	"separate", "shared",
				NULL,
	PANEL_VALUE,		0,
	XV_X, x, XV_Y, y, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    ht = ArrMax(ht, (int)xv_get(item, XV_HEIGHT));

    pCtx->pPlCtx->PLoverlay_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_LABEL_STRING,	"overlay",
	PANEL_CHOICE_STRINGS,	"grid",
				NULL,
	PANEL_VALUE,		0,
	XV_X, x, XV_Y, y, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    ht = ArrMax(ht, (int)xv_get(item, XV_HEIGHT));

    pCtx->pPlCtx->PLhoriz_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_LABEL_STRING,	"horiz axis",
	PANEL_CHOICE_STRINGS,	"time",
				"value",
				"bin number",
				NULL,
	PANEL_VALUE,		0,
	XV_X, x, XV_Y, y, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    ht = ArrMax(ht, (int)xv_get(item, XV_HEIGHT));

    pCtx->pPlCtx->PLstat_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_LABEL_STRING,	"value status",
	PANEL_CHOICE_STRINGS,	"suppress", "show", NULL,
	PANEL_VALUE,		0,
	XV_X, x, XV_Y, y, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    ht = ArrMax(ht, (int)xv_get(item, XV_HEIGHT));

    pCtx->pPlCtx->PLscale_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_LABEL_STRING,	"auto scale",
	PANEL_CHOICE_STRINGS,	"no", "yes", NULL,
	PANEL_VALUE,		0,
	XV_X, x, XV_Y, y, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    ht = ArrMax(ht, (int)xv_get(item, XV_HEIGHT));

    window_fit(panel);
    window_fit(pCtx->pPlCtx->PLsetup_CF);
}

/*+/subr**********************************************************************
* NAME	PLwinInit - initialize the frame for plotting
*
*-*/
static void
PLwinInit(pCtx)
ARR_CTX	*pCtx;
{
    ARR_PL_CTX	*pPlCtx = pCtx->pPlCtx;
    Panel	panel;
    int	y=5, x=5, ht=0;
    static unsigned short plotIcon_bits[]={
#include "ARR_plot.icon"
    };
    unsigned long *colors;

    pPlCtx->PLwin_F = guiFrame("ARR: PLOT", 200, 200, 512, 512, NULL, NULL);
    guiIconCreate(pPlCtx->PLwin_F, plotIcon_bits);
    panel = (Panel)xv_create(pPlCtx->PLwin_F, PANEL,
	XV_X, 0, XV_Y, 0, PANEL_LAYOUT, PANEL_VERTICAL, NULL);
    xv_set(panel, WIN_WIDTH, WIN_EXTEND_TO_EDGE, NULL);
    guiButton("Dismiss", panel, &x, &y, &ht, PLwinDone_xvo, NULL,
		PCTX, pCtx, 0, NULL);
    guiMessage("Plot to:", panel, &x, &y, &ht);
    guiButton("printer", panel, &x, &y, &ht, PLwinPlot_xvo, NULL,
		PCTX, pCtx, OUT_TYPE, ARR_OT_HC);
    guiButton("EPS file", panel, &x, &y, &ht, PLwinPlot_xvo, NULL,
		PCTX, pCtx, OUT_TYPE, ARR_OT_EPS);
    guiButton("screen", panel, &x, &y, &ht, PLwinPlot_xvo, NULL,
		PCTX, pCtx, OUT_TYPE, ARR_OT_PLAIN);
    x += 10;
    guiButton("setup...", panel, &x, &y, &ht, guiCFshowPin_xvo, NULL,
		GUI_PCTX, &pCtx->guiCtx, GUI_CF, pPlCtx->PLsetup_CF);
    y += ht + 5; x = 5; ht = 0;

    pPlCtx->PLmaster.pHead = NULL;
    pPlCtx->PLmaster.pTail = NULL;
    pPlCtx->PLmaster.height = 512;
    pPlCtx->PLmaster.width = 512;
    pPlCtx->PLwinCanvas = (Canvas)xv_create(pPlCtx->PLwin_F, CANVAS,
	XV_X, x, XV_Y, y,
	CANVAS_X_PAINT_WINDOW,	TRUE,
	CANVAS_FIXED_IMAGE,	FALSE,
	WIN_RETAINED,		FALSE,
	CANVAS_AUTO_CLEAR,	TRUE,
	CANVAS_AUTO_SHRINK,	TRUE,
	CANVAS_AUTO_EXPAND,	TRUE,
	NULL);
    xv_set(canvas_paint_window(pPlCtx->PLwinCanvas),
	WIN_CONSUME_EVENTS,	WIN_MOUSE_BUTTONS,
				WIN_REPAINT,
				NULL,
	WIN_COLLAPSE_EXPOSURES,	TRUE,
	WIN_EVENT_PROC,		PLwinEvent,
	XV_KEY_DATA,		PCTX,	pCtx,
	NULL);
    xv_set(pPlCtx->PLwin_F, XV_HEIGHT, 512 + y, NULL);

    colors = (unsigned long *)xv_get(pPlCtx->PLwinCanvas, WIN_X_COLOR_INDICES);
    pPlCtx->PLwinBg = colors[
	    (unsigned long)xv_get(pPlCtx->PLwinCanvas, WIN_BACKGROUND_COLOR)];
    pPlCtx->PLwinFg = colors[
	    (unsigned long)xv_get(pPlCtx->PLwinCanvas, WIN_FOREGROUND_COLOR)];
}

/*+/subr**********************************************************************
* NAME	PLinit - initialize plot panel
*
* DESCRIPTION
*	Sets up a plot panel and associated objects, including:
*	o   PLOT button, to initiate a plot
*
*	Also initializes the actual plotting frame.
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
PLinit(pCtx, panel, pX, pY, pHt)
ARR_CTX	*pCtx;
Panel	panel;
int	*pX, *pY, *pHt;
{
    ARR_PL_CTX *pPlCtx;

    if ((pPlCtx = (ARR_PL_CTX *)GenMalloc(sizeof(ARR_PL_CTX))) == NULL)
	assertAlways(0);
    pCtx->pPlCtx = pPlCtx;
    pPlCtx->PLplotting = 0;

    PLsetupInitCF(pCtx);/* init plot setup command frame */
    PLwinInit(pCtx);	/* init plotting frame */

    guiButton("plotting...", panel, pX, pY, pHt, PLwinPlot_xvo, NULL,
			PCTX, pCtx, OUT_TYPE, ARR_OT_NULL);
}

/*+/subr**********************************************************************
* NAME	PLquit
*
*-*/
void
PLquit(pCtx)
ARR_CTX	*pCtx;
{
    xv_destroy_safe(pCtx->pPlCtx->PLsetup_CF);
    xv_destroy_safe(pCtx->pPlCtx->PLwin_F);
    GenFree(pCtx->pPlCtx);
}
