/*	@(#)ARR_PR.c	1.5 1/28/92
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	02-08-91	rac	initial version
 * .02	07-31-91	rac	installed in SCCS
 * .03	11-18-91	rac	use modified call to guiShellCmd; use
 *				modified call to sydSampleSetPrint; allow
 *				specifying number of columns and column
 *				width; consolidate setup functions onto the
 *				print display; allow specifying font size
 *				and page orientation
 * .04	12-08-91	rac	change point size in textsw; change default
 *				to 8 pt., portrait;
 * .05	01-21-92	rac	put temp files in /tmp
 * ARR_PR.c,v
 * Revision 1.2  1994/05/06 19:05:26  mrk
 * conversion from sccs to cvs
 *
 *
 * make options
 *	-DXWINDOWS	to use xview/X Window System
 */
/*+/mod***********************************************************************
* TITLE	ARR_PR.c - print routines for ARR
*
* DESCRIPTION
*
*-***************************************************************************/

#include <stdio.h>

#if defined XWINDOWS
#   include <xview/xview.h>
#   include <xview/font.h>
#   include <xview/frame.h>
#   include <xview/icon.h>
#   include <xview/notice.h>
#   include <xview/panel.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <sydDefs.h>
#include <db_access.h>

#include "ARR.h"

#define PR_FNAME_DIM 80
typedef struct arrPrCtx {
    int		colCount;	/* number of columns in printout */
    Panel_item	colCount_PT;	/* PANEL_TEXT item for number of columns */
    int		colWidth;	/* width of a column */
    Panel_item	colWidth_PT;	/* PANEL_TEXT item for column width */
    Panel_item	dir_PT;		/* PANEL_TEXT item for directory */
    char	dir[PR_FNAME_DIM];	/* directory */
    Panel_item	file_PT;	/* PANEL_TEXT item for file name */
    char	file[PR_FNAME_DIM];	/* file name */
    Panel_item	fileMode_PCS;	/* PANEL_CHOICE_STACK item for file mode */
    Panel_item	HCmode_PCS;	/* PANEL_CHOICE_STACK item for printer mode */
    Panel_item	point_PT;	/* PANEL_TEXT item for point size */
    int		point;		/* point size */
    Frame	PRwin_CF;	/* command frame */
    char	PRwinName[30];	/* ARRxxxx.pr.tmp" */
    Textsw	PRwinTextsw;	/* text subwindow to display print output */
} ARR_PR_CTX;


/*+/subr**********************************************************************
* NAME	PRwinDel
*-*/
static void PRwinDel(pCtx)
ARR_CTX	*pCtx;
{
    textsw_erase(pCtx->pPrCtx->PRwinTextsw, 0, TEXTSW_INFINITY);
    textsw_reset(pCtx->pPrCtx->PRwinTextsw, 0, TEXTSW_INFINITY);
    unlink(pCtx->pPrCtx->PRwinName);
}
/*+/subr**********************************************************************
* NAME	PRwinResize
*-*/
static void PRwinResize(window, event, arg)
Xv_Window window;
Event	*event;
Notify_arg arg;
{
    Rect	rect;
    int		y;
    ARR_CTX	*pCtx=&arrCtx;

    if (event_id(event) == WIN_RESIZE) {
	rect = *(Rect *)xv_get(pCtx->pPrCtx->PRwin_CF, XV_RECT);
	y = (int)xv_get(pCtx->pPrCtx->PRwinTextsw, XV_Y);
	xv_set(pCtx->pPrCtx->PRwinTextsw, XV_WIDTH, rect.r_width,
				XV_HEIGHT, rect.r_height-y, NULL);
    }
}

/*+/subr**********************************************************************
* NAME	PRint_xvo
*-*/
static void
PRint_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, PCTX);
    ARR_PR_CTX	*pPrCtx=pCtx->pPrCtx;
    ARR_OT_TYPE cmdType=(ARR_OT_TYPE)xv_get(item, XV_KEY_DATA, OUT_TYPE);
    SYD_SPEC	*pSspec=pCtx->pSspec;
    FILE	*f;
    char	command[200];
    int		point;
    char	HCmode[8], HCpoint[8];

    if ((int)xv_get(pPrCtx->PRwin_CF, XV_SHOW) == FALSE) {
	xv_set(pPrCtx->PRwin_CF,
			XV_SHOW, TRUE, FRAME_CMD_PUSHPIN_IN, TRUE, NULL);
    }
    if (pSspec == NULL || pSspec->chanCount <= 0) {
	if (cmdType == ARR_OT_NULL)
	    return;
	guiNotice(&pCtx->guiCtx,
			"no channels selected; use RETRIEVAL Setup...");
	return;
    }
    if (pSspec->sampleCount <= 0) {
	if (cmdType == ARR_OT_NULL)
	    return;
	guiNotice(&pCtx->guiCtx, "no samples to print; use RETRIEVE");
	return;
    }
    strcpy(pCtx->HCname, (char *)xv_get(pCtx->PR_HCname_PT, PANEL_VALUE));
    xv_set(pCtx->PL_HCname_PT, PANEL_VALUE, pCtx->HCname, NULL);
    sscanf((char *)xv_get(pPrCtx->colCount_PT, PANEL_VALUE), "%d",
			&pPrCtx->colCount);
    sscanf((char *)xv_get(pPrCtx->colWidth_PT, PANEL_VALUE), "%d",
			&pPrCtx->colWidth);
    if (pPrCtx->colWidth < 2)
        pPrCtx->colWidth = 2;
    point = 10;
    sscanf((char *)xv_get(pPrCtx->point_PT, PANEL_VALUE), "%d", &point);
    if (point < 5)
	point = 5;
    else if(point > 12)
	point = 12;
    sprintf(HCpoint, "%d", point);
    xv_set(pPrCtx->point_PT, PANEL_VALUE, HCpoint, NULL);
    if ((int)xv_get(pPrCtx->HCmode_PCS, PANEL_VALUE) == 0)
	strcpy(HCmode, "");	/* portrait mode */
    else
	strcpy(HCmode, "-r");	/* rotate to landscape mode */
    if (cmdType == ARR_OT_PLAIN || cmdType == ARR_OT_NULL) {
	PRwinDel(pCtx);
	if ((f = fopen(pPrCtx->PRwinName, "a")) == NULL) {
	    guiNoticeFile(&pCtx->guiCtx,
			"Unable to open for output", pPrCtx->PRwinName);
	    return;
	}
	sydSampleSetPrint(pSspec, f, 0, pPrCtx->colCount, pPrCtx->colWidth);
	fclose(f);
	if (point != pPrCtx->point) {
#define FIND_FONT(f) (Xv_font)xv_find(pPrCtx->PRwin_CF,FONT,FONT_NAME,f,NULL)
	    Xv_font	font;
	    char	pt[10];
	    if (point <= 5)		{font=FIND_FONT("5x8"); point=5;}
	    else if (point <= 6)	{font=FIND_FONT("6x10"); point=6;}
	    else if (point <= 7)	{font=FIND_FONT("7x13"); point=7;}
	    else if (point <= 8)	{font=FIND_FONT("8x13"); point=8;}
	    else if (point <= 9)	{font=FIND_FONT("9x15"); point=9;}
	    else if (point <= 10)	{font=FIND_FONT("10x20"); point=10;}
	    else			{font=FIND_FONT("12x24"); point=12;}
	    if (font != NULL) {
		xv_set(pPrCtx->PRwinTextsw, TEXTSW_FONT, font, NULL);
		sprintf(pt, "%d", point);
		xv_set(pPrCtx->point_PT, PANEL_VALUE, pt, NULL);
		pPrCtx->point = point;
	    }
	}
	xv_set(pPrCtx->PRwinTextsw, TEXTSW_FILE, pPrCtx->PRwinName, NULL);
    }
    else if (cmdType == ARR_OT_HC) {
	if (strlen(pCtx->HCname) == 0) {
	    guiNotice(&pCtx->guiCtx, "no printer selected");
	    return;
	}
	sprintf(pCtx->HCfName, "/tmp/ARR%d.HC.tmp", getpid());
	if ((f = fopen(pCtx->HCfName, "w")) == NULL) {
	    guiNoticeFile(&pCtx->guiCtx,
			"Unable to open for output", pCtx->HCfName);
	    return;
	}
	sydSampleSetPrint(pSspec, f, 1, pPrCtx->colCount, pPrCtx->colWidth);
	fclose(f);
	sprintf(command, "enscript -fCourier%d -q -B -P%s %s %s",
		point, pCtx->HCname, HCmode, pCtx->HCfName);
	guiShellCmd(pCtx->guiCtx, command, lprClient, NULL, NULL);
    }
    else if (cmdType == ARR_OT_DO) {
	char	path[2*PR_FNAME_DIM];
	int	modeCode;

	strcpy(pPrCtx->dir, (char *)xv_get(pPrCtx->dir_PT, PANEL_VALUE));
	strcpy(pPrCtx->file, (char *)xv_get(pPrCtx->file_PT, PANEL_VALUE));
	modeCode = (int)xv_get(pPrCtx->fileMode_PCS, PANEL_VALUE);
	if (strlen(pPrCtx->file) == 0) {
	    guiNotice(&pCtx->guiCtx, "no file selected");
	    return;
	}
	sprintf(path, "%s/%s", pPrCtx->dir, pPrCtx->file);
        if (modeCode == 0) {	/* append mode */
	    if ((f = fopen(path, "a")) == NULL) {
		guiNoticeFile(&pCtx->guiCtx, "Unable to open for output", path);
		return;
	    }
	    fprintf(f, "\f");	/* put in a form feed */
	}
	else {			/* over-write mode */
	    if ((f = fopen(path, "w")) == NULL) {
		guiNoticeFile(&pCtx->guiCtx, "Unable to open for output", path);
		return;
	    }
	}
	sydSampleSetPrint(pSspec, f, 0, pPrCtx->colCount, pPrCtx->colWidth);
	fclose(f);
    }
}

/*+/subr**********************************************************************
* NAME	PRwinInitCF - initialize the print window command frame
*
*-*/
static void PRwinInitCF(pCtx)
ARR_CTX	*pCtx;
{   ARR_PR_CTX	*pPrCtx=pCtx->pPrCtx;
    FILE	*f;
    Panel	panel;
    Xv_opaque	item;
    int		y=5, x=5, ht=0;

    sprintf(pPrCtx->PRwinName, "/tmp/ARR%d.pr.tmp", getpid());
    if ((f = fopen(pPrCtx->PRwinName, "w")) == NULL) {
	guiNoticeFile(&pCtx->guiCtx,
	    "Unable to open temp file for printed output", pPrCtx->PRwinName);
	exit(1);
    }
    fclose(f);

/*-----------------------------------------------------------------------------
* Dismiss  Print to:  printer  file  screen
* Directory: _______________________
* File: _______________________  V append
*                                  over-write
* Printer: _____  # col: ___  col width: ___  point size: __  V portrait
*                                                               landscape
*----------------------------------------------------------------------------*/

    pPrCtx->PRwin_CF =
	guiFrameCmd("ARR: PRINT", pCtx->guiCtx.baseFrame, &panel, PRwinResize);
    guiButton("Dismiss", panel, &x, &y, &ht, guiCFdismiss_xvo, NULL,
				    GUI_CF, pPrCtx->PRwin_CF, 0, NULL);
    guiMessage("Print to:", panel, &x, &y, &ht);
    guiButton("printer", panel, &x, &y, &ht, PRint_xvo, NULL,
		PCTX, pCtx, OUT_TYPE, ARR_OT_HC);
    guiButton("file", panel, &x, &y, &ht, PRint_xvo, NULL,
		PCTX, pCtx, OUT_TYPE, ARR_OT_DO);
    guiButton("screen", panel, &x, &y, &ht, PRint_xvo, NULL,
		PCTX, pCtx, OUT_TYPE, ARR_OT_PLAIN);
    y += ht + 5; x = 5; ht = 0;

    pPrCtx->dir_PT = guiTextField("Directory:", panel, &x,&y,&ht, NULL,
		PR_FNAME_DIM-1, 40, 0, NULL, 0, NULL);
    strcpy(pPrCtx->dir, "./data");
    xv_set(pPrCtx->dir_PT, PANEL_VALUE, pPrCtx->dir, NULL);
    y += ht + 5; x = 5; ht = 0;

    pPrCtx->file_PT = guiTextField("File:", panel, &x,&y,&ht, NULL,
		PR_FNAME_DIM-1, 40, 0, NULL, 0, NULL);
    strcpy(pPrCtx->file, "");
    xv_set(pPrCtx->file_PT, PANEL_VALUE, pPrCtx->file, NULL);
    pPrCtx->fileMode_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_CHOICE_STRINGS,	"append","over-write",NULL,
	PANEL_VALUE,	1,
	XV_X, x, XV_Y, y-5, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;

    y += ht + 5; x = 5; ht = 0;

    pCtx->PR_HCname_PT = guiTextField("Printer:", panel, &x,&y,&ht,
		NULL, 19, 8, 0, NULL, 0, NULL);
    xv_set(pCtx->PR_HCname_PT, PANEL_VALUE, pCtx->dfltPrinter, NULL);
    pPrCtx->colCount_PT = guiTextField("# col:", panel, &x,&y,&ht,
		NULL, 3, 3, 0, NULL, 0, NULL);
    pPrCtx->colCount = 8;
    xv_set(pPrCtx->colCount_PT, PANEL_VALUE, "8", NULL);
    pPrCtx->colWidth_PT = guiTextField("col width:", panel, &x,&y,&ht,
		NULL, 2, 2, 0, NULL, 0, NULL);
    pPrCtx->colWidth = 10;
    xv_set(pPrCtx->colWidth_PT, PANEL_VALUE, "10", NULL);
    pPrCtx->point_PT = guiTextField("point size:", panel, &x,&y,&ht,
		NULL, 2, 2, 0, NULL, 0, NULL);
    xv_set(pPrCtx->point_PT, PANEL_VALUE, "8", NULL);
    pPrCtx->point = 8;
    pPrCtx->HCmode_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_CHOICE_STRINGS,	"portrait","landscape",NULL,
	PANEL_VALUE,	0,
	XV_X, x, XV_Y, y-5, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    y += ht + 5; x = 5; ht = 0;

    window_fit_height(panel);
    pPrCtx->PRwinTextsw =
		guiBrowser(pPrCtx->PRwin_CF, panel, 120, 40, pPrCtx->PRwinName);
    {
	Xv_font	font;
	font = (Xv_font)xv_find(pPrCtx->PRwin_CF,FONT,FONT_NAME,"8x13",NULL);
	if (font != NULL)
	    xv_set(pPrCtx->PRwinTextsw, TEXTSW_FONT, font, NULL);
    }
    window_fit(pPrCtx->PRwin_CF);
}

/*+/subr**********************************************************************
* NAME	PRinit - initialize print panel
*
* DESCRIPTION
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
PRinit(pCtx, panel, pX, pY, pHt)
ARR_CTX	*pCtx;
Panel	panel;
int	*pX, *pY, *pHt;
{
    ARR_PR_CTX *pPrCtx;

    if ((pPrCtx = (ARR_PR_CTX *)GenMalloc(sizeof(ARR_PR_CTX))) == NULL)
	assertAlways(0);
    pCtx->pPrCtx = pPrCtx;
    PRwinInitCF(pCtx);

    guiButton("printing...", panel, pX, pY, pHt, PRint_xvo, NULL,
			PCTX, pCtx, OUT_TYPE, ARR_OT_NULL);
}
void
PRquit(pCtx)
ARR_CTX	*pCtx;
{
    unlink(pCtx->pPrCtx->PRwinName);
    textsw_reset(pCtx->pPrCtx->PRwinTextsw, 0, TEXTSW_INFINITY);
    xv_destroy_safe(pCtx->pPrCtx->PRwin_CF);
    GenFree(pCtx->pPrCtx);
}
