#ifndef INC_ARR_RET_h
#define INC_ARR_RET_h
/*	@(#)ARR_RET.h	1.1 8/24/92
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991-92, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 *  .00 02-08-91 rac	initial version
 *  .01 05-05-92 rac	cloned from ARR_RET.c
 * ARR_RET.h,v
 * Revision 1.2  1994/05/06 19:05:31  mrk
 * conversion from sccs to cvs
 *
 *
 */
/*+/mod***********************************************************************
* TITLE	ARR_RET.h - definitions for ARR_RET, the retrieval software
*
* DESCRIPTION
*
*-***************************************************************************/

typedef enum {
    ARR_RET_LAST=0,	/* following last retrieval */
    ARR_RET_AT,		/* at date/time */
    ARR_RET_BOF,	/* beginning of file */
} ARR_RET_FROM;
typedef enum {
    ARR_RET_NSAMP=0,	/* number of samples */
    ARR_RET_THROUGH,	/* through date/time */
    ARR_RET_PERIOD,	/* period */
} ARR_RET_THRU;
typedef enum {
    ARR_RET_BCF=0,	/* use `by channel' file */
    ARR_RET_SSF,	/* use `sample set' file */
    ARR_RET_CA,		/* use Channel Access */
} ARR_RET_USE;
typedef enum {
    ARR_RET_SYNC=0,	/* add channel as synchronous */
    ARR_RET_FILL,	/* add channel as filled */
    ARR_RET_DEL		/* delete channel */
} ARR_RET_CH;

typedef struct arrRetCtx {
    ARR_CTX	*pCtx;		/* pointer to ARR context */
    char	bcfPath[GUI_TDIM*2];/* path to `by channel' file */
    char	bcfDir[GUI_TDIM];/* directory part of path */
    char	bcfFile[GUI_TDIM];/* file part of path */
    Panel_item	bcfStatus_PM;	/* PANEL_MESSAGE item for file being used */
    GUI_EDIT	*pConfigEdit;	/* editor for configuration files */
    char	configDir[GUI_TDIM];/*temp file for "show config" */
    char	configPath[GUI_TDIM*2];/* temp file for "show config" */
    char	configFile[GUI_TDIM];/* temp file for "show config" */
    Panel_item	config_PM;	/* PANEL_MESSAGE item for config window */
    Panel_item	RETaccept_PCB;	/* PANEL_CHECK_BOX for "accept samples if" */
    Panel_item	RETaccept_PT;	/* PANEL_TEXT for "accept samples if" */
    Panel_item	RETdeadRst_PCB;	/* PANEL_CHECK_BOX for "dead-time restart" */
    Panel_item	RETdeadRst_PT;	/* PANEL_TEXT for "dead-time restart" */
    double	RETdeadRstSec;	/* dead-time restart interval, in sec */
    int		RETdeadRst;	/* do dead-time restart */
    TS_STAMP	RETdeadRefTs;	/* dead-time restart reference time stamp */
    Panel_item	RETfrom_PCB;	/* PANEL_CHECK_BOX for "retrieve from" */
    Panel_item	RETfromStamp_PT;/* PANEL_TEXT item for "from stamp" */
    TS_STAMP	RETfromTS;	/* time stamp for "retrieve from" */
    ARR_RET_FROM RETfrom;	/* code for "retrieve from" */
    Panel_item	RETpartial_PCB;	/* PANEL_CHECK_BOX for "ignore partial" */
    Panel_item	RETroundTs_PCB;	/* PANEL_CHECK_BOX for "round time stamps" */
    Panel_item	RETroundTs_PT;	/* PANEL_TEXT for "round time stamps" */
    Panel_item	RETthru_PCB;	/* PANEL_CHECK_BOX for "retrieve thru" */
    Panel_item	RETthruNsamp_PT;/* PANEL_TEXT item for number of samples */
    Panel_item	RETthruStamp_PT;/* PANEL_TEXT item for "thru stamp" */
    TS_STAMP	RETthruTS;	/* time stamp for "retrieve thru" */
    Panel_item	RETthruPer_PT;	/* PANEL_TEXT item for sample period */
    TS_STAMP	RETthruPerTS;	/* time stamp for sample period */
    ARR_RET_THRU RETthru;	/* code for "retrieve through" */
    Panel_item	RETstatus_PM;	/* PANEL_MESSAGE item for retrieval status */
    Frame	RETsetup_CF;	/* command frame for setup */
    char	ssfPath[120];	/* path to `sample set' file */
    char	ssfDir[60];	/* directory part of path */
    char	ssfFile[60];	/* file part of path */
    Panel_item	ssfStatus_PM;	/* PANEL_MESSAGE item for file being used */
    Panel_item	RETuse_PCB;	/* PANEL_CHECK_BOX for "retrieve from" */
    char	RETwinName[30];	/* "ARRxxxx.inf.tmp" */
    Textsw	RETwinTextsw;	/* text subwindow to display file info */
    Panel_item	CHname_PT;	/* PANEL_TEXT item for channel name */
    Panel_item	CHname_PM;	/* PANEL_MESSAGE item for channel name */
    short	RETcaSampling;	/* store sample in set if flag is non-zero */
} ARR_RET_CTX;

/*-----------------------------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
int RETquitCheck();
void RETquit();

#endif
