/*	@(#)ARR_cmd.c	1.5 2/23/93
 *	Author:	Roger A. Cole
 *	Date:	09-17-90
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991-93, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 *  .01 09-17-90 rac	initial version
 *  .02 07-31-91 rac	installed in SCCS
 *  .03 08-21-91 rac	use new sydXxx routines
 *  .04 09-11-91 joh	updated for v5 vxWorks
 *  .05 09-23-91 rac	specifically include Xlib.h
 *  .06 09-30-91 rac	use new calling sequence for sydChanOpen and
 *			arCFOpen
 *  .07 11-27-91 rac	use new calling sequence for sydSamplePrint
 *			and sydSampleSetPrint;
 *			support ignoring partial samples, rounding
 *			time stamps, stopping filling after a
 *			deadtime; allow specifying number and width
 *			of print columns
 *  .08 12-08-91 rac	use new calling seq for cmdRead; don't use LWP
 *  .09 07-29-92 rac	cloned from ARR_kbd.c
 *  .11 08-31-92 rac	add `l' prefix for plot type
 *  .12 01-25-93 rac	handle exporting, printing, and plotting only means
 * ARR_cmd.c,v
 * Revision 1.2  1994/05/06 19:05:34  mrk
 * conversion from sccs to cvs
 *
 *
 * make options
 *	-DNDEBUG	don't compile assert() checking
 *      -DDEBUG         compile various debug code, including checks on
 *                      malloc'd memory
 */
/*+/mod***********************************************************************
* TITLE	ARR_cmd - archive data retrieval program
*
* DESCRIPTION
*
* WISH LIST
* o	selection by time stamp range or other conditions as for the
*	AR SAMPLE: command
* o	read an archive request to determine channel selection for retrieval
*
* BUGS
* o	retrievals are limited to a single file
*
*-***************************************************************************/
#include <stdio.h>
#include <sys/file.h>        /* for O_RDWR and O_RDONLY definitions */
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include <setjmp.h>

#include <X11/Xlib.h>
#include <genDefs.h>
#include <cmdDefs.h>
#include <tsDefs.h>
#include <sydDefs.h>
#include <pprPlotDefs.h>
#include <sydPlotDefs.h>
#include <alarm.h>
#include <arCS.h>
#include <cadef.h>
#include <db_access.h>


/*/subhead ARR_CTX-------------------------------------------------------
* ARR_CTX - context information
*
*	An ARR descriptor is the `master handle' which is used for
*	handling business.
*----------------------------------------------------------------------------*/

typedef struct ARRCtx {
    CX_CMD	cmdCtx;
    CX_CMD	*pCmdCtx;
    jmp_buf	sigEnv;
    char	bychanFile[80];		/* by channel data file name */
    char	sampleFile[80];		/* sample set data file name */
    SYD_SPEC	*pSspec;		/* synchronous set spec */
    int		useStats;		/* 1 for work with mean and stdDev */
    int		useMeans;		/* 1 for work with mean */
    char	printer[80];		/* printer name */
    char	printerFile[80];	/* print temporary file */
    char	plotEPSfile[120];	/* file for EPS plots */
    SYD_PLAX	plotAxis;		/* type of plot */
    int		linePlot;		/* 1 for line plot */
    int		markPlot;		/* 1 for mark plot */
    int		pointPlot;		/* 1 for point plot */
    int		showStat;		/* 1 to show alarm status */
    int		errBar;			/* 1 to show error bars */
    SYD_PL_MSTR	master;			/* sync data plot master */
    int		plotInit;		/* 1 plot window init needed */
    int		accept;			/* 1 if restricting acceptance */
    char	acceptText[120];	/* text for accept condition */
    int		deadRst;		/* 1 for reset fill after deadtime */
    TS_STAMP	deadRefTs;		/* reference stamp for deadtime */
    double	deadRstSec;		/* deadtime restart interval, in sec */
    int		discardPartial;		/* 1 to discard partial samples */
    int		ncol;			/* number of print columns */
    int		colWidth;		/* width of print columns, in char */
    int		stop;			/* 1 says `stop operations' */
} ARR_CTX;

/*-----------------------------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
int ARR();
void arrCmdCrunch();
long arrInitAtStartup();
long arr();
void arrSigHandler();
void arrSigInit();

/*-----------------------------------------------------------------------------
* global definitions
*----------------------------------------------------------------------------*/
static HELP_TOPIC helpExport;	/* help for "export" command */
static HELP_TOPIC helpOut;	/* help for the various output commands */
static HELP_TOPIC helpPlot;	/* help for "plot" command */
static HELP_TOPIC helpPos;	/* help for "position" command */
static ARR_CTX	*pglArrCtx;

#define P_CMD pCmdCtx->pCommand
#define P_FLD pCmdCtx->pField
#define FLD_LEN pCmdCtx->fldLen
#define P_LINE pCmdCtx->pLine
#define DELIM pCmdCtx->delim

typedef enum {
    ARR_P_PLAIN,	/* ordinary print or plot command */
    ARR_P_HC,		/* printHC and plotHC */
    ARR_P_EPS		/* plotEPS */
} ARR_P_TYPE;

/*+/subr**********************************************************************
* NAME	main
*
*-*/
main()
{
    long	stat;		/* status return from calls */
    ARR_CTX	arrCtx, *pCtx=&arrCtx;

    arrInitAtStartup(pCtx);
    genSigInit(arrSigHandler);

    if (setjmp(pCtx->sigEnv) != 0)
	goto arrWrapup;

/*----------------------------------------------------------------------------
*    "processing loop"
*---------------------------------------------------------------------------*/
    while (!pCtx->stop) {
	if (cmdRead(&pCtx->pCmdCtx) != NULL)
	    arrCmdCrunch(pCtx);

	usleep(100000);
    }

arrWrapup:
    if (pCtx->pSspec != NULL) {
	stat = sydClose(pCtx->pSspec);
	assert(stat == S_syd_OK);
    }

    while (pCtx->pCmdCtx->pPrev != NULL)
	cmdCloseContext(&pCtx->pCmdCtx);

    if (pCtx->pCmdCtx->dataOut != stdout)
	fclose(pCtx->pCmdCtx->dataOut);

    return 0;
}

/*+/subr**********************************************************************
* NAME	arrSig - signal handling and initialization
*
* DESCRIPTION
*	These routines set up for the signals to be caught for arr
*	and handle the signals when they occur.
*
* RETURNS
*	void
*
* BUGS
* o	not all signals are caught
* o	it's not clear how useful it is to catch the signals which originate
*	from the keyboard
*
*-*/
void
arrSigHandler(signo)
int	signo;
{
    printf("entered arrSigHandler for signal:%d\n", signo);
    signal(signo, SIG_DFL);
    longjmp(pglArrCtx->sigEnv, 1);
}

/*+/subr**********************************************************************
* NAME	arrCmdCrunch - process a command line
*
* DESCRIPTION
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
arrCmdCrunch(pArrCtx)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec;
    long	stat;

    pSspec = pArrCtx->pSspec;

    if (strcmp(P_CMD,				"assert0") == 0) {
/*-----------------------------------------------------------------------------
*	under SunOS, assertAlways() generates SIGABRT
*----------------------------------------------------------------------------*/
	assertAlways(0);
    }
    else if (strcmp(P_CMD,			"quit") == 0) {
	pArrCtx->stop = 1;
	return;
    }
    else if (strcmp(P_CMD,			"trap") == 0) {
	int	*j;
	j = (int *)(-1);
	j = (int *)(*j);
	return;
    }
    else if (strcmp(P_CMD,			"dataOut") == 0) {
	if (pCmdCtx->dataOut != stdout)
	    fclose (pCmdCtx->dataOut);
	if (nextNonSpaceField(&P_LINE, &P_FLD, &DELIM) < 1)
	    pCmdCtx->dataOut = stdout;
	else {
	    pCmdCtx->dataOut = fopen(pCmdCtx->pField, "a");
	    if (pCmdCtx->dataOut == NULL) {
		(void)printf("couldn't open %s\n", pCmdCtx->pField);
		pCmdCtx->dataOut = stdout;
	    }
	}
	return;
    }
    else if (strcmp(P_CMD,			"source") == 0) {
	cmdSource(&pArrCtx->pCmdCtx);
	return;
    }
    else if (strcmp(P_CMD,			"byChannel") == 0) {
	if (pArrCtx->pSspec != NULL) {
	    stat = sydClose(pArrCtx->pSspec);
	    assertAlways(stat == S_syd_OK);
	    pArrCtx->pSspec = NULL;
	}
	
	if (nextNonSpaceField(&P_LINE, &P_FLD, &DELIM)>1) {
	    if (strlen(P_FLD) >= 80) {
		(void)printf("file name (or path) too long\n");
		return;
	    }
	    (void)strcpy(pArrCtx->bychanFile, P_FLD);
	}
	else
	    (void)strcpy(pArrCtx->bychanFile, "arChan/arChanData.bin");
	stat = sydOpenCF(&pArrCtx->pSspec, pArrCtx->bychanFile);
	if (stat != S_syd_OK) {
	    (void)printf("initialization failed\n");
	    return;
	}
	return;
    }
    else if (strcmp(P_CMD,			"sampleSet") == 0) {
	if (pArrCtx->pSspec != NULL) {
	    stat = sydClose(pArrCtx->pSspec);
	    assertAlways(stat == S_syd_OK);
	    pArrCtx->pSspec = NULL;
	}
	if (nextNonSpaceField(&P_LINE, &P_FLD, &DELIM) <= 1) {
	    (void)printf("file name (or path) must be specified\n");
	    return;
	}
	if (strlen(P_FLD) >= 80) {
	    (void)printf("file name (or path) too long\n");
	    return;
	}
	(void)strcpy(pArrCtx->sampleFile, P_FLD);
	stat = sydOpenSSF(&pArrCtx->pSspec, pArrCtx->sampleFile);
	if (stat != S_syd_OK) {
	    (void)printf("initialization failed\n");
	    return;
	}
	return;
    }
    if (pSspec == NULL && strcmp(P_CMD, "help") != 0) {
	(void)printf( "no file selected yet; use `byChannel' or `sampleSet'\n");
	return;
    }
    if (strcmp(P_CMD,				"channel") == 0)
	ARR_channel(pArrCtx);
    else if (strcmp(P_CMD,			"delete") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_delete(pArrCtx);
    }
    else if (strcmp(P_CMD,			"export") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_export(pArrCtx);
    }
    else if (strcmp(P_CMD,			"fileInfo") == 0 ||
    	strcmp(P_CMD, "fileinfo") == 0) {
	(void)sydFileInfo(pSspec, stdout);
    }
    else if (strcmp(P_CMD,			"plot") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_plot(pArrCtx, ARR_P_PLAIN);
    }
    else if (strcmp(P_CMD,			"plotEPS") == 0) {
	if (pSspec->chanCount <= 0 || pSspec->sampleCount <= 0) {
	    (void)printf("no data to plot\n");
	    return;
	}
	ARR_plot(pArrCtx, ARR_P_EPS);
    }
    else if (strcmp(P_CMD,			"plotHC") == 0) {
	if (pSspec->chanCount <= 0 || pSspec->sampleCount <= 0) {
	    (void)printf("no data to plot\n");
	    return;
	}
	ARR_plot(pArrCtx, ARR_P_HC);
    }
    else if (strcmp(P_CMD,			"position") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_position(pArrCtx);
    }
    else if (strcmp(P_CMD,			"print") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_print(pArrCtx);
    }
    else if (strcmp(P_CMD,			"printHC") == 0) {
	if (pSspec->chanCount <= 0 || pSspec->sampleCount <= 0) {
	    (void)printf("no data to print\n");
	    return;
	}
	ARR_printHC(pArrCtx);
    }
    else if (strcmp(P_CMD,			"resetFilled") == 0) {
	double	dead;
	if (pSspec->chanCount <= 0) goto noChanErr;
	if (nextFltFieldAsDbl(&P_LINE, &dead, &DELIM) <= 1) {
	    (void)printf("deadtime interval must be specified\n");
	    return;
	}
	if (dead <= 0.) {
	    (void)printf("deadtime must be > 0.\n");
	    return;
	}
	pArrCtx->deadRstSec = dead;
	pArrCtx->deadRst = 1;
    }
    else if (strcmp(P_CMD,			"accept") == 0) {
	sydTestClose(pSspec);
	strcpy(pArrCtx->acceptText, P_LINE);
	if (strncmp(pArrCtx->acceptText, "all", 3) == 0) {
	    pArrCtx->accept = 0;
	    sydTestClose(pSspec);
	}
	else {
	    stat = sydTestAddFromText(pSspec, pArrCtx->acceptText);
	    if (stat != S_syd_OK) {
		(void)printf("illegal `accept if' condition\n");
		sydTestClose(pSspec);
		pArrCtx->accept = 0;
	    }
	    else
		pArrCtx->accept = 1;
	}
    }
    else if (strcmp(P_CMD,			"discardPartial") == 0) {
	pArrCtx->discardPartial = 1;
    }
    else if (strcmp(P_CMD,			"rewind") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	sydPosition(pSspec, NULL);
    }
    else if (strcmp(P_CMD,			"roundStamps") == 0) {
	long	round;
	if (pSspec->chanCount <= 0) goto noChanErr;
	if (nextIntFieldAsLong(&P_LINE, &round, &DELIM) <= 1) {
	    (void)printf("rounding interval must be specified\n");
	    return;
	}
	if (round < 0 || round > 2000) {
	    (void)printf("0 < rounding interval <= 2000 violated\n");
	    return;
	}
	pSspec->roundNsec = round * 1000000;
    }
    else if (strcmp(P_CMD,			"workWith") == 0) {
	if (nextNonSpaceField(&P_LINE, &P_FLD, &DELIM) < 1) {
	    (void)printf("`workWith' must have a parameter\n");
	    return;
	}
	else if (strcmp(P_FLD, "dataVal") == 0)
	    pArrCtx->useStats = pArrCtx->useMeans = 0;
	else if (strcmp(P_FLD, "mean") == 0)
	    pArrCtx->useStats = 0, pArrCtx->useMeans = 1;
	else if (strcmp(P_FLD, "meanAndStdDev") == 0)
	    pArrCtx->useStats = 1, pArrCtx->useMeans = 0;
	else
	    (void)printf("illegal `workWith' parameter\n");
	return;
    }
    else {
/*----------------------------------------------------------------------------
* help (or illegal command)
*----------------------------------------------------------------------------*/
	(void)nextNonSpaceField(&P_LINE, &P_FLD, &DELIM);
	helpIllegalCommand(stdout, &pCmdCtx->helpList, P_CMD, P_FLD);
    }

    return;
noChanErr:
    (void)printf("no channels selected\n");
}

/*+/subr**********************************************************************
* NAME	arrInitAtStartup - initialization for ARR
*
* DESCRIPTION
*	Perform several initialization duties:
*	o   initialize the channel data retrieval set
*	o   initialize the command context block
*	o   initialize the help information
*
* RETURNS
*	OK
*
* BUGS
* o	text
*-*/
long
arrInitAtStartup(pArrCtx)
ARR_CTX	*pArrCtx;
{
    long	stat;
    CX_CMD	*pCmdCtx = &pArrCtx->cmdCtx;

    pArrCtx->pCmdCtx = pCmdCtx;
    pArrCtx->stop = 0;
    pArrCtx->accept = 0;
    pArrCtx->deadRst = 0;
    pArrCtx->discardPartial = 0;
    pArrCtx->useStats = pArrCtx->useMeans = 0;

    pArrCtx->sampleFile[0] = '\0';
    pArrCtx->pSspec = NULL;

    pArrCtx->printer[0] = '\0';
    pArrCtx->printerFile[0] = '\0';
    pArrCtx->ncol = 4;
    pArrCtx->colWidth = 12;

    pArrCtx->plotEPSfile[0] = '\0';
    pArrCtx->plotAxis = SYD_PLAX_TY;
    pArrCtx->linePlot = 1;
    pArrCtx->pointPlot = 0;
    pArrCtx->markPlot = 0;
    pArrCtx->showStat = 0;
    pArrCtx->errBar = 0;
    pArrCtx->plotInit = 1;

    cmdInitContext(pCmdCtx, " ARR_cmd: ");

/*-----------------------------------------------------------------------------
* help information initialization
*----------------------------------------------------------------------------*/
    helpInit(&pCmdCtx->helpList);
/*-----------------------------------------------------------------------------
* help info--generic commands
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCmdCtx->helpList, &pCmdCtx->helpCmds, "commands", "\n\
Generic commands are:\n\
   dataOut    [filePath]     (default is to normal output)\n\
   help       [topic]\n\
   quit (or ^D)\n\
   source     filePath\n\
");
/*-----------------------------------------------------------------------------
*	export command
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCmdCtx->helpList, &helpExport, "export", "\n\
The `export' command serves the dual function of acquiring some samples\n\
and then \"exporting\" them into a desired format.  If `nSamples' is\n\
omitted, then the most recently acquired set of samples is exported.\n\
\n\
The dataOut command can be used to direct the output from the export command\n\
to a specified file.\n\
\n\
    export,type [nSamples]\n\
\n\
If `nSamples' is omitted, then the most recently acquired set of samples\n\
is exported.\n\
\n\
Valid export types are:\n\
    ss		format for reading into a spreadsheet\n\
    ssStat	same as \"ss\", but with a status code preceding each value\n\
                (use \"help output\" to get a list of status codes)\n\
");
/*-----------------------------------------------------------------------------
* help info--ARR-specific commands
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCmdCtx->helpList, &pCmdCtx->helpCmdsSpec, "commands", "\n\
ARR-specific commands are:\n\
  accept         all  or  if chanName op value\n\
  byChannel      [filePath]\n\
  channel        [F] chanName [[F] chanName ...]\n\
  delete         all  or  chanName [chanName ...]\n\
  discardPartial\n\
 *export,type    [nSamples]    type: {ss, ssStat}\n\
  fileInfo\n\
  plot[,type]    [nSamples] type: [l,p,m][t,x]{y,yy}[s][e]  e.g. ty or pxyy\n\
  plotEPS[,type] [filePath]\n\
  plotHC[,type]  [printerName]\n\
  position       timeStamp\n\
 *print          [[nSamples][,ncol,colWidth]]\n\
  printHC        [[printerName][,ncol,colWidth]]\n\
  resetFilled    nSeconds\n\
  rewind\n\
  roundStamps    nMilliSec\n\
  sampleSet      filePath\n\
  workWith       dataValues  or  mean  or  meanAndStdDev\n\
\n\
Output from commands flagged with * can be routed to a file by using the\n\
\"dataOut filePath\" command.  The present contents of the file are\n\
preserved, with new output being written at the end.\n\
");
/*-----------------------------------------------------------------------------
*	retrieval options
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCmdCtx->helpList, &helpOut, "retrievalOptions", "\n\
Several options can be invoked for retrieving data, using special commands.\n\
\n\
    accept if op value		causes accepting only those samples which\n\
			satisfy the test.  `op' can be one of:\n\
			==  !=  <  <=  >  >=\n\
			The `value' must be compatible with the type of the\n\
			channel.  For DBF_ENUM channels, `value' must be\n\
			one of the state strings.\n\
    accept all			causes all samples to be accepted\n\
    discardPartial	omit partial samples from the retrieval\n\
    resetFilled  seconds	causes the data stream for `filled' channels\n\
			to be reset (i.e., the old value is discarded) if\n\
			the interval following a sample exceeds the specified\n\
			time\n\
    roundStamps  milliSec	causes time stamps to be rounded to the\n\
			nearest `milliSec' boundary\n\
");
/*-----------------------------------------------------------------------------
*	output options
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCmdCtx->helpList, &helpOut, "output", "\n\
Several possibilities exist for output from ARR:\n\
\n\
    dataOut        [filePath]\n\
   *export,type    [nSamples],option   (use help export for more info)\n\
    plot[,type]    [nSamples]          (use help plot for more info\n\
    plotEPS[,type] [filePath]\n\
    plotHC[,type]  [printerName]\n\
   *print          [[nSamples][,ncol,colWidth]]\n\
    printHC        [[printerName][,ncol,colWidth]]\n\
    workWith       dataValues  or  mean  or  meanAndStdDev\n\
\n\
The dataOut command can be used to direct the output from `export' and\n\
`print' to a file.\n\
\n\
The workWith command can be used to control whether the output commands\n\
work with the actual data values or with the mean and standard deviation\n\
of snapshots.\n\
\n\
For text output, a one or two character status code is used.  For the\n\
print commands, this status code follows the value; for the export\n\
command, it precedes the value.  The first character will be one of:\n\
    O  value is OK (i.e., it has a valid time stamp)\n\
    R  value is OK, but it restarts the \"run\" of data for the channel.\n\
       This happens, for example, when an archive request is inactivated\n\
       and then re-activated.\n\
    F  value is \"filled\" from one with an earlier time stamp\n\
    M  value is missing (i.e., no value for channel with this time stamp)\n\
The second character conveys additional information about the value.  If\n\
the second character is blank, then the value is normal.\n\
    E  end of file for channel\n\
    N  no data for channel\n\
    H  value violates high alarm limit\n\
    h  value violates high warning limit\n\
    l  value violates low warning limit\n\
    L  value violates low alarm limit\n\
    R  value has a control system read alarm\n\
    M  value has some other major alarm\n\
    m  vlaue has some other minor alarm\n\
\n\
The `print' and `plot' commands serve the dual function of acquiring some\n\
samples and then displaying them on the screen, either as text or as a\n\
plot.  If `nSamples' is omitted, then the most recently acquired set of\n\
samples is displayed.\n\
\n\
The `printHC' and `plotHC' commands prepare hard copies of the most recently\n\
acquired set of samples, in text and plot forms respectively.  (These\n\
commands can not be used to acquire samples.)  These commands write the\n\
output to a printer.  (For `plotHC', the printer must be a PostScript\n\
printer.)  Once a printer has been specified, it need not be specified\n\
again, unless a change is desired.\n\
\n\
For the `print' and `printHC' commands, `ncol' and `colWidth' control\n\
formatting.  They specify the number of text columns and the width of a\n\
column in characters.\n\
");
/*-----------------------------------------------------------------------------
*	plot command
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCmdCtx->helpList, &helpPlot, "plot", "\n\
The `plot', `plotEPS', and `plotHC' commands display samples in graphical\n\
form, either on the screen or to a PostScript file or printer.  The `plot'\n\
command will also optionally acquire a new group of samples prior to\n\
plotting.\n\
\n\
  plot[,type]    [nSamples] type: [l,p,m][t,x]{y,yy}[s][e]  e.g. ty or pxyy\n\
  plotEPS[,type] [filePath]\n\
  plotHC[,type]  [printerName]\n\
\n\
If the `,type' option isn't specified, the most recently specified plot\n\
type is used.  The default when ARR starts is `,ty'.\n\
\n\
Valid plot types are:\n\
  y	`bin number' vs y, primarily for waveforms; each channel has its\n\
	own grid\n\
  yy    `bin number' vs y, primarily for waveforms; all channels share the\n\
	same grid\n\
  ty    time vs y; each channel has its own grid\n\
  tyy   time vs y; all channels share the same grid\n\
  xy    x channel vs y channel; each y channel has its own grid.  (The\n\
        first channel is treated as the x channel; the second and\n\
        following channels are treated as y channels.)\n\
  xyy   x channel vs y channel; all y channels share the same grid.\n\
  l     may precede any of the plot types to produce a line plot.\n\
        For example, lty and lmxyy are valid types.  (`l' is the default.)\n\
  p     may precede any of the plot types to produce a point plot rather\n\
        than a line plot.  For example, pty and pxyy are valid types.\n\
  m     may precede any of the plot types to produce a point plot rather\n\
        than a line plot, using a plot mark (rather than a small dot) at\n\
	each point.  For example, mty and mxyy are valid types.\n\
  s     may follow any of the plot types to show the status of points\n\
        on the plot.  For example, tys and pxyys are valid types.\n\
  e     may follow any of the plot types to draw error bars for snapshots\n\
	when `workWith meanAndStdDev' `workWith mean' is being used.\n\
\n\
ARR `hangs' while a plot window is up; the plot window can be exited by\n\
selecting \"quit\" on the plot window title bar, or by clicking the right\n\
mouse button within the plot window.  ARR remembers (until it exits) the\n\
size and position of the plot window; either size and/or position may be\n\
freely changed.  The size is used by the `plotHC' command in preparing\n\
the hard copy of the plot.\n\
\n\
The `plotEPS' command stores a plot of the most recently acquired set of\n\
samples in the indicated file, erasing its previous contents, if any.\n\
Once a file has been specified, it need not be specified again, unless a\n\
change is desired.  (This command can't be used to acquire samples.)  The\n\
file is in Encapsulated PostScript form, which allows it easily to be\n\
imported into many text processors.  For example, to get an EPS file named\n\
\"plot1\" on your Interleaf desktop, use:\n\
\n\
	plotEPS /home/yourServer/yourName/desktop/plot1.eps\n\
\n\
The `plotHC' command prints a plot of the most recently acquired set of\n\
samples.  (This command can't be used to acquire samples.)  The printer\n\
must be a PostScript printer.  Once a printer has been specified, it need\n\
not be specified again, unless a change is desired.\n\
");
/*-----------------------------------------------------------------------------
*	position command
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCmdCtx->helpList, &helpPos, "position", "\n\
Positioning within a file can be done with any of several commands:\n\
  position        timeStamp\n\
  rewind\n\
\n\
The position command can be used to move around in a file, by\n\
specifying a specific time and date.\n\
\n\
For positioning to a time and date, the command has the form.  The fraction\n\
part of a second may be omitted, for positioning to the nearest second.  If\n\
the fractional part is specified, it must be 3 digits.\n\
  position  10/17/90  13:12:8.020\n\
\n\
");
/*-----------------------------------------------------------------------------
* help info--ARR usage information
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCmdCtx->helpList, &pCmdCtx->helpUsage, "usage", "\n\
ARR acquires data either from a \"by channel\" archive data file or from\n\
a \"sample set\" archive data file.\n\
\n\
Either the `byChannel' or the `sampleSet' command must be used prior to\n\
any command which retrieves data.  The `byChannel' command assumes that\n\
\"by channel\" data is to be retrieved from arChan/arChanData.bin,\n\
which is the usual file for holding \"by channel\" data.  (You can also\n\
specify an explicit path and file with the `by channel' command.)\n\
\n\
ARR keeps a list of channels for which samples are to be retrieved.\n\
Channels can be added to the list with the `channel' command and deleted\n\
from it with the `delete' command.  ARR retrieves data as synchronous\n\
samples, in which all values in the sample have the same time stamp.\n\
(Channels which change slowly or randomly can be included in a synchronous\n\
sample by using the `,fill' option on the `close' command.)\n\
\n\
Example of using ARR for \"by channel\" data files from AR.  Acquire\n\
30 synchronous samples for 4 channels, plot them on the screen, and\n\
then get a hard copy of the plot.\n\
\n\
  % cd ...          (move to directory where AR was run)\n\
  % ARR\n\
    ARR: byChannel\n\
    ARR: channel rai_2000 rai_2001\n\
    ARR: channel,fill rai_2002,rao_2000\n\
    ARR: plot 30\n\
    ARR: plotHC cx1\n\
    ARR: quit\n\
  %\n\
\n\
Example of using ARR for \"sample set\" data files from AR.  Find out\n\
what files are available, select one, and then find out what channels\n\
are available in the file.  Select 2 channels, get 40 synchronous\n\
samples and plot them to verify that the proper data has been located.\n\
Finally, write the samples to a file called `myFile.ss' in the `arSet'\n\
subdirectory for further analysis by a spreadsheet program.\n\
\n\
  % cd ...          (move to directory where AR was run)\n\
  % ls arSet/*.txt  (find out what data files are available)\n\
  % ARR\n\
    ARR: sampleSet arSet/test1.txt          (use the test1.txt data file)\n\
    ARR: fileInfo\n\
    ARR: channel rai_2000 rai_2001\n\
    ARR: channel,fill rai_2002,rao_2000\n\
    ARR: plot 30\n\
    ARR: dataOut arSet/myFile.ss\n\
    ARR: export,ss\n\
    ARR: quit\n\
  %\n\
");
    return OK;
}

/*/subhead channel------------------------------------------------------------
* channel
*----------------------------------------------------------------------------*/
ARR_channel(pArrCtx)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    long	stat;
    int		syncType;

    sydSampleSetFree(pSspec);

    if ((FLD_LEN = nextChanNameField(&P_LINE, &P_FLD, &DELIM)) <= 1)
	(void)printf("you must specify one or more channels\n");
    while (FLD_LEN > 1) {
	syncType = SYD_SY_NONF;
	if (strcmp(P_FLD, "F") == 0) {
	    syncType = SYD_SY_FILLED;
	    FLD_LEN = nextChanNameField(&P_LINE, &P_FLD, &DELIM);
	}
	if (P_FLD[0] == '#')
	    break;
	else if (sydChanFind(pSspec, P_FLD) != NULL)
	    (void)printf("%s already selected\n", P_FLD);
	else {
	    stat = sydChanOpen(pSspec, NULL, P_FLD, syncType, NULL,0);
	    if (stat != S_syd_OK)
		(void)printf("couldn't open %s \n", P_FLD);
	    else
		pSspec->chanCount++;
	}
	FLD_LEN = nextChanNameField(&P_LINE, &P_FLD, &DELIM);
    }
    return;
}

/*/subhead delete-------------------------------------------------------------
* delete
*----------------------------------------------------------------------------*/
ARR_delete(pArrCtx)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    SYD_CHAN	*pSChan, *pSChanNext;

    FLD_LEN = nextChanNameField(&P_LINE, &P_FLD, &DELIM);
    if (FLD_LEN <= 1) {
	(void)printf(
		"you must specify one or more channels or else specify all\n");
	return;
    }
    if (strcmp(P_FLD, "all") == 0) {
	pSChan = pSspec->pChanHead;
	while (pSChan != NULL) {
	    pSChanNext = pSChan->pNext;
	    if (sydChanClose(pSspec, pSChan) != S_syd_OK) {
		(void)printf("error deleting %s\n", P_FLD);
	    }
	    pSChan = pSChanNext;
	}
	return;
    }
    while (FLD_LEN > 1) {
	if ((pSChan = sydChanFind(pSspec, P_FLD)) == NULL)
	    (void)printf("%s not selected\n", P_FLD);
	else {
	    if (sydChanClose(pSspec, pSChan) != S_syd_OK) {
		(void)printf("error deleting %s\n", P_FLD);
	    }
	}
	FLD_LEN = nextChanNameField(&P_LINE, &P_FLD, &DELIM);
    }
}

/*/subhead export-------------------------------------------------------------
* export
*----------------------------------------------------------------------------*/
ARR_export(pArrCtx)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    int		reqCount;
    int		option;

    if (DELIM != ',' ||
        nextANField(&P_LINE, &P_FLD, &DELIM) <= 1) {
	(void)printf("export option must be specified\n");
	return;
    }
    if (strcmp(P_FLD, "ss") == 0)
	option = 1;
    else if (strcmp(P_FLD, "ssStat") == 0)
	option = 2;
    else {
	(void)printf("illegal export option\n");
	return;
    }
    if (nextIntFieldAsInt(&P_LINE, &reqCount, &DELIM) > 1) {
	if (reqCount > 10000) {
	    (void)printf("sample count must be <= 10000\n");
	    return;
	}
    }
    else
	reqCount = 0;

    if (reqCount > 0)
	ARR_sample(pArrCtx, reqCount);

    if (pSspec->sampleCount <= 0)
	(void)printf("no data to export\n");
    else {
	if (pArrCtx->useStats == 0 && pArrCtx->useMeans == 0)
	    sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 0, NULL);
	else {
	    if (pArrCtx->useStats)
		sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 1, NULL);
	    else
		sydSetAttr(pSspec, SYD_ATTR_USE_MEANS, 1, NULL);
	    sydSampleSetStats(pSspec);
	}

	sydSampleSetExport(pSspec, pCmdCtx->dataOut, option);
    }
}

/*/subhead plot---------------------------------------------------------------
* plot
*----------------------------------------------------------------------------*/
ARR_plot(pArrCtx, cmdType)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
ARR_P_TYPE cmdType;
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    int		reqCount;
    char	*pPlotDispName;
    char	*pPlotFile;
    int		i;

    if (DELIM == ',') {
	if (nextANField(&P_LINE, &P_FLD, &DELIM) <= 1)
	    goto ARR_plotTyErr;
	pArrCtx->linePlot = 0;
	pArrCtx->pointPlot = 0;
	pArrCtx->markPlot = 0;
	pArrCtx->showStat = 0;
	pArrCtx->errBar = 0;
	if (*P_FLD == 'p') {
	    pArrCtx->pointPlot = 1;
	    P_FLD++;
	}
	if (*P_FLD == 'l') {
	    pArrCtx->linePlot = 1;
	    P_FLD++;
	}
	if (*P_FLD == 'm') {
	    pArrCtx->markPlot = 1;
	    P_FLD++;
	}
	if (pArrCtx->markPlot == 0 && pArrCtx->linePlot == 0 &&
					pArrCtx->pointPlot == 0) {
	    pArrCtx->linePlot = 1;
	}
	if (strncmp(P_FLD, "tyy", 3) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_TYY;
	    P_FLD += 3;
	}
	else if (strncmp(P_FLD, "ty", 2) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_TY;
	    P_FLD += 2;
	}
	else if (strncmp(P_FLD, "xyy", 3) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_XYY;
	    P_FLD += 3;
	}
	else if (strncmp(P_FLD, "xy", 2) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_XY;
	    P_FLD += 2;
	}
	else if (strncmp(P_FLD, "yy", 2) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_YY;
	    P_FLD += 2;
	}
	else if (strncmp(P_FLD, "y", 1) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_Y;
	    P_FLD += 1;
	}
	else
	    goto ARR_plotTyErr;
	if (*P_FLD == 's') {
	    pArrCtx->showStat = 1;
	    P_FLD++;
	}
	if (*P_FLD == 'e') {
	    pArrCtx->errBar = 1;
	    if (pArrCtx->markPlot == 0)
		pArrCtx->markPlot = 1;
	    P_FLD++;
	}
	if (*P_FLD != '\0')
	    goto ARR_plotTyErr;
    }
    if (cmdType == ARR_P_PLAIN) {
	if (DELIM != '\n') {
	    if (nextIntFieldAsInt(&P_LINE, &reqCount, &DELIM) > 1) {
		if (reqCount > 10000) {
		    (void)printf("sample count must be <= 10000\n");
		    return;
		}
		ARR_sample(pArrCtx, reqCount);
	    }
	    else {
		(void)printf("syntax error\n");
		return;
	    }
	}
	pPlotFile = NULL;
	pPlotDispName = NULL;
    }
    else if (cmdType == ARR_P_EPS) {
	if (nextNonSpaceField(&P_LINE, &pPlotFile, &DELIM) <= 1)
	    pPlotFile = NULL;

	if (pPlotFile == NULL) {
	    if (pArrCtx->plotEPSfile[0] == '\0') {
		(void)printf("An EPS file hasn't previously been specified.\n");
		(void)printf("Use the command again, specifying a filePath\n");
		return;
	    }
	    pPlotFile = pArrCtx->plotEPSfile;
	}
	else if (strlen(pPlotFile) >= 120) {
	    (void)printf("plot file name too long\n");
	    return;
	}
	else if (sscanf(pPlotFile, "%d", &i) > 0) {
	    (void)printf("plot file name must contain at least 1 letter\n");
	    return;
	}
	else
	    (void)strcpy(pArrCtx->plotEPSfile, pPlotFile);
	pPlotDispName = NULL;
    }
    else if (cmdType == ARR_P_HC) {
	if (nextNonSpaceField(&P_LINE, &pPlotDispName, &DELIM) <= 1)
	    pPlotDispName = NULL;
	(void)sprintf(pArrCtx->printerFile, "/tmp/ARR%d.HC", getpid());
	pPlotFile = pArrCtx->printerFile;

	if (pPlotDispName == NULL) {
	    if (pArrCtx->printer[0] == '\0')
		goto plotHC_error;
	    pPlotDispName = pArrCtx->printer;
	}
	else if (strlen(pPlotDispName) >= 80) {
	    (void)printf("printer name too long\n");
	    return;
	}
	else if (sscanf(pPlotDispName, "%d", &i) > 0) {
	    (void)printf("printer name must contain at least 1 letter\n");
	    return;
	}
	else
	    (void)strcpy(pArrCtx->printer, pPlotDispName);
    }

    if (pSspec->sampleCount <= 0)
	(void)printf("no data to plot\n");
    else
	ARR_samplePlot(pArrCtx, cmdType, pPlotDispName, pPlotFile);

    return;
ARR_plotNErr:
    (void)printf("need more than 1 channel to use this type\n");
    return;
ARR_plotTyErr:
    (void)printf("error in plot type\n");
    return;
plotHC_error:
    (void)printf("\
a printer name is needed; the first time you use an xxxxHC command,\n\
you must specify one.  Use the form:\n\
	xxxxHC printerName\n");
}

/*/subhead position-----------------------------------------------------------
* position
*----------------------------------------------------------------------------*/
ARR_position(pArrCtx)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    TS_STAMP	posStamp;
    double	deltaTime;
    int		stat;
    SYD_CHAN	*pSChan;

    (void)nextNonSpace(&P_LINE);
    if (tsTextToStamp(&posStamp, &P_LINE) == S_ts_OK)
	sydPosition(pSspec, &posStamp);
    else
	(void)printf("error in time specification\n");
}

/*/subhead print[HC]----------------------------------------------------------
* print
*----------------------------------------------------------------------------*/
ARR_print(pArrCtx)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    int		reqCount;
    int		ncol, colWidth;

    if (nextIntFieldAsInt(&P_LINE, &reqCount, &DELIM) > 1) {
	if (reqCount > 10000) {
	    (void)printf("sample count must be <= 10000\n");
	    return;
	}
	ARR_sample(pArrCtx, reqCount);
    }
    if (DELIM == ',') {
	if (nextIntFieldAsInt(&P_LINE, &ncol, &DELIM) <= 1) {
	    (void)printf("illegal number of columns\n");
	    return;
	}
	if (ncol < 1 || ncol > 100) {
	    (void)printf("illegal number of columns\n");
	    return;
	}
	pArrCtx->colWidth = colWidth;
    }
    if (DELIM == ',') {
	if (nextIntFieldAsInt(&P_LINE,&colWidth,&DELIM) <= 1) {
	    (void)printf("illegal column width\n");
	    return;
	}
	if (colWidth < 2 || colWidth > 20) {
	    (void)printf("illegal column width\n");
	    return;
	}
	pArrCtx->colWidth = colWidth;
    }

    if (pSspec->sampleCount <= 0)
	(void)printf("no data to print\n");
    else
	ARR_samplePrint(pArrCtx, NULL, NULL);
}
ARR_printHC(pArrCtx)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    char	*pPrinter;
    int		i;
    int		ncol, colWidth;

    if (nextANField(&P_LINE, &pPrinter, &DELIM) <= 1)
	pPrinter = NULL;
    if (pArrCtx->printerFile[0] == '\0')
	(void)sprintf(pArrCtx->printerFile, "/tmp/ARR%d.HC", getpid());

    if (pPrinter == NULL) {
	if (pArrCtx->printer[0] == '\0')
	    goto printHC_error;
    }
    else if (strlen(pPrinter) >= 80) {
	(void)printf("printer name too long\n");
	return;
    }
    else if (sscanf(pPrinter, "%d", &i) > 0) {
	(void)printf("printer name must contain at least 1 letter\n");
	return;
    }
    else
	(void)strcpy(pArrCtx->printer, pPrinter);

    if (DELIM == ',') {
	if (nextIntFieldAsInt(&P_LINE, &ncol, &DELIM) <= 1) {
	    (void)printf("illegal number of columns\n");
	    return;
	}
	if (ncol < 1 || ncol > 100) {
	    (void)printf("illegal number of columns\n");
	    return;
	}
	pArrCtx->colWidth = colWidth;
    }
    if (DELIM == ',') {
	if (nextIntFieldAsInt(&P_LINE,&colWidth,&DELIM) <= 1) {
	    (void)printf("illegal column width\n");
	    return;
	}
	if (colWidth < 2 || colWidth > 20) {
	    (void)printf("illegal column width\n");
	    return;
	}
	pArrCtx->colWidth = colWidth;
    }

    ARR_samplePrint(pArrCtx, pArrCtx->printer, pArrCtx->printerFile);
    return;
printHC_error:
    (void)printf("\
a printer name is needed; the first time you use an xxxxHC command,\n\
you must specify one.  Use the form:\n\
	xxxxHC printerName\n");
}

/*/subhead sample-------------------------------------------------------------
* sample
*----------------------------------------------------------------------------*/
ARR_sample(pArrCtx, reqCount)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
int	reqCount;
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    long	stat;
    int 	done=0;
    double	diff;
    
/*-----------------------------------------------------------------------------
*	start time fresh for all channels
*----------------------------------------------------------------------------*/
    sydInputResetSampled(pSspec);
/*----------------------------------------------------------------------------
*	reserve memory for the various sample storage arrays if there is
*	no storage or its size has increased
*----------------------------------------------------------------------------*/
    if (sydSampleSetAlloc(pSspec, reqCount) != S_syd_OK)
	return ERROR;
/*----------------------------------------------------------------------------
*	now, gather the data  (note that if `accept if' was specified, it
*	was applied to pSspec when the command was encountered)
*----------------------------------------------------------------------------*/
    while (!done) {
	stat = sydInputGet(pSspec, NULL);
	if (pArrCtx->deadRst == 1) {
	    if (pArrCtx->deadRefTs.secPastEpoch == 0)
		pArrCtx->deadRefTs = SydInputTs(pSspec);
	    else {
		TsDiffAsDouble(&diff,&SydInputTs(pSspec),&pArrCtx->deadRefTs);
		if (diff > pArrCtx->deadRstSec) {
		    sydInputResetSampled(pSspec);
		    stat = sydInputGet(pSspec, NULL);
		}
		pArrCtx->deadRefTs = SydInputTs(pSspec);
	    }
	}
	if (stat == S_syd_OK || stat == S_syd_partial) {
	    sydInputStoreInSet(pSspec, pArrCtx->discardPartial);
	    if (pSspec->sampleCount >= pSspec->reqCount)
		done = 1;
	}
	else
	    done = 1;
    }
}

/*/subhead samplePlot---------------------------------------------------------
* sample plot
*----------------------------------------------------------------------------*/
ARR_samplePlot(pArrCtx, cmdType, printerName, fileName)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
ARR_P_TYPE cmdType;
char	*printerName;
char	*fileName;
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    int		iChan;		/* index to step through channels */
    SYD_PL_SLAVE	*pSlave;
    long	stat;
    SYD_CHAN	*pSChan;
    TS_STAMP	now;
    char	nowText[32], begText[28], endText[28];
    char	title[120];
    char	command[160];
    char	resultBuf[80];

    if (pArrCtx->useStats == 0 && pArrCtx->useMeans == 0)
	sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 0, NULL);
    else {
	if (pArrCtx->useStats)
	    sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 1, NULL);
	else
	    sydSetAttr(pSspec, SYD_ATTR_USE_MEANS, 1, NULL);
	sydSampleSetStats(pSspec);
    }

    (void)tsLocalTime(&now);
    (void)tsStampToText(&now, TS_TEXT_MONDDYYYY, nowText);
    (void)tsStampToText(&pSspec->pTimeStamp[pSspec->firstData], TS_TEXT_MMDDYY,
								begText);
    (void)tsStampToText(&pSspec->pTimeStamp[pSspec->lastData], TS_TEXT_MMDDYY,
								endText);
    (void)sprintf(title, "samples for %s through %s", begText, endText);

    if (cmdType == ARR_P_PLAIN) {
	stat = sydPlotInit(&pArrCtx->master, pSspec,
			PPR_WIN_SCREEN, NULL, "ARR plot",
			pArrCtx->plotInit);
    }
    else if (cmdType == ARR_P_EPS) {
	assert(fileName != NULL);
	stat = sydPlotInit(&pArrCtx->master, pSspec,
			PPR_WIN_EPS, fileName, NULL,
			pArrCtx->plotInit);
    }
    else {
	assert(fileName != NULL);
	stat = sydPlotInit(&pArrCtx->master, pSspec,
			PPR_WIN_POSTSCRIPT, fileName, NULL,
			pArrCtx->plotInit);
    }
    if (stat == S_syd_OK)
	stat = sydPlotSetTitles(&pArrCtx->master, title, NULL, NULL, NULL);
    sydPlotSetAttr(&pArrCtx->master, SYD_PLATTR_STDDEV, pArrCtx->errBar, NULL);
    if (stat == S_syd_OK) {
	pArrCtx->plotInit = 0;
	pArrCtx->master.plotAxis = pArrCtx->plotAxis;
	pArrCtx->master.linePlot = pArrCtx->linePlot;
	pArrCtx->master.pointPlot = pArrCtx->pointPlot;
	pArrCtx->master.markPlot = pArrCtx->markPlot;
	pArrCtx->master.showStat = pArrCtx->showStat;
	iChan = 0;
	pSChan = pSspec->pChanHead;
	while (pSChan != NULL) {
	    assert(iChan < pSspec->chanCount);
	    if (pSChan->dataChan) {
		pSlave = sydPlotChanAdd(&pArrCtx->master, pSChan);
		assert(pSlave != NULL);
		iChan++;
		pSChan = pSChan->pNext;
	    }
	}

	stat = sydPlotWinLoop(&pArrCtx->master);
	assert(stat == S_syd_OK);

	stat = sydPlotDone(&pArrCtx->master, 1);
	assert(stat == S_syd_OK);

	if (cmdType == ARR_P_HC) {
	    (void)sprintf(command, "lpr -P%s %s", printerName, fileName);
	    genShellCommand(command, resultBuf, 80);
	    if (resultBuf[0] != '\0')
		(void)printf("%s\n", resultBuf);
	    unlink(fileName);
	}
    }
}

/*/subhead samplePrint--------------------------------------------------------
* samplePrint
*----------------------------------------------------------------------------*/
ARR_samplePrint(pArrCtx, printerName, fileName)
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
char	*printerName;
char	*fileName;
{
    CX_CMD	*pCmdCtx=pArrCtx->pCmdCtx;
    SYD_SPEC	*pSspec=pArrCtx->pSspec;
    long	stat;
    FILE	*out;
    char	command[160];
    char	resultBuf[80];
    int		fmtFlag=0;

    if (pArrCtx->useStats == 0 && pArrCtx->useMeans == 0)
	sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 0, NULL);
    else {
	if (pArrCtx->useStats)
	    sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 1, NULL);
	else
	    sydSetAttr(pSspec, SYD_ATTR_USE_MEANS, 1, NULL);
	sydSampleSetStats(pSspec);
    }

    if (printerName != NULL) {
	assert(fileName != NULL);
	if ((out = fopen(fileName, "w")) == NULL) {
	    (void)printf("error opening %s\n", fileName);
	    perror(NULL);
	    return;
	}
    }
    else
	out = pCmdCtx->dataOut;

    if (out != stdout)
	fmtFlag = 1;
    stat = sydSampleSetPrint(pSspec, out, fmtFlag,
		pArrCtx->ncol, pArrCtx->colWidth);

    if (printerName != NULL) {
	fclose(out);
	(void)sprintf(command, "lpr -P%s %s", printerName, fileName);
	genShellCommand(command, resultBuf, 80);
	if (resultBuf[0] != '\0')
	    (void)printf("%s\n", resultBuf);
	unlink(fileName);
    }
}
