/*	@(#)ARR_gui.c	1.3 8/21/91
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .00	02-08-91	rac	initial version
 * .01	07-31-91	rac	installed in SCCS
 * .02	08-14-91	rac	add callback for `by channel' file listing
 * ARR_gui.c,v
 * Revision 1.2  1994/05/06 19:05:36  mrk
 * conversion from sccs to cvs
 *
 *
 * make options
 *	-DXWINDOWS	to use xview/X Window System
 */
/*+/mod***********************************************************************
* TITLE	ARR_gui.c - general gui routines for ARR
*
* DESCRIPTION
*
*-***************************************************************************/
#include <stdio.h>
#include <sys/ioctl.h>	/* for use with notifier */
#include <sys/wait.h>	/* for use with notifier */
#include <signal.h>	/* for use with notifier */

#if defined XWINDOWS
#   include <xview/xview.h>
#   include <xview/frame.h>
#   include <xview/notice.h>
#   include <xview/notify.h>
#   include <xview/openmenu.h>
#   include <xview/panel.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <sydDefs.h>

#include "ARR.h"


/*+/internal******************************************************************
* NAME	guiShellCmd - issue a shell command, cooperating with the notifier
*
* DESCRIPTION
*	This routine issues a shell command under the auspices of the
*	xview notifier.  The command is issued following a fork, with a
*	popen call.  The critical issue is that the fork results in a
*	SIGCHLD signal which the notifier must handle.
*
*	This routine (actually guiCmdSigchld) must be customized for each
*	application.
*
*	This customization handles:
*	o   doing a file load into sample set text subwindow of the file
*	    containing a list of file names
*
* RETURNS
*	void
*
* BUGS
* o	when running under dbx, the child process always results in a
*	core dump
* o	this routine isn't re-entrant, as it uses global variables for
*	pipe_io fd's and for pCtx.  I haven't figured out how to get the
*	pCtx attached to something the notifier handles, and thus haven't
*	dealt with the pipe_io issue.
*
*-*/
static int	pipe_io[2][2];
Notify_value	guiCmdRead(), guiCmdSigchld();

void
guiShellCmd(cmdBuf, clientNum)
char	*cmdBuf;	/* I the command string to give the shell */
Notify_client clientNum;/* I a unique, arbitrary value to be used by
				guiCmdSigchld when the child is done
				processing the command */
{
    FILE	*fp;
    int		i, pid;
    char	resultBuf[80];

/*-----------------------------------------------------------------------------
* blatantly stolen from "XView Programming Manual", Dan Heller, O'Reilly &
*	Associates, Inc., July 1990, pp.386-389; section 19.8.2, Reading and
*	Writing on Pipes; Example 19-4.  The ntfy_pipe.c program
*
*    child reads:  |<=========== pipe_io[0] <===========|  parent writes:
*  pipe_io[0][0]                                           pipe_io[0][1]
*
*   parent reads:  |<=========== pipe_io[1] <===========|  child writes:
*  pipe_io[1][0]                                           pipe_io[1][1]
*----------------------------------------------------------------------------*/

    pipe(pipe_io[0]);
    pipe(pipe_io[1]);
    if ((pid = fork()) == -1) {
	perror("error executing shell command");
	close(pipe_io[0][0]);
	close(pipe_io[0][1]);
	close(pipe_io[1][0]);
	close(pipe_io[1][1]);
	return;
    }
    else if (pid == 0) {				/* child process */
	dup2(pipe_io[0][0], 0);		/* stdin */
	dup2(pipe_io[1][1], 1);		/* stdout */
	dup2(pipe_io[1][1], 2);		/* stderr */
	for (i=getdtablesize(); i>2; i--)
	    close(i);			/* close the pipe fd's */
	for (i=0; i<NSIG; i++)
	    signal(i, SIG_DFL);		/* don't need parent's sig masks */
	if ((fp = popen(cmdBuf, "r")) == NULL) {
	    perror("couldn't do popen to shell");
	    return;
	}
	while ((i = fread(resultBuf, 1, sizeof(resultBuf), fp)) > 0)
	    fwrite(resultBuf, 1, i, stdout);
	_exit(0);
    }
    close(pipe_io[0][0]);
    close(pipe_io[0][1]);
    close(pipe_io[1][1]);
    notify_set_input_func(clientNum, guiCmdRead, pipe_io[1][0]);
    notify_set_wait3_func(clientNum, guiCmdSigchld, pid);
}
Notify_value
guiCmdRead(client, fd)
Notify_client client;
int		fd;
{
    char	buf[80];
    int		nBytes, i;

    if (ioctl(fd, FIONREAD, &nBytes) == 0) {
	while (nBytes > 0) {
	    if ((i = read(fd, buf, sizeof(buf))) > 0) {
#if   0
		textsw_insert(textsw, buf, i);
		write(1, buf, i);
#endif
		nBytes -= i;
	    }
	}
    }
    return NOTIFY_DONE;
}

/*/subhead guiCmdSigchld-------------------------------------------------------
*	When the child process gets done creating a file, it will die,
*	producing SIGCHLD.  At that, time, load the resultant file into
*	the text subwindow.
*----------------------------------------------------------------------------*/
Notify_value
guiCmdSigchld(client, pid, status)
Notify_client client;
int	pid;
union wait *status;
{
    Notify_value i;

    if (WIFEXITED(*status)) {
	notify_set_input_func(client, NOTIFY_FUNC_NULL,
		(client == arrCtx.RETssfLsClient) ? pipe_io[1][0] : 0);
	i = NOTIFY_DONE;
    }
    else
	i = NOTIFY_IGNORED;
    if (client == arrCtx.RETssfLsClient)
	RETssfLsDone(&arrCtx);
    else if (client == arrCtx.RETbcfLsClient)
	RETbcfLsDone(&arrCtx);
    return i;
}
