/*	@(#)ARR_kbd.c	1.11 8/11/93
 *	Author:	Roger A. Cole
 *	Date:	09-17-90
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	09-17-90	rac	initial version
 * .02	07-31-91	rac	installed in SCCS
 * .03	08-21-91	rac	use new sydXxx routines
 * .04	09-11-91	joh	updated for v5 vxWorks
 * .05	09-23-91	rac	specifically include Xlib.h
 * .06	09-30-91	rac	use new calling sequence for sydChanOpen and
 *				arCFOpen
 * .07	11-27-91	rac	use new calling sequence for sydSamplePrint
 *				and sydSampleSetPrint;
 *				support ignoring partial samples, rounding
 *				time stamps, stopping filling after a
 *				deadtime; allow specifying number and width
 *				of print columns
 * 08	12-08-91	rac	use new calling seq for cmdRead; don't use LWP
 * 09	08-11-93	mrk	Removed ifdef V5_vxWorks
 *
 * make options
 *	-DvxWorks	makes a version for VxWorks
 *	-DNDEBUG	don't compile assert() checking
 *      -DDEBUG         compile various debug code, including checks on
 *                      malloc'd memory
 */
/*+/mod***********************************************************************
* TITLE	ARR_kbd - archive data retrieval program
*
* DESCRIPTION
*
* WISH LIST
* o	selection by time stamp range or other conditions as for the
*	AR SAMPLE: command
* o	read an archive request to determine channel selection for retrieval
*
* BUGS
* o	retrievals are limited to a single file
* o	this program should use tasks.h to establish priorities, stack
*	sizes, etc.
* o	using this skeleton, it isn't possible to have several invocations
*	of the program running simultaneously under VxWorks
*
*-***************************************************************************/
/*-----------------------------------------------------------------------------
* under SunOS, the lightweight process library interferes with the operation
* of the sunview library, so this program runs single threaded
*----------------------------------------------------------------------------*/
#define USETASKS
#ifndef vxWorks
#   undef USETASKS
#endif

#include <X11/Xlib.h>
#include <genDefs.h>
#ifdef USETASKS
#   include <genTasks.h>
#endif
#include <cmdDefs.h>
#include <tsDefs.h>
#include <sydDefs.h>
#include <pprPlotDefs.h>
#include <sydPlotDefs.h>
#include <alarm.h>
#ifndef INC_arCS_h
#   include <arCS.h>
#endif
#ifndef INCLcadefh
#   include <cadef.h>
#endif
#ifndef INCLdb_accessh
#   include <db_access.h>
#endif

#ifdef vxWorks
/*----------------------------------------------------------------------------
*    includes and defines for VxWorks compile
*---------------------------------------------------------------------------*/
#   include <vxWorks.h>
#   include <ioLib.h>           /* for O_RDWR and O_RDONLY definitions */
#   include <stdioLib.h>
#   include <strLib.h>
#   include <ctype.h>
#   include <sigLib.h>
#   include <setjmp.h>
#ifdef USETASKS
#   include <taskLib.h>
#   define MAXPRIO 160
#endif
#else
/*----------------------------------------------------------------------------
*    includes and defines for Sun compile
*---------------------------------------------------------------------------*/
#   include <stdio.h>
#   include <sys/file.h>        /* for O_RDWR and O_RDONLY definitions */
#   include <string.h>
#   include <ctype.h>
#   include <signal.h>
#   include <setjmp.h>
#ifdef USETASKS
#   define MAXPRIO 50
#endif
#endif

/*/subhead ARR_CTX-------------------------------------------------------
* ARR_CTX - context information
*
*	A ARR descriptor is the `master handle' which is used for
*	handling business.
*----------------------------------------------------------------------------*/
#ifdef USETASKS
#   define ArrLock semTake(pglArrCtx->semLock, WAIT_FOREVER)
#   define ArrUnlock semGive(pglArrCtx->semLock)
#   define ArrLockCheck semClear(pglArrCtx->semLock)
#   define ArrLockInitAndLock semInit(pglArrCtx->semLock)
#endif

typedef struct {
#ifdef USETASKS
    TASK_ID	id;		/* ID of task */
#endif
    int		status;		/* status of task--initially ERROR */
    int		stop;		/* task requested to stop if != 0 */
    int		stopped;	/* task has stopped if != 0 */
    int		serviceNeeded;	/* task needs servicing */
    int		serviceDone;	/* task servicing completed */
    jmp_buf	sigEnv;		/* environment for longjmp at signal time */
} ARR_TASK_INFO;

typedef struct ARRCtx {
#ifdef USETASKS
    SEM_ID	semLock;
#endif
    ARR_TASK_INFO ARRTaskInfo;
    ARR_TASK_INFO ARRInTaskInfo;
    int		showStack;		/* show stack stats on task terminate */
    char	bychanFile[80];		/* by channel data file name */
    char	sampleFile[80];		/* sample set data file name */
    SYD_SPEC	*pSspec;		/* synchronous set spec */
    char	printer[80];		/* printer name */
    char	printerFile[80];	/* print temporary file */
    char	plotEPSfile[120];	/* file for EPS plots */
    SYD_PLAX	plotAxis;		/* type of plot */
    int		linePlot;		/* 1 for line plot */
    int		markPlot;		/* 1 for mark plot */
    int		pointPlot;		/* 1 for point plot */
    int		showStat;		/* 1 to show alarm status */
    SYD_PL_MSTR	master;			/* sync data plot master */
    int		plotInit;		/* 1 plot window init needed */
    int		deadRst;		/* 1 for reset fill after deadtime */
    TS_STAMP	deadRefTs;		/* reference stamp for deadtime */
    double	deadRstSec;		/* deadtime restart interval, in sec */
    int		ignorePartial;		/* 1 to ignore partial samples */
    int		ncol;			/* number of print columns */
    int		colWidth;		/* width of print columns, in char */
} ARR_CTX;

/*-----------------------------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
int ARR();
void ARRCmdProcess();
long ARRInitAtStartup();
char *ARRInTask();
long ARRTask();
#ifdef USETASKS
long ARRTaskCheck();
#endif
void ARRTaskSigHandler();
void ARRTaskSigInit();

/*-----------------------------------------------------------------------------
* global definitions
*----------------------------------------------------------------------------*/
CX_CMD	glArrCxCmd;
CX_CMD	*pglArrCxCmd=NULL;
ARR_CTX	glArrCtx;
ARR_CTX	*pglArrCtx=NULL;
static HELP_TOPIC helpExport;	/* help for "export" command */
static HELP_TOPIC helpOut;	/* help for the various output commands */
static HELP_TOPIC helpPlot;	/* help for "plot" command */
static HELP_TOPIC helpPos;	/* help for "position" command */

typedef enum {
    ARR_P_PLAIN,	/* ordinary print or plot command */
    ARR_P_HC,		/* printHC and plotHC */
    ARR_P_EPS		/* plotEPS */
} ARR_P_TYPE;

#ifndef vxWorks
    main()
    {
#ifdef USETASKS
/*----------------------------------------------------------------------------
*    do some lwp initialization:
*    o  set allowable priorities between 1 and MAXPRIO
*    o  set up a cache of stacks for MAXTHREAD stacks
*
*    A side effect is that this routine is turned into a lwp thread with a
*    priority of MAXPRIO.
*---------------------------------------------------------------------------*/
        (void)pod_setmaxpri(MAXPRIO);
        lwp_setstkcache(100000, 9);
#endif

	return ARR();
    }
#endif

/*+/subr**********************************************************************
* NAME	ARR - shell callable interface for ARR
*
* DESCRIPTION
*	This routine is the only part of ARR which is intended to be
*	called directly from the shell.  Several functions are performed here:
*	o   if the ARRTask doesn't exist, spawn it.  Ideally, this
*	    stage would also detect if the ARRTask is suspended
*	    and take appropriate action.
*	o   if the ARRInTask doesn't exist, spawn it.  If it does exist, an
*	    error exists.
*	o   wait until the ARRInTask quits, then return to the shell.  If
*	    other tasks belonging to ARR are being stopped, then this
*	    routine waits until they, too, are stopped before returning to
*	    the shell.
*
* RETURNS
*	OK, or
*	ERROR
*
* BUGS
* o	stack size and priority should come from tasks.h
* o	there are lots of "holes" in detecting whether tasks exist, are
*	suspended, etc.
*
*-*/
int
ARR()
{
    long	stat;		/* status return from calls */

    if (pglArrCxCmd == NULL ||
		(pglArrCtx->ARRTaskInfo.stop == 1 &&
		 pglArrCtx->ARRInTaskInfo.stop == 1)   ) {
	pglArrCtx = &glArrCtx;
	pglArrCxCmd = &glArrCxCmd;
	pglArrCtx->deadRst = 0;
	pglArrCtx->ignorePartial = 0;
	pglArrCtx->ncol = 4;
	pglArrCtx->colWidth = 12;
	stat = ARRInitAtStartup(pglArrCtx, pglArrCxCmd);
	assert(stat == OK);

#ifdef vxWorks
	assert(taskNameToId("ARRTask") == ERROR);
	assert(taskNameToId("ARRInTask") == ERROR);
#endif
	pglArrCtx->showStack = 0;

	pglArrCtx->ARRTaskInfo.status = ERROR;
	pglArrCtx->ARRTaskInfo.stop = 0;
	pglArrCtx->ARRTaskInfo.stopped = 1;

	pglArrCtx->ARRInTaskInfo.status = ERROR;
	pglArrCtx->ARRInTaskInfo.stop = 0;
	pglArrCtx->ARRInTaskInfo.stopped = 1;
    }
    pglArrCtx->ARRInTaskInfo.serviceNeeded = 0;
    pglArrCtx->ARRInTaskInfo.serviceDone = 1;

#ifdef USETASKS
/*-----------------------------------------------------------------------------
* ARRTask
*	check its status; spawn if it doesn't exist
*----------------------------------------------------------------------------*/
    stat = ARRTaskCheck(pglArrCxCmd, "ARRTask", &pglArrCtx->ARRTaskInfo);
    if (stat == ERROR)
	return ERROR;
    if (pglArrCtx->ARRTaskInfo.status == ERROR) {
	pglArrCtx->ARRTaskInfo.id = taskSpawn("ARRTask", MAXPRIO,
		VX_STDIO | VX_FP_TASK, 50000, ARRTask, &pglArrCxCmd);
    }
    if (GenTaskNull(pglArrCtx->ARRTaskInfo.id)) {
	(void)printf("error spawning ARRTask\n");
	return ERROR;
    }
    pglArrCtx->ARRTaskInfo.status = OK;
    pglArrCtx->ARRTaskInfo.stopped = 0;

/*-----------------------------------------------------------------------------
* ARRInTask
*	check its status; spawn if it doesn't exist
*----------------------------------------------------------------------------*/
    stat = ARRTaskCheck(pglArrCxCmd, "ARRInTask", &pglArrCtx->ARRInTaskInfo);
    if (stat == ERROR)
	return ERROR;
    if (pglArrCtx->ARRInTaskInfo.status == ERROR) {
	pglArrCtx->ARRInTaskInfo.id = taskSpawn("ARRInTask", MAXPRIO,
		VX_STDIO | VX_FP_TASK, 50000, ARRInTask, &pglArrCxCmd);
    }
    if (GenTaskNull(pglArrCtx->ARRInTaskInfo.id)) {
	(void)printf("error spawning ARRInTask\n");
	return ERROR;
    }
    pglArrCtx->ARRInTaskInfo.status = OK;
    pglArrCtx->ARRInTaskInfo.stopped = 0;

/*-----------------------------------------------------------------------------
*    wait for ARRInTask to exit and then return to the shell.  If other
*    "ARR tasks" are also exiting, wait until their wrapups are complete.
*----------------------------------------------------------------------------*/
    while (!pglArrCtx->ARRInTaskInfo.stopped)
	taskSleep(SELF, 1, 0);
    if (pglArrCtx->ARRTaskInfo.stop) {
	while (!pglArrCtx->ARRTaskInfo.stopped)
	    taskSleep(SELF, 1, 0);
    }
#else
    ARRTask(&pglArrCxCmd);
#endif

    return OK;
}

/*+/subr**********************************************************************
* NAME	ARRTask - main processing task for ARR
*
* DESCRIPTION
*
* RETURNS
*	OK, or
*	ERROR
*
* BUGS
* o	text
*
*-*/
long
ARRTask(ppCxCmd)
CX_CMD	**ppCxCmd;
{
    long	stat;
    CX_CMD	*pCxCmd;

    pCxCmd = *ppCxCmd;

#ifdef USETASKS
    ArrLockInitAndLock;
    ArrUnlock;
#endif

    ARRTaskSigInit();

    if (setjmp(pglArrCtx->ARRTaskInfo.sigEnv) != 0)
	goto ARRTaskWrapup;

/*----------------------------------------------------------------------------
*    "processing loop"
*---------------------------------------------------------------------------*/
    while (!pglArrCtx->ARRTaskInfo.stop) {
#ifndef USETASKS
	ARRInTask(&pglArrCxCmd);
#endif
	if (pglArrCtx->ARRInTaskInfo.serviceNeeded) {
	    ARRCmdProcess(ppCxCmd, pglArrCtx);
	    pCxCmd = *ppCxCmd;
	    pglArrCtx->ARRInTaskInfo.serviceNeeded = 0;
	    pglArrCtx->ARRInTaskInfo.serviceDone = 1;
	}
#ifdef vxWorks		/* fflush on vxWorks isn't automatically flushed */
	fflush(stdout);
	fflush(pCxCmd->dataOut);
	fflush(stderr);
#endif
#ifdef USETASKS
	taskSleep(SELF, 0, 100000);	/* wait .1 sec */
#else
	usleep(100000);
#endif
    }

ARRTaskWrapup:
    if (pglArrCtx->pSspec != NULL) {
	stat = sydClose(pglArrCtx->pSspec);
	assert(stat == S_syd_OK);
    }

#if 0
    if (pglArrCtx->pCfDesc != NULL) {
	stat = arCFClose(&pglArrCtx->pCfDesc);
	assert(stat == OK);
    }
    if (pglArrCtx->sampleFP != NULL)
	(void)close(pglArrCtx->sampleFP);
#endif

    while ((*ppCxCmd)->pPrev != NULL)
	cmdCloseContext(ppCxCmd);
    pCxCmd = *ppCxCmd;

#ifdef USETASKS
    pglArrCtx->ARRInTaskInfo.stop = 1;
    while (pglArrCtx->ARRInTaskInfo.stopped == 0) {
	taskSleep(SELF, 1, 0);
    }
#endif

    if (pCxCmd->dataOut != stdout)
	fclose(pCxCmd->dataOut);

#ifdef vxWorks
    if (pglArrCtx->showStack)
	checkStack(pglArrCtx->ARRTaskInfo.id);
#endif

    pglArrCtx->ARRTaskInfo.stopped = 1;
    pglArrCtx->ARRTaskInfo.status = ERROR;

    return 0;
}

/*+/subr**********************************************************************
* NAME	ARRTaskSig - signal handling and initialization
*
* DESCRIPTION
*	These routines set up for the signals to be caught for ARRTask
*	and handle the signals when they occur.
*
* RETURNS
*	void
*
* BUGS
* o	not all signals are caught
* o	under VxWorks, taskDeleteHookAdd isn't used
* o	it's not clear how useful it is to catch the signals which originate
*	from the keyboard
*
*-*/
void
ARRTaskSigHandler(signo)
int	signo;
{
    printf("entered ARRTaskSigHandler for signal:%d\n", signo);
    signal(signo, SIG_DFL);
    longjmp(pglArrCtx->ARRTaskInfo.sigEnv, 1);
}


void
ARRTaskSigInit()
{
    (void)signal(SIGTERM, ARRTaskSigHandler);	/* SunOS plain kill (not -9) */
    (void)signal(SIGQUIT, ARRTaskSigHandler);	/* SunOS ^\ */
    (void)signal(SIGINT, ARRTaskSigHandler);	/* SunOS ^C */
    (void)signal(SIGILL, ARRTaskSigHandler);	/* illegal instruction */
#ifndef vxWorks
    (void)signal(SIGABRT, ARRTaskSigHandler);	/* SunOS assert */
    (void)signal(SIGCHLD, SIG_IGN);	/* child status changed */
#else
    (void)signal(SIGUSR1, ARRTaskSigHandler);	/* VxWorks assert */
#endif
    (void)signal(SIGBUS, ARRTaskSigHandler);	/* bus error */
    (void)signal(SIGSEGV, ARRTaskSigHandler);	/* segmentation violation */
    (void)signal(SIGFPE, ARRTaskSigHandler);	/* arithmetic exception */
}

/*+/subr**********************************************************************
* NAME	ARRInTask - handle the keyboard and keyboard commands
*
* DESCRIPTION
*	Gets input text and passes the input to ARRTask.
*
*	This task exists for two primary purposes: 1) avoid the possibility
*	of blocking ARRTask while waiting for operator input; and
*	2) allow detaching ARRTask from the keyboard while still
*	allowing ARRTask to run.
*
*	It is important to note that this task does no command processing
*	on ARRTask's behalf--all commands relating to ARRTask
*	are executed in ARRTask's own context.  Some
*	commands are related to getting input--these commands are processed
*	by this task and are never visible to ARRTask.
*
*	This task waits for input to be available from the keyboard.  When
*	an input line is ready, this task does some preliminary processing:
*	o   if the command is "quit" (or ^D), ARRTask is signalled
*	    and this task wraps itself up and returns.
*	o   if the command is "dataOut", this task sets a new destination
*	    for data output, closing the previous destination, if appropriate,
*	    and opening the new destination, if appropriate.
*	o   if the command is "bg", this task wraps itself up and returns.
*	    A flag is left in the command context indicating that ARRInTask
*	    doesn't exist any more.
*	o   if the command is "source", this task pushes down a level in
*	    the command context and begins reading commands from the file.
*	    When EOF occurs on the file, ARRInTask closes the file, pops
*	    to the previous level in the command context, and resumes
*	    reading at that level.
*	o   otherwise, ARRTask is signalled and this task
*	    goes into a sleeping loop until ARRTask signals
*	    that it is ready for the next command.
*
*	Ideally, this task would also support a mechanism for ARRTask
*	to wait explicitly for keyboard input.  An example would be when
*	operator permission is needed prior to taking an action.
*
* RETURNS
*	OK, or
*	ERROR
*
* BUGS
* o	doesn't support socket I/O
* o	doesn't support flexible redirection of output
* o	the implementation of "dataOut" is clumsy
*
*-*/
char *
ARRInTask(ppCxCmd)
CX_CMD	**ppCxCmd;	/* IO ptr to pointer to command context */
{
    char	*input=NULL;
/*----------------------------------------------------------------------------
*    wait for input from keyboard.  When some is received, signal caller,
*    wait for caller to process it, and then wait for some more input.
*
*    stay in the main loop until get ^D or 'quit'
*---------------------------------------------------------------------------*/
    while (1) {
#ifdef USETASKS
	while (pglArrCtx->ARRInTaskInfo.serviceDone == 0) {
	    if (pglArrCtx->ARRInTaskInfo.stop == 1)
		goto ARRInTaskDone;
	    taskSleep(SELF, 0, 500000);		/* sleep .5 sec */
	}
#endif
	input = cmdRead(ppCxCmd);

#ifdef USETASKS
	if (pglArrCtx->ARRInTaskInfo.stop == 1)
	    goto ARRInTaskDone;
#endif
	if (input == NULL)
#ifdef USETASKS
	    ;	/* blank lines and those which start non-AN are ignored */
#else
	    goto ARRInTaskDone;
#endif
	else if (strcmp((*ppCxCmd)->pCommand,		"dataOut") == 0) {
	    if ((*ppCxCmd)->dataOut != stdout)
		fclose ((*ppCxCmd)->dataOut);
	    if (nextNonSpaceField( &(*ppCxCmd)->pLine, &(*ppCxCmd)->pField,
							&(*ppCxCmd)->delim) < 1)
		(*ppCxCmd)->dataOut = stdout;
	    else {
		(*ppCxCmd)->dataOut = fopen((*ppCxCmd)->pField, "a");
		if ((*ppCxCmd)->dataOut == NULL) {
		    (void)printf("couldn't open %s\n", (*ppCxCmd)->pField);
		    (*ppCxCmd)->dataOut = stdout;
		}
	    }
	}
#ifdef USETASKS
	else if (strcmp((*ppCxCmd)->pCommand,		"bg") == 0) {
	    if (cmdBgCheck(*ppCxCmd) == OK)
		goto ARRInTaskDone;
	}
	else if (strcmp((*ppCxCmd)->pCommand,		"quit") == 0) {
	    pglArrCtx->ARRInTaskInfo.serviceDone = 0;
	    pglArrCtx->ARRInTaskInfo.serviceNeeded = 1;
	    goto ARRInTaskDone;
	}
#endif
	else if (strcmp((*ppCxCmd)->pCommand,		"source") == 0) {
	    cmdSource(ppCxCmd);
	}
	else {
	    pglArrCtx->ARRInTaskInfo.serviceDone = 0;
	    pglArrCtx->ARRInTaskInfo.serviceNeeded = 1;
#ifndef USETASKS
	    goto ARRInTaskDone;
#endif
	}
    }

ARRInTaskDone:
#ifdef USETASKS
    while ((*ppCxCmd)->pPrev != NULL)
	cmdCloseContext(ppCxCmd);

#ifdef vxWorks
    if (pglArrCtx->showStack)
	checkStack(pglArrCtx->ARRInTaskInfo.id);
#endif
    pglArrCtx->ARRInTaskInfo.stop = 1;
    pglArrCtx->ARRInTaskInfo.stopped = 1;
    pglArrCtx->ARRInTaskInfo.status = ERROR;
#endif

    return input;
}

/*+/subr**********************************************************************
* NAME	ARRCmdProcess - process a command line
*
* DESCRIPTION
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
ARRCmdProcess(ppCxCmd, pArrCtx)
CX_CMD	**ppCxCmd;	/* IO ptr to pointer to command context */
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
{
    CX_CMD	*pCxCmd;	/* local copy of pointer, for convenience */
    SYD_SPEC	*pSspec;
    long	stat;

    pCxCmd = *ppCxCmd;
    pSspec = pArrCtx->pSspec;

    if (strcmp(pCxCmd->pCommand,			"assert0") == 0) {
/*-----------------------------------------------------------------------------
*	under SunOS, assertAlways() generates SIGABRT; for VxWorks, SIGUSR1
*----------------------------------------------------------------------------*/
	assertAlways(0);
    }
    else if (strcmp(pCxCmd->pCommand,			"quit") == 0) {
	pArrCtx->ARRTaskInfo.stop = 1;
	return;
    }
#ifdef vxWorks
    else if (strcmp(pCxCmd->pCommand,			"showStack") == 0) {
	pArrCtx->showStack = 1;
	return;
    }
#endif
    else if (strcmp(pCxCmd->pCommand,			"trap") == 0) {
	int	*j;
	j = (int *)(-1);
	j = (int *)(*j);
	return;
    }
    else if (strcmp(pCxCmd->pCommand,			"byChannel") == 0 ||
    		strcmp(pCxCmd->pCommand, "bychannel") == 0) {
	if (pArrCtx->pSspec != NULL) {
	    stat = sydClose(pArrCtx->pSspec);
	    assertAlways(stat == S_syd_OK);
	    pArrCtx->pSspec = NULL;
	}
	if (nextNonSpaceField(&pCxCmd->pLine,&pCxCmd->pField,&pCxCmd->delim)>1) {
	    if (strlen(pCxCmd->pField) >= 80) {
		(void)printf("file name (or path) too long\n");
		return;
	    }
	    (void)strcpy(pArrCtx->bychanFile, pCxCmd->pField);
	}
	else
	    (void)strcpy(pArrCtx->bychanFile, "arChan/arChanData.bin");
	stat = sydOpenCF(&pArrCtx->pSspec, pArrCtx->bychanFile);
	if (stat != S_syd_OK) {
	    (void)printf("initialization failed\n");
	    return;
	}
	return;
    }
    else if (strcmp(pCxCmd->pCommand,			"sampleSet") == 0 ||
    		strcmp(pCxCmd->pCommand, "sampleset") == 0) {
	if (pArrCtx->pSspec != NULL) {
	    stat = sydClose(pArrCtx->pSspec);
	    assertAlways(stat == S_syd_OK);
	    pArrCtx->pSspec = NULL;
	}
	if (nextNonSpaceField(&pCxCmd->pLine, &pCxCmd->pField, &pCxCmd->delim)
								    <= 1) {
	    (void)printf("file name (or path) must be specified\n");
	    return;
	}
	if (strlen(pCxCmd->pField) >= 80) {
	    (void)printf("file name (or path) too long\n");
	    return;
	}
	(void)strcpy(pArrCtx->sampleFile, pCxCmd->pField);
	stat = sydOpenSSF(&pArrCtx->pSspec, pArrCtx->sampleFile);
	if (stat != S_syd_OK) {
	    (void)printf("initialization failed\n");
	    return;
	}
	return;
    }
    if (pSspec == NULL && strcmp(pCxCmd->pCommand, "help") != 0) {
	(void)printf( "no file selected yet; use `byChannel' or `sampleSet'\n");
	return;
    }
    if (strcmp(pCxCmd->pCommand,			"channel") == 0)
	ARR_channel(pCxCmd, pSspec);
    else if (strcmp(pCxCmd->pCommand,			"delete") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_delete(pCxCmd, pSspec);
    }
    else if (strcmp(pCxCmd->pCommand,			"export") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_export(pCxCmd, pSspec);
    }
    else if (strcmp(pCxCmd->pCommand,			"fileInfo") == 0 ||
    	strcmp(pCxCmd->pCommand, "fileinfo") == 0) {
	(void)sydFileInfo(pSspec, stdout);
    }
    else if (strcmp(pCxCmd->pCommand,			"plot") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_plot(pCxCmd, pArrCtx, pSspec, ARR_P_PLAIN);
    }
    else if (strcmp(pCxCmd->pCommand,			"plotEPS") == 0) {
	if (pSspec->chanCount <= 0 || pSspec->sampleCount <= 0) {
	    (void)printf("no data to plot\n");
	    return;
	}
	ARR_plot(pCxCmd, pArrCtx, pSspec, ARR_P_EPS);
    }
    else if (strcmp(pCxCmd->pCommand,			"plotHC") == 0) {
	if (pSspec->chanCount <= 0 || pSspec->sampleCount <= 0) {
	    (void)printf("no data to plot\n");
	    return;
	}
	ARR_plot(pCxCmd, pArrCtx, pSspec, ARR_P_HC);
    }
    else if (strcmp(pCxCmd->pCommand,			"position") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_position(pCxCmd, pSspec);
    }
    else if (strcmp(pCxCmd->pCommand,			"print") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_print(pCxCmd, pSspec);
    }
    else if (strcmp(pCxCmd->pCommand,			"printHC") == 0) {
	if (pSspec->chanCount <= 0 || pSspec->sampleCount <= 0) {
	    (void)printf("no data to print\n");
	    return;
	}
	ARR_printHC(pCxCmd, pArrCtx, pSspec);
    }
    else if (strcmp(pCxCmd->pCommand,			"resetFilled") == 0 ||
		strcmp(pCxCmd->pCommand, "resetfilled") == 0) {
	double	dead;
	if (pSspec->chanCount <= 0) goto noChanErr;
	if (nextFltFieldAsDbl(&pCxCmd->pLine, &dead, &pCxCmd->delim) <= 1) {
	    (void)printf("deadtime interval must be specified\n");
	    return;
	}
	if (dead <= 0.) {
	    (void)printf("deadtime must be > 0.\n");
	    return;
	}
	pArrCtx->deadRstSec = dead;
	pArrCtx->deadRst = 1;
    }
    else if (strcmp(pCxCmd->pCommand,			"ignorePartial") == 0 ||
		strcmp(pCxCmd->pCommand, "ignorepartial") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	pArrCtx->ignorePartial = 1;
    }
    else if (strcmp(pCxCmd->pCommand,			"rewind") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	sydPosition(pSspec, NULL);
    }
    else if (strcmp(pCxCmd->pCommand,			"roundStamps") == 0 ||
		strcmp(pCxCmd->pCommand, "roundstamps") == 0) {
	long	round;
	if (pSspec->chanCount <= 0) goto noChanErr;
	if (nextIntFieldAsLong(&pCxCmd->pLine, &round, &pCxCmd->delim) <= 1) {
	    (void)printf("rounding interval must be specified\n");
	    return;
	}
	if (round < 0 || round > 2000) {
	    (void)printf("0 < rounding interval <= 2000 violated\n");
	    return;
	}
	pSspec->roundNsec = round * 1000000;
    }
    else if (strcmp(pCxCmd->pCommand,			"skip") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_skip(pCxCmd, pSspec);
    }
    else if (strcmp(pCxCmd->pCommand,			"tsTest") == 0 ||
    		strcmp(pCxCmd->pCommand, "tstest") == 0) {
	if (pSspec->chanCount <= 0) goto noChanErr;
	ARR_tsTest(pCxCmd, pSspec);
    }
    else {
/*----------------------------------------------------------------------------
* help (or illegal command)
*----------------------------------------------------------------------------*/
	(void)nextNonSpaceField(&pCxCmd->pLine, &pCxCmd->pField,
							&pCxCmd->delim);
	helpIllegalCommand(stdout, &pCxCmd->helpList, pCxCmd->pCommand,
							pCxCmd->pField);
    }

    return;
noChanErr:
    (void)printf("no channels selected\n");
}

/*+/subr**********************************************************************
* NAME	ARRInitAtStartup - initialization for ARR
*
* DESCRIPTION
*	Perform several initialization duties:
*	o   initialize the channel data retrieval set
*	o   initialize the command context block
*	o   initialize the help information
*
* RETURNS
*	OK
*
* BUGS
* o	text
*-*/
long
ARRInitAtStartup(pArrCtx, pCxCmd)
ARR_CTX	*pArrCtx;
CX_CMD	*pCxCmd;
{
    long	stat;

    pArrCtx->sampleFile[0] = '\0';
    pArrCtx->pSspec = NULL;
    pArrCtx->printer[0] = '\0';
    pArrCtx->printerFile[0] = '\0';
    pArrCtx->plotEPSfile[0] = '\0';
    pArrCtx->plotAxis = SYD_PLAX_TY;
    pArrCtx->linePlot = 1;
    pArrCtx->plotInit = 1;

    cmdInitContext(pCxCmd, "  ARR:  ");

/*-----------------------------------------------------------------------------
* help information initialization
*----------------------------------------------------------------------------*/
    helpInit(&pCxCmd->helpList);
/*-----------------------------------------------------------------------------
* help info--generic commands
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCxCmd->helpList, &pCxCmd->helpCmds, "commands", "\n\
Generic commands are:\n\
   dataOut    [filePath]     (default is to normal output)\n\
   bg\n\
   help       [topic]\n\
   quit (or ^D)\n\
   source     filePath\n\
");
/*-----------------------------------------------------------------------------
* help info--bg command
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCxCmd->helpList, &pCxCmd->helpBg, "bg", "\n\
The bg command under vxWorks allows the ARR process to continue\n\
running without accepting commands from the keyboard.  This allows the\n\
vxWorks shell to be used for other purposes without the need to\n\
terminate ARR.\n\
\n\
Under SunOS, no bg command is directly available.  Instead, if ARR\n\
is being run from the C shell (csh), it can be detached using several\n\
steps from the keyboard (with the first % on each line being the prompt\n\
from csh):\n\
	type ^Z, then type\n\
	% bg %ARR\n\
\n\
To move cau to the foreground, type\n\
	% fg %ARR\n\
");
/*-----------------------------------------------------------------------------
*	export command
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCxCmd->helpList, &helpExport, "export", "\n\
The `export' command serves the dual function of acquiring some samples\n\
and then \"exporting\" them into a desired format.  If `nSamples' is\n\
omitted, then the most recently acquired set of samples is exported.\n\
\n\
The dataOut command can be used to direct the output from the export command\n\
to a specified file.\n\
\n\
    export      [nSamples],option\n\
\n\
If `nSamples' is omitted, then the most recently acquired set of samples\n\
is exported.\n\
\n\
Valid options are:\n\
    ss		format for reading into a spreadsheet\n\
    ssStat	same as \"ss\", but with a status code preceding each value\n\
                (use \"help output\" to get a list of status codes)\n\
");
/*-----------------------------------------------------------------------------
* help info--ARR-specific commands
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCxCmd->helpList, &pCxCmd->helpCmdsSpec, "commands", "\n\
ARR-specific commands are:\n\
  byChannel       [filePath]\n\
  channel[,fill]  chanName [chanName ...]\n\
  delete          all  or  chanName [chanName ...]\n\
 *export,type     [nSamples]    type: {ss, ssStat}\n\
  fileInfo\n\
  ignorePartial\n\
  plot[,type]     [nSamples]    type: [p,m][t,x]{y,yy}[s]  e.g. ty or pxyy\n\
  plotEPS[,type]  [filePath]\n\
  plotHC[,type]   [printerName]\n\
  position        timeStamp  or  +nHours  or  -nHours\n\
 *print           [[nSamples][,ncol,colWidth]]\n\
  printHC         [[printerName][,ncol,colWidth]]\n\
  resetFilled     seconds\n\
  rewind\n\
  roundStamps     milliSec\n\
  sampleSet       filePath\n\
  skip            nSamples\n\
 *tsTest          [nSamples][,interval[,slop]]\n\
                  default for nSamples is 1000000000\n\
                  interval in floating seconds\n\
                  if interval specified, default for slop is +/- .0015 sec\n\
\n\
Output from commands flagged with * can be routed to a file by using the\n\
\"dataOut filePath\" command.  The present contents of the file are\n\
preserved, with new output being written at the end.\n\
");
/*-----------------------------------------------------------------------------
*	retrieval options
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCxCmd->helpList, &helpOut, "retrievalOptions", "\n\
Several options can be invoked for retrieving data, using special commands.\n\
\n\
    ignorePartial	omit partial samples from the retrieval.  (Not used\n\
			by the tsTest command.)\n\
    resetFilled  seconds	causes the data stream for `filled' channels\n\
			to be reset (i.e., the old value is discarded) if\n\
			the interval following a sample exceeds the specified\n\
			time\n\
    roundStamps  milliSec	causes time stamps to be rounded to the\n\
			nearest `milliSec' boundary\n\
");
/*-----------------------------------------------------------------------------
*	output options
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCxCmd->helpList, &helpOut, "output", "\n\
Several possibilities exist for output from ARR:\n\
\n\
    dataOut        [filePath]\n\
   *export,type    [nSamples],option   (use help export for more info)\n\
    plot[,type]    [nSamples]          (use help plot for more info\n\
    plotEPS[,type] [filePath]\n\
    plotHC[,type]  [printerName]\n\
   *print          [[nSamples][,ncol,colWidth]]\n\
    printHC        [[printerName][,ncol,colWidth]]\n\
\n\
The dataOut command can be used to direct the output from `export' and\n\
`print' to a file.\n\
\n\
For text output, a one or two character status code is used.  For the\n\
print commands, this status code follows the value; for the export\n\
command, it precedes the value.  The first character will be one of:\n\
    O  value is OK (i.e., it has a valid time stamp)\n\
    R  value is OK, but it restarts the \"run\" of data for the channel.\n\
       This happens, for example, when an archive request is inactivated\n\
       and then re-activated.\n\
    F  value is \"filled\" from one with an earlier time stamp\n\
    M  value is missing (i.e., no value for channel with this time stamp)\n\
The second character conveys additional information about the value.  If\n\
the second character is blank, then the value is normal.\n\
    E  end of file for channel\n\
    N  no data for channel\n\
    H  value violates high alarm limit\n\
    h  value violates high warning limit\n\
    l  value violates low warning limit\n\
    L  value violates low alarm limit\n\
    R  value has a control system read alarm\n\
    M  value has some other major alarm\n\
    m  vlaue has some other minor alarm\n\
\n\
The `print' and `plot' commands serve the dual function of acquiring some\n\
samples and then displaying them on the screen, either as text or as a\n\
plot.  If `nSamples' is omitted, then the most recently acquired set of\n\
samples is displayed.\n\
\n\
The `printHC' and `plotHC' commands prepare hard copies of the most recently\n\
acquired set of samples, in text and plot forms respectively.  (These\n\
commands can not be used to acquire samples.)  These commands write the\n\
output to a printer.  (For `plotHC', the printer must be a PostScript\n\
printer.)  Once a printer has been specified, it need not be specified\n\
again, unless a change is desired.\n\
\n\
For the `print' and `printHC' commands, `ncol' and `colWidth' control\n\
formatting.  They specify the number of text columns and the width of a\n\
column in characters.\n\
");
/*-----------------------------------------------------------------------------
*	plot command
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCxCmd->helpList, &helpPlot, "plot", "\n\
The `plot', `plotEPS', and `plotHC' commands display samples in graphical\n\
form, either on the screen or to a PostScript file or printer.  The `plot'\n\
command will also optionally acquire a new group of samples prior to\n\
plotting.\n\
\n\
  plot[,type]    [nSamples]    type: [p,m]{t,x[s]}{y,yy}[s]  e.g. ty or pxyy\n\
  plotEPS[,type] [filePath]\n\
  plotHC[,type]  [printerName]\n\
\n\
If the `,type' option isn't specified, the most recently specified plot\n\
type is used.  The default when ARR starts is `,ty'.\n\
\n\
Valid plot types are:\n\
  y	`bin number' vs y, primarily for waveforms; each channel has its\n\
	own grid\n\
  yy    `bin number' vs y, primarily for waveforms; all channels share the\n\
	same grid\n\
  ty    time vs y; each channel has its own grid\n\
  tyy   time vs y; all channels share the same grid\n\
  xy    x channel vs y channel; each y channel has its own grid.  (The\n\
        first channel is treated as the x channel; the second and\n\
        following channels are treated as y channels.)\n\
  xsy   same as xy, except the samples are sorted using the value of the\n\
	x channel as a sort key\n\
  xyy   x channel vs y channel; all y channels share the same grid.\n\
  xsyy  same as xyy, except the samples are sorted using the value of the\n\
	x channel as a sort key\n\
  p     may precede any of the plot types to produce a point plot rather\n\
        than a line plot.  For example, pty and pxyy are valid types.\n\
  m     may precede any of the plot types to produce a point plot rather\n\
        than a line plot, using a plot mark (rather than a small dot) at\n\
	each point.  For example, mty and mxyy are valid types.\n\
  s     may follow any of the plot types to show the status of points\n\
        on the plot.  For example, tys and pxyys are valid types.\n\
\n\
ARR `hangs' while a plot window is up; the plot window can be exited by\n\
selecting \"quit\" on the plot window title bar, or by clicking the right\n\
mouse button within the plot window.  ARR remembers (until it exits) the\n\
size and position of the plot window; either size and/or position may be\n\
freely changed.  The size is used by the `plotHC' command in preparing\n\
the hard copy of the plot.\n\
\n\
The `plotEPS' command stores a plot of the most recently acquired set of\n\
samples in the indicated file, erasing its previous contents, if any.\n\
Once a file has been specified, it need not be specified again, unless a\n\
change is desired.  (This command can't be used to acquire samples.)  The\n\
file is in Encapsulated PostScript form, which allows it easily to be\n\
imported into many text processors.  For example, to get an EPS file named\n\
\"plot1\" on your Interleaf desktop, use:\n\
\n\
	plotEPS /home/yourServer/yourName/desktop/plot1.eps\n\
\n\
The `plotHC' command prints a plot of the most recently acquired set of\n\
samples.  (This command can't be used to acquire samples.)  The printer\n\
must be a PostScript printer.  Once a printer has been specified, it need\n\
not be specified again, unless a change is desired.\n\
");
/*-----------------------------------------------------------------------------
*	position command
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCxCmd->helpList, &helpPos, "position", "\n\
Positioning within a file can be done with any of several commands:\n\
  position        timeStamp  or  +hh:mm:ss.msc  or  -hh:mm:ss.msc\n\
  rewind\n\
  skip            nSamples\n\
\n\
The position command can be used to move around in a file, either by\n\
specifying a specific time and date or by specifying a delta time in hours.\n\
\n\
For positioning to a time and date, the command has the form.  The fraction\n\
part of a second may be omitted, for positioning to the nearest second.  If\n\
the fractional part is specified, it must be 3 digits.\n\
  position  10/17/90  13:12:8.020\n\
\n\
For relative positioning specify a + or - sign followed by a delta time\n\
expressed as the time portion of a time stamp.\n\
  position  +1:0:0\n\
  position  -0:0:1.500\n\
");
/*-----------------------------------------------------------------------------
* help info--ARR usage information
*----------------------------------------------------------------------------*/
    helpTopicAdd(&pCxCmd->helpList, &pCxCmd->helpUsage, "usage", "\n\
ARR acquires data either from a \"by channel\" archive data file or from\n\
a \"sample set\" archive data file.\n\
\n\
Either the `byChannel' or the `sampleSet' command must be used prior to\n\
any command which retrieves data.  The `byChannel' command assumes that\n\
\"by channel\" data is to be retrieved from arChan/arChanData.bin,\n\
which is the usual file for holding \"by channel\" data.  (You can also\n\
specify an explicit path and file with the `by channel' command.)\n\
\n\
ARR keeps a list of channels for which samples are to be retrieved.\n\
Channels can be added to the list with the `channel' command and deleted\n\
from it with the `delete' command.  ARR retrieves data as synchronous\n\
samples, in which all values in the sample have the same time stamp.\n\
(Channels which change slowly or randomly can be included in a synchronous\n\
sample by using the `,fill' option on the `close' command.)\n\
\n\
Example of using ARR for \"by channel\" data files from AR.  Acquire\n\
30 synchronous samples for 4 channels, plot them on the screen, and\n\
then get a hard copy of the plot.\n\
\n\
  % cd ...          (move to directory where AR was run)\n\
  % ARR\n\
    ARR: byChannel\n\
    ARR: channel rai_2000 rai_2001\n\
    ARR: channel,fill rai_2002,rao_2000\n\
    ARR: plot 30\n\
    ARR: plotHC cx1\n\
    ARR: quit\n\
  %\n\
\n\
Example of using ARR for \"sample set\" data files from AR.  Find out\n\
what files are available, select one, and then find out what channels\n\
are available in the file.  Select 2 channels, get 40 synchronous\n\
samples and plot them to verify that the proper data has been located.\n\
Finally, write the samples to a file called `myFile.ss' in the `arSet'\n\
subdirectory for further analysis by a spreadsheet program.\n\
\n\
  % cd ...          (move to directory where AR was run)\n\
  % ls arSet/*.txt  (find out what data files are available)\n\
  % ARR\n\
    ARR: sampleSet arSet/test1.txt          (use the test1.txt data file)\n\
    ARR: fileInfo\n\
    ARR: channel rai_2000 rai_2001\n\
    ARR: channel,fill rai_2002,rao_2000\n\
    ARR: plot 30\n\
    ARR: dataOut arSet/myFile.ss\n\
    ARR: export,ss\n\
    ARR: quit\n\
  %\n\
\n\
For information on moving ARR into the background, use the \"help bg\"\n\
command.\n\
");
    return OK;
}

/*+/subr**********************************************************************
* NAME	ARRTaskCheck - check on a task
*
* DESCRIPTION
*	Check to see if a task exists (based on its name).  If it does:
*	o   verify that its status in the ARR_TASK_INFO block is OK
*	o   verify that it isn't suspended
*
*	If the task doesn't exist:
*	o   verify that its status in the ARR_TASK_INFO block is ERROR
*
* RETURNS
*	OK, or
*	ERROR if an inconsistency is found
*
* BUGS
* o	This isn't a general purpose routine.  In particular, under SunOS,
*	the assumption is that the task doesn't exist.
*
*-*/
#ifdef USETASKS
long
ARRTaskCheck(pCxCmd, name, pTaskInfo)
CX_CMD	*pCxCmd;	/* I pointer to command context */
char	*name;		/* I name of task */
ARR_TASK_INFO *pTaskInfo;/* IO pointer to task info block */
{

#ifdef vxWorks
    if (taskNameToId(name) == ERROR) {
#else
    if (1) {
#endif
	assert(pTaskInfo->status == ERROR);
	pTaskInfo->stop = 0;
	pTaskInfo->stopped = 0;
    }
#ifdef vxWorks
    else {
	assert(pTaskInfo->status == OK);
	assert(!GenTaskNull(pTaskInfo->id));
	if (taskIsSuspended(pTaskInfo->id)) {
	    (void)printf("%s is suspended\n", name);
	    return ERROR;
	}
    }
#endif
    return OK;
}
#endif

/*/subhead channel------------------------------------------------------------
* channel
*----------------------------------------------------------------------------*/
ARR_channel(pCxCmd, pSspec)
CX_CMD	*pCxCmd;
SYD_SPEC *pSspec;
{
    long	stat;
    int		syncType=SYD_SY_NONF;

    sydSampleSetFree(pSspec);

    if (pCxCmd->delim == ',') {
	if (nextANField(&pCxCmd->pLine, &pCxCmd->pField, &pCxCmd->delim) <= 1)
	    goto chanFillErr;
	if (strcmp(pCxCmd->pField, "fill") != 0)
	    goto chanFillErr;
	syncType = SYD_SY_FILLED;
    }
    if ((pCxCmd->fldLen = nextChanNameField(&pCxCmd->pLine, &pCxCmd->pField,
							&pCxCmd->delim)) <= 1)
	(void)printf("you must specify one or more channels\n");
    while (pCxCmd->fldLen > 1) {
	if (sydChanFind(pSspec, pCxCmd->pField) != NULL)
	    (void)printf("%s already selected\n", pCxCmd->pField);
	else {
	    stat = sydChanOpen(pSspec, NULL, pCxCmd->pField, syncType, NULL,0);
	    if (stat != S_syd_OK)
		(void)printf("couldn't open %s \n", pCxCmd->pField);
	    else {
		pSspec->chanCount++;
	    }
	}
	pCxCmd->fldLen = nextChanNameField(&pCxCmd->pLine, &pCxCmd->pField,
								&pCxCmd->delim);
    }
    return;
chanFillErr:
    (void)printf("syntax error on option\n");
}

/*/subhead delete-------------------------------------------------------------
* delete
*----------------------------------------------------------------------------*/
ARR_delete(pCxCmd, pSspec)
CX_CMD *pCxCmd;
SYD_SPEC *pSspec;
{
    SYD_CHAN	*pSChan, *pSChanNext;

    pCxCmd->fldLen = nextChanNameField(&pCxCmd->pLine, &pCxCmd->pField,
							    &pCxCmd->delim);
    if (pCxCmd->fldLen <= 1) {
	(void)printf(
		"you must specify one or more channels or else specify all\n");
	return;
    }
    if (strcmp(pCxCmd->pField, "all") == 0) {
	pSChan = pSspec->pChanHead;
	while (pSChan != NULL) {
	    pSChanNext = pSChan->pNext;
	    if (sydChanClose(pSspec, pSChan) != S_syd_OK) {
		(void)printf("error deleting %s\n", pCxCmd->pField);
	    }
	    pSChan = pSChanNext;
	}
	return;
    }
    while (pCxCmd->fldLen > 1) {
	if ((pSChan = sydChanFind(pSspec, pCxCmd->pField)) == NULL)
	    (void)printf("%s not selected\n", pCxCmd->pField);
	else {
	    if (sydChanClose(pSspec, pSChan) != S_syd_OK) {
		(void)printf("error deleting %s\n", pCxCmd->pField);
	    }
	}
	pCxCmd->fldLen = nextChanNameField(&pCxCmd->pLine, &pCxCmd->pField,
								&pCxCmd->delim);
    }
}

/*/subhead export-------------------------------------------------------------
* export
*----------------------------------------------------------------------------*/
ARR_export(pCxCmd, pSspec)
CX_CMD *pCxCmd;
SYD_SPEC *pSspec;
{
    int		reqCount;
    int		option;

    if (pCxCmd->delim != ',' ||
        nextANField(&pCxCmd->pLine, &pCxCmd->pField, &pCxCmd->delim) <= 1) {
	(void)printf("export option must be specified\n");
	return;
    }
    if (strcmp(pCxCmd->pField, "ss") == 0)
	option = 1;
    else if (strcmp(pCxCmd->pField, "ssStat") == 0)
	option = 2;
    else {
	(void)printf("illegal export option\n");
	return;
    }
    if (nextIntFieldAsInt(&pCxCmd->pLine, &reqCount, &pCxCmd->delim) > 1) {
	if (reqCount > 10000) {
	    (void)printf("sample count must be <= 10000\n");
	    return;
	}
    }
    else
	reqCount = 0;


    if (reqCount > 0)
	ARR_sample(pCxCmd, pSspec, reqCount);

    if (pSspec->sampleCount <= 0)
	(void)printf("no data to export\n");
    else
	sydSampleSetExport(pSspec, pCxCmd->dataOut, option);
}

/*/subhead plot---------------------------------------------------------------
* plot
*----------------------------------------------------------------------------*/
ARR_plot(pCxCmd, pArrCtx, pSspec, cmdType)
CX_CMD	*pCxCmd;
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
SYD_SPEC *pSspec;
ARR_P_TYPE cmdType;
{
    int		reqCount;
    char	delim;
    char	*pPlotDispName;
    char	*pPlotFile;
    int		i;

    if (pCxCmd->delim == ',') {
	if (nextANField(&pCxCmd->pLine, &pCxCmd->pField, &pCxCmd->delim) <= 1)
	    goto ARR_plotTyErr;
	pArrCtx->linePlot = 1;
	pArrCtx->pointPlot = 0;
	pArrCtx->markPlot = 0;
	if (*pCxCmd->pField == 'p') {
	    pArrCtx->pointPlot = 1;
	    pArrCtx->linePlot = 0;
	    pCxCmd->pField++;
	}
	else if (*pCxCmd->pField == 'm') {
	    pArrCtx->markPlot = 1;
	    pArrCtx->linePlot = 0;
	    pCxCmd->pField++;
	}
	if (strncmp(pCxCmd->pField, "tyy", 3) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_TYY;
	    pCxCmd->pField += 3;
	}
	else if (strncmp(pCxCmd->pField, "ty", 2) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_TY;
	    pCxCmd->pField += 2;
	}
	else if (strncmp(pCxCmd->pField, "xyy", 3) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_XYY;
	    pCxCmd->pField += 3;
	}
	else if (strncmp(pCxCmd->pField, "xy", 2) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_XY;
	    pCxCmd->pField += 2;
	}
	else if (strncmp(pCxCmd->pField, "yy", 2) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_YY;
	    pCxCmd->pField += 2;
	}
	else if (strncmp(pCxCmd->pField, "y", 1) == 0) {
	    if (pSspec->chanCount <= 1)
		goto ARR_plotNErr;
	    pArrCtx->plotAxis = SYD_PLAX_Y;
	    pCxCmd->pField += 1;
	}
	else
	    goto ARR_plotTyErr;
	if (*pCxCmd->pField == 's') {
	    pArrCtx->showStat = 1;
	    pCxCmd->pField++;
	}
	else
	    pArrCtx->showStat = 0;
	if (*pCxCmd->pField != '\0')
	    goto ARR_plotTyErr;
    }
    if (cmdType == ARR_P_PLAIN) {
	if (pCxCmd->delim != '\n') {
	    if (nextIntFieldAsInt(&pCxCmd->pLine, &reqCount, &delim) > 1) {
		if (reqCount > 10000) {
		    (void)printf("sample count must be <= 10000\n");
		    return;
		}
		ARR_sample(pCxCmd, pSspec, reqCount);
	    }
	    else {
		(void)printf("syntax error\n");
		return;
	    }
	}
	pPlotFile = NULL;
	pPlotDispName = NULL;
    }
    else if (cmdType == ARR_P_EPS) {
	if (nextNonSpaceField(&pCxCmd->pLine, &pPlotFile, &delim) <= 1)
	    pPlotFile = NULL;

	if (pPlotFile == NULL) {
	    if (pArrCtx->plotEPSfile[0] == '\0') {
		(void)printf("An EPS file hasn't previously been specified.\n");
		(void)printf("Use the command again, specifying a filePath\n");
		return;
	    }
	    pPlotFile = pArrCtx->plotEPSfile;
	}
	else if (strlen(pPlotFile) >= 120) {
	    (void)printf("plot file name too long\n");
	    return;
	}
	else if (sscanf(pPlotFile, "%d", &i) > 0) {
	    (void)printf("plot file name must contain at least 1 letter\n");
	    return;
	}
	else
	    (void)strcpy(pArrCtx->plotEPSfile, pPlotFile);
	pPlotDispName = NULL;
    }
    else if (cmdType == ARR_P_HC) {
	if (nextNonSpaceField(&pCxCmd->pLine, &pPlotDispName, &delim) <= 1)
	    pPlotDispName = NULL;
	(void)sprintf(pArrCtx->printerFile, "ARR%d.HC", getpid());
	pPlotFile = pArrCtx->printerFile;

	if (pPlotDispName == NULL) {
	    if (pArrCtx->printer[0] == '\0')
		goto plotHC_error;
	    pPlotDispName = pArrCtx->printer;
	}
	else if (strlen(pPlotDispName) >= 80) {
	    (void)printf("printer name too long\n");
	    return;
	}
	else if (sscanf(pPlotDispName, "%d", &i) > 0) {
	    (void)printf("printer name must contain at least 1 letter\n");
	    return;
	}
	else
	    (void)strcpy(pArrCtx->printer, pPlotDispName);
    }

    if (pSspec->sampleCount <= 0)
	(void)printf("no data to plot\n");
    else {
	ARR_samplePlot(pCxCmd, pArrCtx, pSspec, cmdType,
						pPlotDispName, pPlotFile);
    }
    return;
ARR_plotNErr:
    (void)printf("need more than 1 channel to use this type\n");
    return;
ARR_plotTyErr:
    (void)printf("error in plot type\n");
    return;
plotHC_error:
    (void)printf("\
a printer name is needed; the first time you use an xxxxHC command,\n\
you must specify one.  Use the form:\n\
	xxxxHC printerName\n");
}

/*/subhead position-----------------------------------------------------------
* position
*----------------------------------------------------------------------------*/
ARR_position(pCxCmd, pSspec)
CX_CMD *pCxCmd;
SYD_SPEC *pSspec;
{
    TS_STAMP	posStamp;
    double	deltaTime;
    int		stat;
    SYD_CHAN	*pSChan;

    (void)nextNonSpace(&pCxCmd->pLine);
    if (*pCxCmd->pLine == '+') {
	if (tsTimeTextToStamp(&posStamp, &pCxCmd->pLine) != S_ts_OK)
	    goto posErr;
	if (pSspec->sampleTs.secPastEpoch == 0)
	    (void)sydInputGet(pSspec, NULL);
	TsAddStamps(&posStamp, &pSspec->sampleTs, &posStamp);
    }
    else if (*pCxCmd->pLine == '-') {
	if (tsTimeTextToStamp(&posStamp, &pCxCmd->pLine) != S_ts_OK)
	    goto posErr;
	if (pSspec->sampleTs.secPastEpoch == 0)
	    (void)sydInputGet(pSspec, NULL);
	TsDiffAsStamp(&posStamp, &pSspec->sampleTs, &posStamp);
    }
    else if (tsTextToStamp(&posStamp, &pCxCmd->pLine) == S_ts_OK)
	;		/* no additional action here */
    else
	goto posErr;

    sydPosition(pSspec, &posStamp);
    return;
posErr:
    (void)printf("error in time specification\n");
}

/*/subhead print[HC]----------------------------------------------------------
* print
*----------------------------------------------------------------------------*/
ARR_print(pCxCmd, pSspec)
CX_CMD *pCxCmd;
SYD_SPEC *pSspec;
{
    int		reqCount;
    int		ncol, colWidth;

    if (nextIntFieldAsInt(&pCxCmd->pLine, &reqCount, &pCxCmd->delim) > 1) {
	if (reqCount > 10000) {
	    (void)printf("sample count must be <= 10000\n");
	    return;
	}
	ARR_sample(pCxCmd, pSspec, reqCount);
    }
    if (pCxCmd->delim == ',') {
	if (nextIntFieldAsInt(&pCxCmd->pLine, &ncol, &pCxCmd->delim) <= 1) {
	    (void)printf("illegal number of columns\n");
	    return;
	}
	if (ncol < 1 || ncol > 100) {
	    (void)printf("illegal number of columns\n");
	    return;
	}
	pglArrCtx->ncol = ncol;
    }
    if (pCxCmd->delim == ',') {
	if (nextIntFieldAsInt(&pCxCmd->pLine,&colWidth,&pCxCmd->delim) <= 1) {
	    (void)printf("illegal column width\n");
	    return;
	}
	if (ncol < 2 || ncol > 20) {
	    (void)printf("illegal column width\n");
	    return;
	}
	pglArrCtx->colWidth = colWidth;
    }

    if (pSspec->sampleCount <= 0)
	(void)printf("no data to print\n");
    else
	ARR_samplePrint(pCxCmd, pSspec, NULL, NULL);
}
ARR_printHC(pCxCmd, pArrCtx, pSspec)
CX_CMD	*pCxCmd;
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
SYD_SPEC *pSspec;
{
    char	*pPrinter;
    int		i;
    int		ncol, colWidth;

    if (nextANField(&pCxCmd->pLine, &pPrinter, &pCxCmd->delim) <= 1)
	pPrinter = NULL;
    if (pArrCtx->printerFile[0] == '\0')
	(void)sprintf(pArrCtx->printerFile, "ARR%d.HC", getpid());

    if (pPrinter == NULL) {
	if (pArrCtx->printer[0] == '\0')
	    goto printHC_error;
    }
    else if (strlen(pPrinter) >= 80) {
	(void)printf("printer name too long\n");
	return;
    }
    else if (sscanf(pPrinter, "%d", &i) > 0) {
	(void)printf("printer name must contain at least 1 letter\n");
	return;
    }
    else
	(void)strcpy(pArrCtx->printer, pPrinter);

    if (pCxCmd->delim == ',') {
	if (nextIntFieldAsInt(&pCxCmd->pLine, &ncol, &pCxCmd->delim) <= 1) {
	    (void)printf("illegal number of columns\n");
	    return;
	}
	if (ncol < 1 || ncol > 100) {
	    (void)printf("illegal number of columns\n");
	    return;
	}
	pglArrCtx->ncol = ncol;
    }
    if (pCxCmd->delim == ',') {
	if (nextIntFieldAsInt(&pCxCmd->pLine,&colWidth,&pCxCmd->delim) <= 1) {
	    (void)printf("illegal column width\n");
	    return;
	}
	if (ncol < 2 || ncol > 20) {
	    (void)printf("illegal column width\n");
	    return;
	}
	pglArrCtx->colWidth = colWidth;
    }

    ARR_samplePrint(pCxCmd, pSspec, pArrCtx->printer, pArrCtx->printerFile);
    return;
printHC_error:
    (void)printf("\
a printer name is needed; the first time you use an xxxxHC command,\n\
you must specify one.  Use the form:\n\
	xxxxHC printerName\n");
}

/*/subhead skip---------------------------------------------------------------
* skip
*----------------------------------------------------------------------------*/
ARR_skip(pCxCmd, pSspec)
CX_CMD *pCxCmd;
SYD_SPEC *pSspec;
{
    int		reqCount;

    if (nextIntFieldAsInt(&pCxCmd->pLine, &reqCount, &pCxCmd->delim) > 1) {
	if (reqCount > 10000) {
	    (void)printf("sample count must be <= 10000\n");
	    return;
	}
	ARR_sample(pCxCmd, pSspec, reqCount);
    }
}

/*/subhead tsTest-------------------------------------------------------------
* tsTest
*----------------------------------------------------------------------------*/
ARR_tsTest(pCxCmd, pSspec)
CX_CMD	*pCxCmd;
SYD_SPEC *pSspec;
{
    double	intervalI, deltaI, diff, delta;
    char	stamp[28];
    int		errMode, i, stat, hdrFlag;
    FILE	*out;
    int		formatFlag;
    int		reqCount;

    out = pCxCmd->dataOut;
    formatFlag = 0;

    if (pCxCmd->delim == ',') {
	(void)printf("usage: tsTest [nSamples][,interval[,jitter]]\n");
	return;
    }
    if ((stat = sydSampleSetAlloc(pSspec, 1)) != S_syd_OK) {
	return;
    }

    if (nextIntFieldAsInt(&pCxCmd->pLine, &reqCount, &pCxCmd->delim) < 2)
	reqCount = 1000000000;
    intervalI = 0.;
    deltaI = .0015;
    if (pCxCmd->delim == ',') {
	if (nextFltFieldAsDbl(&pCxCmd->pLine, &intervalI, &pCxCmd->delim) < 2)
	    intervalI = 0.;
	else if (pCxCmd->delim == ',') {
	    if (nextFltFieldAsDbl(&pCxCmd->pLine, &deltaI, &pCxCmd->delim) < 2)
		deltaI = .0015;
	}
    }
    (void)printf("tsTest; looking for coincidence violations\n");
    if (intervalI > 0.) {
	(void)printf("tsTest; also checking interval of %.3f sec +/- %.4f\n",
	    intervalI, deltaI);
    }
    i = 0;
    errMode = 0;
    hdrFlag = 1;
    stat = S_syd_OK;
    while (stat != S_syd_EOF && i++ < reqCount) {
	delta = deltaI;
	stat = sydSampleSetGet(pSspec);
	reqCount++;
	if (i > 1 && stat != S_syd_EOF && intervalI > 0.) {
	    TsDiffAsDouble(&diff, &pSspec->sampleTs, &pSspec->priorTs);
	    delta = diff - intervalI;
	    if (delta < 0.)
		delta = -delta;
	    if (delta > deltaI) {
		(void)printf("  %s  interval to next sample is %.3f\n",
		    tsStampToText(&pSspec->priorTs, TS_TEXT_MMDDYY, stamp),
		    diff);
	    }
	}
	if (stat == S_syd_partial || delta > deltaI || i == 1) {
	    if (stat == S_syd_partial && !errMode) {
		(void)printf("  %s  prior to partial sample\n",
		    tsStampToText(&pSspec->priorTs, TS_TEXT_MMDDYY, stamp));
	    }
	    (void)sydSamplePrint(pSspec, out, formatFlag, hdrFlag, 6, 10, 0);
	    errMode = 1;
	    hdrFlag = 0;
	}
	else if (pSspec->eofCount > 0 || stat == S_syd_EOF) {
	    stat = S_syd_EOF;
	    (void)printf("tsTest done due to EOF\n");
	    (void)sydSamplePrint(pSspec, out, formatFlag, hdrFlag, 6, 10, 0);
	    hdrFlag = 0;
	}
	else if (i >= reqCount) {
	    (void)printf("tsTest done due to count satisfied\n");
	    (void)sydSamplePrint(pSspec, out, formatFlag, hdrFlag, 6, 10, 0);
	    hdrFlag = 0;
	}
	else if (errMode) {
	    if (pSspec->eofCount == 0) {
		(void)printf("  %s  back on track\n",
		    tsStampToText(&pSspec->sampleTs, TS_TEXT_MMDDYY, stamp));
	    }
	    errMode = 0;
	}
    }
}

/*/subhead sample-------------------------------------------------------------
* sample
*----------------------------------------------------------------------------*/
ARR_sample(pCxCmd, pSspec, reqCount)
CX_CMD	*pCxCmd;
SYD_SPEC *pSspec;
int	reqCount;
{
    long	stat;
    int 	done=0;
    double	diff;
    
/*-----------------------------------------------------------------------------
*	start time fresh for all channels
*----------------------------------------------------------------------------*/
    sydInputResetSampled(pSspec);
/*----------------------------------------------------------------------------
*	reserve memory for the various sample storage arrays if there is
*	no storage or its size has increased
*----------------------------------------------------------------------------*/
    if (sydSampleSetAlloc(pSspec, reqCount) != S_syd_OK)
	return ERROR;
/*----------------------------------------------------------------------------
*	now, gather the data
*----------------------------------------------------------------------------*/
    while (!done) {
	stat = sydInputGet(pSspec, NULL);
	if (pglArrCtx->deadRst == 1) {
	    if (pglArrCtx->deadRefTs.secPastEpoch == 0)
		pglArrCtx->deadRefTs = SydInputTs(pSspec);
	    else {
		TsDiffAsDouble(&diff,&SydInputTs(pSspec),&pglArrCtx->deadRefTs);
		if (diff > pglArrCtx->deadRstSec) {
		    sydInputResetSampled(pSspec);
		    stat = sydInputGet(pSspec, NULL);
		}
		pglArrCtx->deadRefTs = SydInputTs(pSspec);
	    }
	}
	if (stat == S_syd_OK || stat == S_syd_partial) {
	    sydInputStoreInSet(pSspec, pglArrCtx->ignorePartial);
	    if (pSspec->sampleCount >= pSspec->reqCount)
		done = 1;
	}
	else
	    done = 1;
    }
}

/*/subhead samplePlot---------------------------------------------------------
* sample plot
*----------------------------------------------------------------------------*/
ARR_samplePlot(pCxCmd, pArrCtx, pSspec, cmdType, printerName, fileName)
CX_CMD	*pCxCmd;
ARR_CTX *pArrCtx;	/* IO pointer to ARR context */
SYD_SPEC *pSspec;
ARR_P_TYPE cmdType;
char	*printerName;
char	*fileName;
{
    int		iChan;		/* index to step through channels */
    SYD_PL_SLAVE	*pSlave;
    long	stat;
    SYD_CHAN	*pSChan;
    TS_STAMP	now;
    char	nowText[32], begText[28], endText[28];
    char	title[120];
    char	command[160];
    char	resultBuf[80];

    (void)tsLocalTime(&now);
    (void)tsStampToText(&now, TS_TEXT_MONDDYYYY, nowText);
    (void)tsStampToText(&pSspec->pTimeStamp[pSspec->firstData], TS_TEXT_MMDDYY,
								begText);
    (void)tsStampToText(&pSspec->pTimeStamp[pSspec->lastData], TS_TEXT_MMDDYY,
								endText);
    (void)sprintf(title, "samples for %s through %s", begText, endText);

    if (cmdType == ARR_P_PLAIN) {
	stat = sydPlotInit(&pArrCtx->master, pSspec,
			PPR_WIN_SCREEN, NULL, "ARR plot",
			pArrCtx->plotInit);
    }
    else if (cmdType == ARR_P_EPS) {
	assert(fileName != NULL);
	stat = sydPlotInit(&pArrCtx->master, pSspec,
			PPR_WIN_EPS, fileName, NULL,
			pArrCtx->plotInit);
    }
    else {
	assert(fileName != NULL);
	stat = sydPlotInit(&pArrCtx->master, pSspec,
			PPR_WIN_POSTSCRIPT, fileName, NULL,
			pArrCtx->plotInit);
    }
    if (stat == S_syd_OK)
	stat = sydPlotSetTitles(&pArrCtx->master, title, NULL, NULL, NULL);
    if (stat == S_syd_OK) {
	pArrCtx->plotInit = 0;
	pArrCtx->master.plotAxis = pArrCtx->plotAxis;
	pArrCtx->master.linePlot = pArrCtx->linePlot;
	pArrCtx->master.pointPlot = pArrCtx->pointPlot;
	pArrCtx->master.markPlot = pArrCtx->markPlot;
	pArrCtx->master.showStat = pArrCtx->showStat;
	iChan = 0;
	pSChan = pSspec->pChanHead;
	while (pSChan != NULL) {
	    assert(iChan < pSspec->chanCount);
	    pSlave = sydPlotChanAdd(&pArrCtx->master, pSChan);
	    assert(pSlave != NULL);
	    iChan++;
	    pSChan = pSChan->pNext;
	}

	stat = sydPlotWinLoop(&pArrCtx->master);
	assert(stat == S_syd_OK);

	stat = sydPlotDone(&pArrCtx->master, 1);
	assert(stat == S_syd_OK);

	if (cmdType == ARR_P_HC) {
	    (void)sprintf(command, "lpr -P%s %s", printerName, fileName);
	    genShellCommand(command, resultBuf, 80);
	    if (resultBuf[0] != '\0')
		(void)printf("%s\n", resultBuf);
	    unlink(fileName);
	}
    }
}

/*/subhead samplePrint--------------------------------------------------------
* samplePrint
*----------------------------------------------------------------------------*/
ARR_samplePrint(pCxCmd, pSspec, printerName, fileName)
CX_CMD *pCxCmd;
SYD_SPEC *pSspec;
char	*printerName;
char	*fileName;
{
    long	stat;
    FILE	*out;
    char	command[160];
    char	resultBuf[80];
    int		fmtFlag=0;

    if (printerName != NULL) {
	assert(fileName != NULL);
	if ((out = fopen(fileName, "w")) == NULL) {
	    (void)printf("error opening %s\n", fileName);
	    perror(NULL);
	    return;
	}
    }
    else
	out = pCxCmd->dataOut;

    if (out != stdout)
	fmtFlag = 1;
    stat = sydSampleSetPrint(pSspec, out, fmtFlag,
		pglArrCtx->ncol, pglArrCtx->colWidth);

    if (printerName != NULL) {
	fclose(out);
	(void)sprintf(command, "lpr -P%s %s", printerName, fileName);
	genShellCommand(command, resultBuf, 80);
	if (resultBuf[0] != '\0')
	    (void)printf("%s\n", resultBuf);
	unlink(fileName);
    }
}
