/*	@(#)ARR_util.c	1.3 10/16/91
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	02-08-91	rac	initial version
 * .02	07-31-91	rac	installed in SCCS
 * .03	10-15-91	rac	initialize printer with default name
 * ARR_util.c,v
 * Revision 1.2  1994/05/06 19:05:38  mrk
 * conversion from sccs to cvs
 *
 *
 * make options
 *	-DXWINDOWS	to use xview/X Window System
 */
/*+/mod***********************************************************************
* TITLE	ARR_util.c - utility routines for ARR
*
* DESCRIPTION
*
*-***************************************************************************/

#include <stdio.h>

#if defined XWINDOWS
#   include <xview/xview.h>
#   include <xview/frame.h>
#   include <xview/panel.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <sydDefs.h>

#include "ARR.h"


/*/subhead HC------------------------------------------------------------------
*
* Hard Copy routines
*
*	These routines handle getting a printer name for handling Hard
*	Copy commands.  Initialization creates a command frame which is
*	used to get a printer name from the keyboard.  Routines associated
*	with the command frame install the printer name into the ARR_CTX
*	structure, and also fuss with the menus and menu items associated
*	with Hard Copy output.
*
*  HCnameInitCF initializes the command frame and panel; call as follows:
*
*		HCnameInitCF(pCtx);
*
*	When the Commit button is pressed (or \n is typed):
*	o   the printer name is installed in pCtx->HCname
*
*----------------------------------------------------------------------------*/
static void HCnameCancel_pb(item)
Panel_item item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, PCTX);
    xv_set(pCtx->HCname_PT, PANEL_VALUE, pCtx->HCname, NULL);
    xv_set((Frame)xv_get(item, XV_KEY_DATA, GUI_CF), XV_SHOW, FALSE, NULL);
}

static void HCnameInstall_pb(item)
Panel_item item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, PCTX);
    xv_set(pCtx->HCname_CF, XV_SHOW, FALSE, NULL);
    strcpy(pCtx->HCname, (char *)xv_get(pCtx->HCname_PT, PANEL_VALUE));
}
static Panel_setting HCnameInstall_ps(item)
Panel_item	item;
{   HCnameInstall_pb(item);
    return PANEL_NONE;
}

void			/* initialize command frame and panel */
HCnameInitCF(pCtx)
ARR_CTX	*pCtx;
{
    Panel	panel;
    int		x=5, y=5, ht=0;
    Panel_item	item1, item2;

/*-----------------------------------------------------------------------------
*    ARR: Pick printer
*
* Printer name _________
*
*    Cancel   Commit
*
*----------------------------------------------------------------------------*/
    pCtx->HCname_CF =
	guiFrameCmd("ARR: Pick printer", pCtx->guiCtx.baseFrame, &panel, NULL);

    pCtx->HCname_PT = guiTextField("Printer name", panel, &x, &y, &ht,
		HCnameInstall_ps, 10, 10, PCTX, pCtx, 0, NULL);
    y += ht + 20; x = 5; ht = 0;
    item1 = guiButton("Cancel", panel, &x, &y, &ht, HCnameCancel_pb, NULL,
		PCTX, pCtx, GUI_CF, pCtx->HCname_CF);
    item2 = guiButton("Commit", panel, &x, &y, &ht, HCnameInstall_pb, NULL,
		PCTX, pCtx, GUI_CF, pCtx->HCname_CF);

    strcpy(pCtx->HCname, pCtx->dfltPrinter);
    xv_set(pCtx->HCname_PT, PANEL_VALUE, pCtx->HCname, NULL);

    window_fit(panel);
    guiButtonCenter(item1, item2, panel);
    window_fit(pCtx->HCname_CF);
}
