/*	@(#)PR.c	1.3 2/23/93
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991-93, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 *  .01 02-08-91 rac	initial version
 *  .02 07-31-91 rac	installed in SCCS
 *  .03 11-18-91 rac	use modified call to guiShellCmd; use
 *			modified call to sydSampleSetPrint; allow
 *			specifying number of columns and column
 *			width; consolidate setup functions onto the
 *			print display; allow specifying font size
 *			and page orientation
 *  .04 12-08-91 rac	change point size in textsw; change default
 *			to 8 pt., portrait;
 *  .05 01-21-92 rac	put temp files in /tmp
 *  .06 04-07-92 rac	assume make for XVIEW; remove resize code (and use
 *			generic resize capabilities); remove dependence
 *			on ARR context block; cloned from ARR_PR.c; handle
 *			the "useStats" flag; discontinue use of special malloc
 *			routines
 *  .07 01-25-93 rac	handle the "useMeans" flag
 * PR.c,v
 * Revision 1.2  1994/05/06 19:05:55  mrk
 * conversion from sccs to cvs
 *
 *
 */
/*+/mod***********************************************************************
* TITLE	PR.c - print routines for ARR and AR
*
* DESCRIPTION
*
*-***************************************************************************/

#include <stdio.h>

#define XVIEW
#if defined XVIEW
#   include <xview/xview.h>
#   include <xview/font.h>
#   include <xview/frame.h>
#   include <xview/icon.h>
#   include <xview/notice.h>
#   include <xview/panel.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <sydDefs.h>
#include <db_access.h>
#include <enumShared.h>
#include <guiSubr.h>

#include "PR.h"


/*+/subr**********************************************************************
* NAME	PRwinDel
*-*/
static void PRwinDel(pPrCtx)
PR_CTX	*pPrCtx;
{
    textsw_erase(pPrCtx->PRwinTextsw, 0, TEXTSW_INFINITY);
    textsw_reset(pPrCtx->PRwinTextsw, 0, TEXTSW_INFINITY);
    unlink(pPrCtx->PRwinName);
}

/*+/subr**********************************************************************
* NAME	PRint_xvo
*
* DESCRIPTION
*	Handle a "print" button push.  The item is expected to be set up
*	with:
*	    PRCTX, pPrtCtx	where pPrtCtx is a PR_CTX *
*	    PRTYPE, type	where type is a PR_TYPE
*
*-*/
void PRint_xvo();
static void PRint_mi(menu, item) Menu menu;Menu_item item;{PRint_xvo(item);}
static void PRint_xvo(item)
Xv_opaque item;
{   PR_CTX	*pPrCtx=(PR_CTX *)xv_get(item, XV_KEY_DATA, PRCTX);
    PR_TYPE	cmdType=(PR_TYPE)xv_get(item, XV_KEY_DATA, PRTYPE);
    SYD_SPEC	*pSspec=*pPrCtx->ppSspec;
    GUI_PRT	*pPrt=pPrCtx->pGuiPrt;
    FILE	*f;
    char	command[200];
    int		point;
    char	HCpoint[8];

    if (xv_get(pPrCtx->PRwin_CF, XV_SHOW) == FALSE)
	xv_set(pPrCtx->PRwin_CF, XV_SHOW,TRUE, FRAME_CMD_PUSHPIN_IN,TRUE, NULL);

    if (pSspec == NULL || pSspec->chanCount <= 0) {
	if (cmdType == PR_NULL)
	    return;
	guiNotice(pPrCtx->pGuiCtx, "no channels to print");
	return;
    }
    if (pSspec->sampleCount <= 0) {
	if (cmdType == PR_NULL)
	    return;
	guiNotice(pPrCtx->pGuiCtx, "no samples to print");
	return;
    }

    sscanf((char *)xv_get(pPrCtx->colCount_PT, PANEL_VALUE), "%d",
			&pPrCtx->colCount);
    sscanf((char *)xv_get(pPrCtx->colWidth_PT, PANEL_VALUE), "%d",
			&pPrCtx->colWidth);
    if (pPrCtx->colWidth < 2)
        pPrCtx->colWidth = 2;
    point = 10;
    sscanf((char *)xv_get(pPrt->point_PT, PANEL_VALUE), "%d", &point);
    if (point < 5)
	point = 5;
    else if(point > 12)
	point = 12;
    sprintf(HCpoint, "%d", point);
    xv_set(pPrt->point_PT, PANEL_VALUE, HCpoint, NULL);
    if (cmdType == PR_PLAIN || cmdType == PR_NULL) {
	PRwinDel(pPrCtx);
	if ((f = fopen(pPrCtx->PRwinName, "a")) == NULL) {
	    guiNoticeFile(pPrCtx->pGuiCtx,
			"Unable to open for output", pPrCtx->PRwinName);
	    return;
	}
	if (pPrCtx->useStats == 0 && pPrCtx->useMeans == 0)
	    sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 0, NULL);
	else {
	    if (pPrCtx->useStats == 1)
		sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 1, NULL);
	    else
		sydSetAttr(pSspec, SYD_ATTR_USE_MEANS, 1, NULL);
	    sydSampleSetStats(pSspec);
	}
	sydSampleSetPrint(pSspec, f, 0, pPrCtx->colCount, pPrCtx->colWidth);
	fclose(f);
	if (point != pPrCtx->point) {
#define FIND_FONT(f) (Xv_font)xv_find(pPrCtx->PRwin_CF,FONT,FONT_NAME,f,NULL)
	    Xv_font	font;
	    char	pt[10];
	    if (point <= 5)		{font=FIND_FONT("5x8"); point=5;}
	    else if (point <= 6)	{font=FIND_FONT("6x10"); point=6;}
	    else if (point <= 7)	{font=FIND_FONT("7x13"); point=7;}
	    else if (point <= 8)	{font=FIND_FONT("8x13"); point=8;}
	    else if (point <= 9)	{font=FIND_FONT("9x15"); point=9;}
	    else if (point <= 10)	{font=FIND_FONT("10x20"); point=10;}
	    else			{font=FIND_FONT("12x24"); point=12;}
	    if (font != NULL) {
		xv_set(pPrCtx->PRwinTextsw, TEXTSW_FONT, font, NULL);
		sprintf(pt, "%d", point);
		xv_set(pPrt->point_PT, PANEL_VALUE, pt, NULL);
		pPrCtx->point = point;
	    }
	}
	xv_set(pPrCtx->PRwinTextsw, TEXTSW_FILE, pPrCtx->PRwinName, NULL);
    }
    else if (cmdType == PR_HC) {
	if ((f = fopen(pPrCtx->HCfName, "w")) == NULL) {
	    guiNoticeFile(pPrCtx->pGuiCtx,
			"Unable to open for output", pPrCtx->HCfName);
	    return;
	}
	sydSampleSetPrint(pSspec, f, 1, pPrCtx->colCount, pPrCtx->colWidth);
	fclose(f);
	guiPrintFile(pPrt, pPrCtx->HCfName);
    }
    else if (cmdType == PR_DO) {
	char	path[2*GUI_TDIM];
	int	modeCode;

	strcpy(pPrCtx->dir, (char *)xv_get(pPrCtx->dir_PT, PANEL_VALUE));
	strcpy(pPrCtx->file, (char *)xv_get(pPrCtx->file_PT, PANEL_VALUE));
	modeCode = (int)xv_get(pPrCtx->fileMode_PCS, PANEL_VALUE);
	if (strlen(pPrCtx->file) == 0) {
	    guiNotice(pPrCtx->pGuiCtx, "no file selected");
	    return;
	}
	sprintf(path, "%s/%s", pPrCtx->dir, pPrCtx->file);
        if (modeCode == 0) {	/* append mode */
	    if ((f = fopen(path, "a")) == NULL) {
		guiNoticeFile(pPrCtx->pGuiCtx, "Unable to open for output", path);
		return;
	    }
	    fprintf(f, "\f");	/* put in a form feed */
	}
	else {			/* over-write mode */
	    if ((f = fopen(path, "w")) == NULL) {
		guiNoticeFile(pPrCtx->pGuiCtx, "Unable to open for output", path);
		return;
	    }
	}
	sydSampleSetPrint(pSspec, f, 0, pPrCtx->colCount, pPrCtx->colWidth);
	fclose(f);
    }
}

/*+/subr**********************************************************************
* NAME	PRwinInitCF - initialize the print window command frame
*
*-*/
static void PRwinInitCF(pPrCtx, title, pGuiPrt)
PR_CTX	*pPrCtx;
char	*title;
GUI_PRT	*pGuiPrt;
{
    FILE	*f;
    Panel	panel;
    Xv_opaque	item;
    Menu	menu;
    int		y=5, x=5, ht=0;

    sprintf(pPrCtx->PRwinName, "/tmp/PR%d.textsw", getpid());
    sprintf(pPrCtx->HCfName, "/tmp/PR%d.print.tmp", getpid());
    if ((f = fopen(pPrCtx->PRwinName, "w")) == NULL) {
	guiNoticeFile(pPrCtx->pGuiCtx,
	    "Unable to open temp file for printed output", pPrCtx->PRwinName);
	exit(1);
    }
    fclose(f);

/*-----------------------------------------------------------------------------
* Print v  # col: ___  col width: ___
*       to screen
*       to printer
*	printer setup...
*       to file
* Directory: _______________________
* File: _______________________  v append
*                                  over-write
* 
*----------------------------------------------------------------------------*/

    pPrCtx->PRwin_CF =
	guiFrameCmd(title, pPrCtx->pGuiCtx->baseFrame, &panel, NULL);
    menu = guiMenu(NULL, 0,NULL,0,NULL);
    guiMenuItem("to screen", menu,PRint_mi,0,1,PRCTX,pPrCtx,PRTYPE,PR_PLAIN);
    guiMenuItem("to printer", menu,PRint_mi,0,0,PRCTX,pPrCtx,PRTYPE,PR_HC);
    guiMenuItem("printer setup...", menu,guiPrinterShowCF_mi,0,0,
		GUI_PPRT,pPrCtx->pGuiPrt,0,NULL);
    guiMenuItem("to file", menu,PRint_mi,0,0,PRCTX,pPrCtx,PRTYPE,PR_DO);
    guiButton("Print", panel, &x, &y, &ht, NULL, menu, 0,NULL,0,NULL);
    pPrCtx->colCount_PT = guiTextField("# col:", panel, &x,&y,&ht,
		NULL, 3, 3, 0, NULL, 0, NULL);
    pPrCtx->colCount = 7;
    xv_set(pPrCtx->colCount_PT, PANEL_VALUE, "7", NULL);
    pPrCtx->colWidth_PT = guiTextField("col width:", panel, &x,&y,&ht,
		NULL, 2, 2, 0, NULL, 0, NULL);
    pPrCtx->colWidth = 10;
    xv_set(pPrCtx->colWidth_PT, PANEL_VALUE, "10", NULL);
    y += ht + 5; x = 5; ht = 0;

    pPrCtx->dir_PT = guiTextField("Directory:", panel, &x,&y,&ht, NULL,
		GUI_TDIM-1, 40, 0, NULL, 0, NULL);
    sprintf(pPrCtx->dir, "%s/data", pPrCtx->pGuiCtx->pwd);
    xv_set(pPrCtx->dir_PT, PANEL_VALUE, pPrCtx->dir, NULL);
    y += ht + 5; x = 5; ht = 0;

    pPrCtx->file_PT = guiTextField("File:", panel, &x,&y,&ht, NULL,
		GUI_TDIM-1, 40, 0, NULL, 0, NULL);
    strcpy(pPrCtx->file, "");
    xv_set(pPrCtx->file_PT, PANEL_VALUE, pPrCtx->file, NULL);
    pPrCtx->fileMode_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_CHOICE_STRINGS,	"append","over-write",NULL,
	PANEL_VALUE,	1,
	XV_X, x, XV_Y, y-5, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;

    y += ht + 5; x = 5; ht = 0;

    window_fit_height(panel);
    pPrCtx->PRwinTextsw =
		guiBrowser(pPrCtx->PRwin_CF, panel, 120, 40, pPrCtx->PRwinName);
    {
	Xv_font	font;
	font = (Xv_font)xv_find(pPrCtx->PRwin_CF,FONT,FONT_NAME,"8x13",NULL);
	if (font != NULL)
	    xv_set(pPrCtx->PRwinTextsw, TEXTSW_FONT, font, NULL);
    }
    window_fit(pPrCtx->PRwin_CF);
}

/*+/subr**********************************************************************
* NAME	PRinitButton - initialize print window, tied to a button
*
* DESCRIPTION
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
PRinitButton(ppPrCtx, pGuiCtx, pGuiPrt, title, panel, pX, pY, pHt, ppSspec)
PR_CTX	**ppPrCtx;
GUI_CTX	*pGuiCtx;
GUI_PRT	*pGuiPrt;
char	*title;
Panel	panel;
int	*pX, *pY, *pHt;
SYD_SPEC **ppSspec;
{
    PR_CTX *pPrCtx;

    if ((pPrCtx = (PR_CTX *)malloc(sizeof(PR_CTX))) == NULL)
	assertAlways(0);
    *ppPrCtx = pPrCtx;
    pPrCtx->pGuiCtx = pGuiCtx;
    pPrCtx->pGuiPrt = pGuiPrt;
    pPrCtx->ppSspec = ppSspec;
    pPrCtx->HCfName[0] = '\0';
    pPrCtx->useStats = 0;
    PRwinInitCF(pPrCtx, title, pGuiPrt);

    guiButton("printing...", panel, pX, pY, pHt, PRint_xvo, NULL,
			PRCTX, pPrCtx, PRTYPE, PR_NULL);
}

/*+/subr**********************************************************************
* NAME	PRinitMenu - initialize print window, tied to a menu item
*
*-*/
void
PRinitMenu(ppPrCtx, pGuiCtx, pGuiPrt, title, menu, dflt, ppSspec)
PR_CTX	**ppPrCtx;
GUI_CTX	*pGuiCtx;
GUI_PRT	*pGuiPrt;
char	*title;
Menu	menu;
int	dflt;
SYD_SPEC **ppSspec;
{
    PR_CTX *pPrCtx;

    if ((pPrCtx = (PR_CTX *)malloc(sizeof(PR_CTX))) == NULL)
	assertAlways(0);
    *ppPrCtx = pPrCtx;
    pPrCtx->pGuiCtx = pGuiCtx;
    pPrCtx->pGuiPrt = pGuiPrt;
    pPrCtx->ppSspec = ppSspec;
    pPrCtx->useStats = 0;
    PRwinInitCF(pPrCtx, title, pGuiPrt);

    guiMenuItem("printing...", menu, PRint_mi, 0, dflt,
			PRCTX, pPrCtx, PRTYPE, PR_NULL);
}

/*+/subr**********************************************************************
* NAME	PRquit - handle wrapup processing
*
*-*/
void
PRquit(pPrCtx, destroyFlag)
PR_CTX	*pPrCtx;
int	destroyFlag;
{
    unlink(pPrCtx->PRwinName);
    unlink(pPrCtx->HCfName);
    if (destroyFlag) {
	textsw_reset(pPrCtx->PRwinTextsw, 0, TEXTSW_INFINITY);
	xv_destroy_safe(pPrCtx->PRwin_CF);
    }
    free(pPrCtx);
}
