/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/*	@(#)ARR_RET.c	1.13 9/18/92
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 */
/*+/mod***********************************************************************
* TITLE	ARR_RET.c - retrieval routines for ARR
*
* DESCRIPTION
*
* BUGS
* o	stop button should be active for file retrieves
* o	aborts if mistakenly read a spreadsheet file while expecting ssf
* o	should be a "check if quit is OK" routine to call prior to calling
*	RETquit
*
*-***************************************************************************/

#include <stdio.h>
#include <string.h>

#define XVIEW
#if defined XVIEW
#   include <xview/xview.h>
#   include <xview/frame.h>
#   include <xview/icon.h>
#   include <xview/notice.h>
#   include <xview/panel.h>
#   include <xview/seln.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <sydDefs.h>
#include <db_access.h>
#include <cadef.h>

#include "enumShared.h"
#include "ARR.h"
#include "ARR_RET.h"
#include <pprPlotDefs.h>
#include <sydPlotDefs.h>
#include "PL.h"
#include "reqSubr.h"
#include "tsDefs.h"


/*+/subr**********************************************************************
* NAME	RETchoose_xvo - handle choice checkbox
*
*-*/
void RETbcf();
void RETssf();
void RETca_xvo();
static void RETchoose_xvo(item, value)
Xv_opaque item;
int	value;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    if (value < 0)
	return;
    else if (value == ARR_RET_BCF)
	RETbcf(pCtx, pRetCtx->bcfPath, pRetCtx->bcfDir, pRetCtx->bcfFile);
    else if (value == ARR_RET_SSF)
	RETssf(pCtx, pRetCtx->ssfPath, pRetCtx->ssfDir, pRetCtx->ssfFile);
    else if (value == ARR_RET_CA)
	RETca_xvo(item);
}

/*+/subr**********************************************************************
* NAME	RETbcf - install a `by channel' file for retrieval
*
*	o   all channels are deleted and the retrieval source is closed
*	    prior to opening the `by channel' file.
*	o   set the retrieval source message
*	o   get the file information into the text subwindow in CH CF
*
*-*/
static void RETbcf(pCtx, path, dir, file)
ARR_CTX	*pCtx;
char	*path, *dir, *file;	/* only `path' is used */
{   ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    long	stat;
    FILE	*f;
    char	message[400];

    guiTextswReset(pRetCtx->RETwinTextsw);
    if (pCtx->pSspec != NULL) {
	RETchan_DelAll(pCtx);
	stat = sydClose(pCtx->pSspec);
	assertAlways(stat == S_syd_OK);
	pCtx->pSspec = NULL;
    }
    
    if (file[0] != '\0') {
	stat = sydOpenCF(&pCtx->pSspec, path);
	if (stat != S_syd_OK) {
	    guiNoticeFile(&pCtx->guiCtx,
			"couldn't initialize for ", path);
	    return;
	}
	strcpy(pRetCtx->bcfFile, file);
	strcpy(pRetCtx->bcfDir, dir);
	strcpy(pRetCtx->bcfPath, path);
    }
    if (pRetCtx->bcfFile[0] != '\0') {
	sprintf(pRetCtx->RETwinName, "/tmp/ARR%d.inf.tmp", getpid());
	if ((f = fopen(pRetCtx->RETwinName, "w")) == NULL) {
	    guiNoticeFile(&pCtx->guiCtx,
		    "Unable to open the file for AR file info",
		    pRetCtx->RETwinName);
	    return;
	}

	stat = sydFileInfo(pCtx->pSspec, f);
	assertAlways(stat == S_syd_OK);
	fclose(f);

	xv_set(pRetCtx->RETwinTextsw, TEXTSW_FILE, pRetCtx->RETwinName, NULL);

	sprintf(message, "`by channel' file: %s", pRetCtx->bcfPath);
	xv_set(pRetCtx->bcfStatus_PM, PANEL_LABEL_STRING, message, NULL);
	xv_set(pRetCtx->RETuse_PCB, PANEL_VALUE, ARR_RET_BCF, NULL);
    }
    else {
	xv_set(pRetCtx->bcfStatus_PM, PANEL_LABEL_STRING, "", NULL);
	guiNotice(&pCtx->guiCtx, "no `by channel' file selected");
	xv_set(pRetCtx->RETuse_PCB, PANEL_VALUE, -1, NULL);
    }

}

/*+/subr**********************************************************************
* NAME	RETbcfResync_xvo - synchronize with `by channel' file from archiver
*
*-*/
void RETbcfResync_xvo();
static void RETbcfResync_mi(menu, item)
Menu menu; Menu_item item; { RETbcfResync_xvo(item); }
static void RETbcfResync_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    long	stat;
    char	message[200];
    FILE	*f;

    if (pCtx->pSspec != NULL) {
	if (pCtx->pSspec->type == SYD_TY_CF) {
	    stat = sydInputSync(pCtx->pSspec);
	    assertAlways(stat == S_syd_OK);
	}
	else {
	    guiNotice(&pCtx->guiCtx,
		"can't resync--retrieval isn't from `by channel' file");
	    return;
	}
    }
    if (pRetCtx->bcfFile[0] != '\0') {
	sprintf(pRetCtx->RETwinName, "/tmp/ARR%d.inf.tmp", getpid());
	if ((f = fopen(pRetCtx->RETwinName, "w")) == NULL) {
	    guiNoticeFile(&pCtx->guiCtx,
		    "Unable to open the file for AR file info",
		    pRetCtx->RETwinName);
	    return;
	}

	stat = sydFileInfo(pCtx->pSspec, f);
	assertAlways(stat == S_syd_OK);
	fclose(f);

	guiTextswReset(pRetCtx->RETwinTextsw);
	xv_set(pRetCtx->RETwinTextsw, TEXTSW_FILE, pRetCtx->RETwinName, NULL);

	sprintf(message, "`by channel' file: %s", pRetCtx->bcfPath);
	xv_set(pRetCtx->bcfStatus_PM, PANEL_LABEL_STRING, message, NULL);
	xv_set(pRetCtx->RETuse_PCB, PANEL_VALUE, ARR_RET_BCF, NULL);
    }
    else {
	xv_set(pRetCtx->bcfStatus_PM, PANEL_LABEL_STRING, "", NULL);
	guiTextswReset(pRetCtx->RETwinTextsw);
	guiNotice(&pCtx->guiCtx, "no `by channel' file selected");
	xv_set(pRetCtx->RETuse_PCB, PANEL_VALUE, -1, NULL);
    }
}

/*+/subr**********************************************************************
* NAME	RETbcfPick_xvo - handle Pick file button for `by channel' file
*
*-*/
RETbcfPick_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;

    guiFileSelect(&pCtx->guiCtx, "ARR: Pick `by channel' file",
		pRetCtx->bcfDir, pRetCtx->bcfFile, GUI_TDIM, RETbcf, pCtx);
}

/*+/subr**********************************************************************
* NAME	RETca_xvo - handle the Channel Access button
*
*	o   update the retrieval source message
*	o   clear the text subwindow of channel information
*-*/
static void RETca_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_CHAN	*pChan, *pChanNext;
    long	status;

    if (pCtx->pSspec != NULL) {
	RETchan_DelAll(pCtx);
	status = sydClose(pCtx->pSspec);
	assertAlways(status == S_syd_OK);
	pCtx->pSspec = NULL;
    }
    if (pCtx->pSspec == NULL) {
	status = sydOpenCA(&pCtx->pSspec, NULL);
	if (status != S_syd_OK) {
	    guiNotice(&pCtx->guiCtx,"couldn't initialize for Channel Access\n");
	    return;
	}
    }

    guiTextswReset(pCtx->pRetCtx->RETwinTextsw);
}

/*+/subr**********************************************************************
* NAME	RETconfigApply_xvo - set configuration from file
*
* DESCRIPTION
*	Changes to a new channel `configuration'.  The current set of channels
*	is deleted and a new set is read from the text subwindow.  Channel
*	names preceded by an `F' are added as `filled'.
*
*	If the text subwindow contains an archive request file, then the
*	sync set spec is closed and reopened to the file implied by the
*	OUTPUT: command.
*
* BUGS
* o	better reporting for syntax errors is needed
*
*-*/
static void RETconfigApply_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_RET_CTX *pRetCtx=pCtx->pRetCtx;
    int		pos;
    char	buf[120];
    char	path[GUI_TDIM*2];
    char	*pBuf, *pField, dlm;
    int		l;
    int		usingReq=0;
    enum sydChanSync sync;
    RAW_REQ	*pRaw;
    long	stat;

/*-----------------------------------------------------------------------------
*	first, find out if this is a .ret or .req file.  The .req has a
*	%endHeader% in it.
*----------------------------------------------------------------------------*/
    pos = 0;
    while (guiEditorGets(pRetCtx->pConfigEdit, buf, 120, &pos) != NULL) {
	if (strncmp(buf, "%endHeader%", 11) == 0) {
	    usingReq = 1;
	    break;
	}
    }
    if (usingReq == 0 && pCtx->pSspec == NULL) {
	guiNotice(&pCtx->guiCtx,
			"no retrieval source selected; use Retrieve from");
	guiMessagePrintf(pRetCtx->config_PM, "\"apply config\" aborted");
	return;
    }
    guiMessagePrintf(pRetCtx->config_PM, "applying config...");
    XFlush(pCtx->guiCtx.pDisp);

    RETchan_DelAll(pCtx);
    if (usingReq) {
	GUI_EDIT *pEdit=pRetCtx->pConfigEdit;
	if (pCtx->pSspec != NULL) {
	    stat = sydClose(pCtx->pSspec);
	    assertAlways(stat == S_syd_OK);
	    pCtx->pSspec = NULL;
	}    
	pRaw = rawReqAlloc();
	assertAlways(pRaw != NULL);
	strcpy(pRaw->file, (char *)xv_get(pEdit->file_PT, PANEL_VALUE));
	strcpy(pRaw->dir, (char *)xv_get(pEdit->dir_PT, PANEL_VALUE));
	pos = 0;
	while (guiEditorGets(pEdit, buf, 120, &pos) != NULL)
	    rawReqParseLine(pRaw, buf, NULL, NULL);
	rawReqCheckSemantics(pRaw, NULL, NULL);
	if (pRaw->totalErrors > 0) {
	    guiNotice(&pCtx->guiCtx, "errors encountered in .req file");
	    rawReqFree(pRaw);
	    return;
	}
	if (pRaw->sample.outputType == REQ_OUT_BCF) {
	    strcpy(path, pRaw->sample.outDir);
	    if (strlen(path) > (size_t)0)
		strcat(path, "/");
	    else
		sprintf(path, "%s/", pCtx->guiCtx.pwd);
	    strcat(path, pRaw->sample.outFile);
	    RETbcf(pCtx, path, pRaw->sample.outDir, pRaw->sample.outFile);
	}
	else if (pRaw->sample.outputType == REQ_OUT_SSF) {
	    strcpy(path, pRaw->sample.outDir);
	    if (strlen(path) > (size_t)0)
		strcat(path, "/");
	    else
		sprintf(path, "%s/", pCtx->guiCtx.pwd);
	    strcat(path, pRaw->sample.outFile);
	    RETssf(pCtx, path, pRaw->sample.outDir, pRaw->sample.outFile);
	}
	if (pCtx->pSspec != NULL) {
	    RAW_CHAN	*pRChan;
	    enum sydChanSync sync;
	    for (pRChan=pRaw->pChanHead; pRChan!=NULL; pRChan=pRChan->pNext) {
		if (pRChan->filled)
		    sync = SYD_SY_FILLED;
		else
		    sync = SYD_SY_NONF;
		RETwinAddChan(pCtx, pRChan->name, sync, 0);
	    }
	}

	rawReqFree(pRaw);
	guiMessagePrintf(pRetCtx->config_PM, "\"apply config\" complete");
	return;
    }

    pos = 0;
    while (guiEditorGets(pRetCtx->pConfigEdit, buf, 120, &pos) != NULL) {
	pBuf = buf;
	l = 9999;
	while (l > 1) {
	    sync = SYD_SY_NONF;
	    if ((l = nextNonSpaceField(&pBuf, &pField, &dlm)) > 1) {
		if (*pField != '#') {
		    if (strcmp(pField, "F") == 0) {
			sync = SYD_SY_FILLED;
			l = nextNonSpaceField(&pBuf, &pField, &dlm);
		    }
		    else if (strcmp(pField, "S") == 0)
			l = nextNonSpaceField(&pBuf, &pField, &dlm);
		    if (l > 1) {
			if (*pField != '#')
			    RETwinAddChan(pCtx, pField, sync, 0);
			else
			    l = 0;
		    }
		}
		else
		    l = 0;
	    }
	}
    }
    guiMessagePrintf(pRetCtx->config_PM, "\"apply config\" complete");
}

/*+/subr**********************************************************************
* NAME	RETconfigShow_xvo - show actual configuration
*
*-*/
static void RETconfigShow_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    GUI_EDIT	*pEdit=pCtx->pRetCtx->pConfigEdit;
    ARR_CHAN	*pChan;
    FILE	*pFile;

    if (pCtx->pChanHead == NULL) {
	guiNotice(&pCtx->guiCtx,
	    "no channels selected; use Add channel or Apply configuration");
	guiMessagePrintf(pRetCtx->config_PM, "\"show config\" aborted");
	return;
    }
    if (strcmp(pEdit->file, pRetCtx->configFile) == 0)
	guiEditorReset(pEdit);
    else if (pEdit->updFlag == 1) {
	guiNotice(&pCtx->guiCtx,
		"text has been modified; use \"Save\" or \"discard edits\"");
	guiMessagePrintf(pRetCtx->config_PM, "\"show config\" aborted");
	return;
    }
    if ((pFile = fopen(pRetCtx->configPath, "w+")) == NULL) {
	guiNoticeFile(&pCtx->guiCtx, "couldn't create temp file",
			pRetCtx->configPath);
	guiMessagePrintf(pRetCtx->config_PM, "\"show config\" aborted");
	return;
    }
    fclose(pFile);
    guiEditorLoadFile(pEdit, pRetCtx->configDir, pRetCtx->configFile);
    for (pChan=pCtx->pChanHead; pChan!=NULL; pChan=pChan->pNext)
	guiEditorPrintf(pEdit, "%s\n", pChan->name);
    guiMessagePrintf(pRetCtx->config_PM, "\"show config\" complete");
}

/*+/subr**********************************************************************
* NAME	RETconfigInitEditor - init the stored configuration file command frame
*
*-*/
static void RETconfigInitEditor(pCtx)
ARR_CTX	*pCtx;
{   ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    Panel	panel;
    int		x=5, y=5, ht=0;
    int		x1, x2, y1, y2;
    char	dir[GUI_TDIM];
    void	RETconfigExtras();

/*-----------------------------------------------------------------------------
* Dir:  ______________  File:  ________________
* guiEditor stuff...
* Show configuration  Apply configuration
* status
*----------------------------------------------------------------------------*/
    sprintf(dir, "%s/%s", pCtx->guiCtx.pwd, pCtx->dfltRet_dir);
    pRetCtx->pConfigEdit = guiEditor(&pCtx->guiCtx,
			"ARR: channel configuration", dir, NULL,
			pCtx->pGuiPrt, RETconfigExtras, pRetCtx);
}

/*+/subr**********************************************************************
* NAME	RETconfigExtras
*
*-*/
static void
RETconfigExtras(pGuiCtx, pEdit, panel, pX,pY,pHt, pRetCtx)
GUI_CTX *pGuiCtx;
GUI_EDIT *pEdit;
Panel	panel;
int	*pX, *pY, *pHt;
ARR_RET_CTX *pRetCtx;
{
    guiButton("Show configuration", panel, pX,pY,pHt,
		RETconfigShow_xvo, NULL, ARRCTX, pRetCtx->pCtx, 0, NULL);
    guiButton("Apply configuration", panel, pX,pY,pHt,
		RETconfigApply_xvo, NULL, ARRCTX, pRetCtx->pCtx, 0, NULL);
    *pY += 5 + *pHt; *pX = 5; *pHt = 0;
    pRetCtx->config_PM = guiMessage("", panel, pX,pY,pHt);
}

/*+/subr**********************************************************************
* NAME	RETstamp - translate text to a time stamp
*
*-*/
long
RETstamp(pCtx, item, pStamp)
ARR_CTX	*pCtx;
Xv_opaque item;		/* panel text item containing a stamp */
TS_STAMP *pStamp;
{
    char	stampTxt[32], *pTxt;

    strcpy(stampTxt, (char *)xv_get(item, PANEL_VALUE));
    pTxt = stampTxt;
    if (tsTextToStamp(pStamp, &pTxt) != S_ts_OK) {
	(void)notice_prompt(pCtx->guiCtx.baseFrame, NULL,
	    NOTICE_MESSAGE_STRINGS,
		"Error in time stamp.  Several forms are legal:",
		"mm/dd/yy hh:mm:ss.msc  (yy can also be 4 digits)",
		"mon dd, yyyy hh:mm:ss.msc  (3 or more letters for month)",
		"",
		"The milli-seconds can be omitted, but it must be 3 digits",
		"if it is used.  Leading zeros can be omitted, but place-",
		"holding zeros must be present.",
		"",
		"Times during the switch from daylight to standard time",
		"have a special representation, as shown by the following:",
		"1d32:33.570    (01:32:33.570 before the switch to standard)",
		"1s32:33.570    (01:32:33.570 after the switch to standard)",
		NULL,
	    NOTICE_BUTTON_YES, "Continue",
	    NULL);
	return ERROR;
    }
    return OK;
}

/*+/subr**********************************************************************
* NAME	RETtimeStamp - convert text to delta time
*
*-*/
long
RETtimeStamp(pCtx, item, pStamp)
ARR_CTX	*pCtx;
Xv_opaque item;		/* panel text item containing a stamp */
TS_STAMP *pStamp;
{
    char	stampTxt[32], *pTxt;

    strcpy(stampTxt, (char *)xv_get(item, PANEL_VALUE));
    pTxt = stampTxt;
    if (tsTimeTextToStamp(pStamp, &pTxt) != S_ts_OK) {
	(void)notice_prompt(pCtx->guiCtx.baseFrame, NULL,
	    NOTICE_MESSAGE_STRINGS,
		"Error in delta time.  The accepted form is:",
		"hh:mm:ss.msc",
		"",
		"The milli-seconds can be omitted, but it must be 3 digits",
		"if it is used.  Leading zeros can be omitted, but place-",
		"holding zeros must be present.",
		NULL,
	    NOTICE_BUTTON_YES, "Continue",
	    NULL);
	return ERROR;
    }
    return OK;
}

/*+/subr**********************************************************************
* NAME	RETbutton_mi - handle the RETRIEVE menu item
*
*	o   handle the `special case' conditions for retrieval
*	o   reset the `restricted time' button
*	o   read the number of samples and convert to an integer
*	o   change the status message to "retrieving..."
*	o   retrieve the samples
*	o   write retrieval status in the status message
*
*-*/
static void RETbutton_mi(menu, item)
Menu	menu;
Menu_item item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    SYD_SPEC	*pSspec=pCtx->pSspec;
    int		nSamp;
    long	stat;
    char	message[200];
    char	fromTxt[28], toTxt[28];
    int		from, thru, ignorePartial, ignored, acceptIf;
    TS_STAMP	stamp, throughTS, amountTS;
    double	diff;
    unsigned long round;

    xv_set(pCtx->restrict_PC, PANEL_VALUE, 0, NULL);

    if (pSspec == NULL) {
	guiNotice(&pCtx->guiCtx, "no retrieval source selected");
	guiCFshowPin(pRetCtx->RETsetup_CF);
	return;
    }
    if (pSspec->chanCount <= 0) {
	guiNotice(&pCtx->guiCtx, "no channels selected");
	guiCFshowPin(pRetCtx->RETsetup_CF);
	return;
    }

    ignorePartial = 1 + (int)xv_get(pRetCtx->RETpartial_PCB, PANEL_VALUE);
    if ((int)xv_get(pRetCtx->RETroundTs_PCB, PANEL_VALUE) == 0) {
	round = -1;
	sscanf((char *)xv_get(pRetCtx->RETroundTs_PT, PANEL_VALUE),
				"%ld", &round);
	if (round > 0)
	    pSspec->roundNsec = round * 1000000;  /* convert to nanosec */
	else {
	    if (round < 0)
		guiNotice(&pCtx->guiCtx, "illegal rounding factor--ignored");
	    pSspec->roundNsec = 0;
	}
    }
    else
	pSspec->roundNsec = 0;
    pRetCtx->RETdeadRstSec = -1.;
    pRetCtx->RETdeadRst = 1 + (int)xv_get(pRetCtx->RETdeadRst_PCB, PANEL_VALUE);
    if (pRetCtx->RETdeadRst == 1) {
	if (pSspec->type == SYD_TY_CA) {
	    /* this isn't allowed for Channel Access because a filled channel
	    which isn't changing won't EVER send us a new value */
	    guiNotice(&pCtx->guiCtx,
		"\"reset filled\" not allowed with Channel Access retrievals");
	    xv_set(pRetCtx->RETdeadRst_PCB, PANEL_VALUE, -1, NULL);
	    pRetCtx->RETdeadRst = 0;
	}
	else {
	    sscanf((char *)xv_get(pRetCtx->RETdeadRst_PT, PANEL_VALUE),
				"%lf", &pRetCtx->RETdeadRstSec);
	}
    }
    acceptIf = 1 + (int)xv_get(pRetCtx->RETaccept_PCB, PANEL_VALUE);
    if (acceptIf) {
	char	test[120];
	sydTestClose(pSspec);
	sprintf(test, "if %s",
			(char *)xv_get(pRetCtx->RETaccept_PT, PANEL_VALUE));
	if ((stat = sydTestAddFromText(pSspec, test)) != S_syd_OK) {
	    guiNotice(&pCtx->guiCtx, "illegal `accept if' condition");
	    sydTestClose(pSspec);
	    return;
	}
    }
    else
	sydTestClose(pSspec);
    from = (int)xv_get(pRetCtx->RETfrom_PCB, PANEL_VALUE);
    pRetCtx->RETfrom = from;
    thru = (int)xv_get(pRetCtx->RETthru_PCB, PANEL_VALUE);
    pRetCtx->RETthru = thru;

    if (from == ARR_RET_LAST)
	;	/* no action */
    else if (from == ARR_RET_AT) {
	stat = RETstamp(pCtx, pRetCtx->RETfromStamp_PT, &stamp);
	if (stat != OK)
	    return;
	pRetCtx->RETfromTS = stamp;
	if (pSspec->type != SYD_TY_CA) {
	    xv_set(pRetCtx->RETstatus_PM,
				PANEL_LABEL_STRING, "positioning", NULL);
	    XFlush((Display *)xv_get(pCtx->guiCtx.baseFrame, XV_DISPLAY));
	    stat = sydPosition(pSspec, &stamp);
	    if (stat == S_syd_EOF) {
		guiNotice(&pCtx->guiCtx,
			"last data in file earlier than start date/time");
		xv_set(pRetCtx->RETstatus_PM,
				    PANEL_LABEL_STRING, "end of file", NULL);
		XFlush((Display *)xv_get(pCtx->guiCtx.baseFrame, XV_DISPLAY));
		return;
	    }
	    assertAlways(stat == S_syd_OK);
	}
    }
    else if (from == ARR_RET_BOF) {
	stat = sydPosition(pSspec, NULL);
	assertAlways(stat == S_syd_OK);
    }

    stat = sscanf((char *)xv_get(pRetCtx->RETthruNsamp_PT, PANEL_VALUE),
								"%d", &nSamp);
    if (stat != 1 || nSamp <= 0) {
	if (thru == ARR_RET_NSAMP)
	    guiNotice(&pCtx->guiCtx, "illegal number of samples requested");
	else if (thru == ARR_RET_THROUGH || thru == ARR_RET_PERIOD) {
	    guiNotice(&pCtx->guiCtx,
		"a max number of samples must be specified");
	}
	xv_set(pCtx->pRetCtx->RETstatus_PM, PANEL_LABEL_STRING, "", NULL);
	XFlush((Display *)xv_get(pCtx->guiCtx.baseFrame, XV_DISPLAY));
	return;
    }
    if (thru == ARR_RET_THROUGH) {
	stat = RETstamp(pCtx, pRetCtx->RETthruStamp_PT, &throughTS);
	if (stat != OK)
	    return;
	pRetCtx->RETthruTS = throughTS;
    }
    else if (thru == ARR_RET_PERIOD) {
	stat = RETtimeStamp(pCtx, pRetCtx->RETthruPer_PT, &amountTS);
	if (stat != OK)
	    return;
	pRetCtx->RETthruPerTS = amountTS;
    }
								
    xv_set(pRetCtx->RETstatus_PM, PANEL_LABEL_STRING, "retrieving...", NULL);
    XFlush((Display *)xv_get(pCtx->guiCtx.baseFrame, XV_DISPLAY));

    if (pSspec->type != SYD_TY_CA) {
	int done = 0, moreFlag;
	ignored = 0;
	sydInputResetSampled(pSspec);
	sydSampleSetAlloc(pSspec, nSamp);
/*-----------------------------------------------------------------------------
*	get first sample; if a time duration was specified, figure out the
*	`thru' time which corresponds.  Then loop until EOF or desired time.
*----------------------------------------------------------------------------*/
	pRetCtx->RETdeadRefTs.secPastEpoch = 0;
	while (!done) {
	    stat = sydInputGet(pSspec, &moreFlag);
	    if (pRetCtx->RETdeadRst == 1) {
		if (pRetCtx->RETdeadRefTs.secPastEpoch == 0)
		    pRetCtx->RETdeadRefTs = SydInputTs(pSspec);
		else {
		    TsDiffAsDouble(&diff,
				&SydInputTs(pSspec), &pRetCtx->RETdeadRefTs);
		    if (diff > pRetCtx->RETdeadRstSec) {
			sydInputResetSampled(pSspec);
			stat = sydInputGet(pSspec, &moreFlag);
		    }
		    pRetCtx->RETdeadRefTs = SydInputTs(pSspec);
		}
	    }
	    if (thru == ARR_RET_PERIOD) {
		TsAddStamps(&throughTS, &SydInputTs(pSspec), &amountTS);
		thru = ARR_RET_THROUGH;
	    }
	    if (stat == S_syd_OK || stat == S_syd_partial) {
		if (thru == ARR_RET_THROUGH &&
			    TsCmpStampsGT(&SydInputTs(pSspec), &throughTS)) {
		    done = 1;
		}
		else {
		    if (ignorePartial == 1 && stat == S_syd_partial)
			ignored++;
		    else if (acceptIf == 1) {
			if (!sydTest(pSspec))
			    ignored++;
		    }
		    sydInputStoreInSet(pSspec, ignorePartial);
		    if (pSspec->sampleCount >= pSspec->reqCount)
			done = 1;
		}
	    }
	    else
		done = 1;
	}
	if (pSspec->sampleCount > 0) {
	    tsStampToText(&pSspec->pTimeStamp[pSspec->firstData],
		    TS_TEXT_MMDDYY, fromTxt);
	    tsStampToText(&pSspec->pTimeStamp[pSspec->lastData],
		    TS_TEXT_MMDDYY, toTxt);
	    if (ignorePartial == 0 && ignored == 0) {
		sprintf(message, "retrieved %d samples, from %s to %s",
		    pSspec->sampleCount, fromTxt, toTxt);
	    }
	    else {
		sprintf(message,
		    "retrieved %d samples, from %s to %s (%d discarded)",
		    pSspec->sampleCount, fromTxt, toTxt, ignored);
	    }
	}
	else {
	    if (stat == S_syd_EOF)
		sprintf(message, "no samples retrieved: end of file");
	    else if (ignored > 0)
		sprintf(message, "no samples retrieved: %d discarded", ignored);
	    else
		sprintf(message, "no samples retrieved: error on retrieve");
	}
	xv_set(pCtx->pRetCtx->RETstatus_PM, PANEL_LABEL_STRING, message, NULL);
    }
    else {
	SYD_CHAN	*pSChan;
	int		n=0;
	sydInputResetKeepNewest(pSspec);

	pSChan = pSspec->pChanHead;
	while (pSChan != NULL) {
	    if (pSChan->discon == 0)
		n++;
	    pSChan = pSChan->pNext;
	}

	if (n <= 0) {
	    sprintf(message, "no samples retrieved: no channels connected");
	    xv_set(pCtx->pRetCtx->RETstatus_PM,
				PANEL_LABEL_STRING, message, NULL);
	}
	else {
	    sydSampleSetAlloc(pSspec, nSamp);
	    pSspec->sampleCount = 0;
	    pSspec->refTs.secPastEpoch = 0;
	    pSspec->refTs.nsec = 0;
	    pCtx->pRetCtx->RETcaSampling = 1;
	}
    }
}

/*+/subr**********************************************************************
* NAME	RETbutton_timerFunc - periodic checks for Channel Access input
*
* DESCRIPTION
*	Routine to call periodically while actively retrieving from
*	Channel Access.  This routine loops until full sample is
*	available.
*
*	If the "sample done" criterion is met, then this routine will
*	wrap up, as indicated in the description for RETbutton_mi.
*
*	to be called from common timer routine to avoid pre-emption
*	problems.  The most appeal is to call immediately following
*	ca_pend_event.
*-*/
void
RETbutton_timerFunc(pCtx)
ARR_CTX	*pCtx;
{
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    SYD_SPEC	*pSspec=pCtx->pSspec;
    SYD_CHAN	*pSChan;
    char	message[200];
    char	fromTxt[28], toTxt[28];
    int		ignorePartial;
    long	stat;
    double	diff;

    if (pRetCtx == NULL || pRetCtx->RETcaSampling == 0)
	return;
    while (1) {
	stat = sydInputGet(pSspec, NULL);
	if (stat != S_syd_noDataNow) {
	    if (pRetCtx->RETdeadRst == 1) {
		if (pRetCtx->RETdeadRefTs.secPastEpoch == 0)
		    pRetCtx->RETdeadRefTs = SydInputTs(pSspec);
		else {
		    TsDiffAsDouble(&diff,
				&SydInputTs(pSspec), &pRetCtx->RETdeadRefTs);
		    if (diff > pRetCtx->RETdeadRstSec) {
			sydInputResetSampled(pSspec);
			stat = sydInputGet(pSspec, NULL);
		    }
		    pRetCtx->RETdeadRefTs = SydInputTs(pSspec);
		}
	    }
	}
	if (stat != S_syd_noDataNow) {
/*-----------------------------------------------------------------------------
*	if retrieval is from a particular time stamp, wait for data with
*	that stamp.  When data with that time stamp is found, change to
*	ARR_RET_LAST mode.
*----------------------------------------------------------------------------*/
	    if (pRetCtx->RETfrom == ARR_RET_AT) {
		if (TsCmpStampsLT(&SydInputTs(pSspec), &pRetCtx->RETfromTS)) {
		    pSChan = pSspec->pChanHead;
		    while (pSChan != NULL) {
			if (SydInputStatus(pSChan) == SYD_B_FULL ||
				SydInputStatus(pSChan) == SYD_B_RESTART) {
			    SydInputMarkAsSampled(pSChan);	/* skip input */
			}
			pSChan = pSChan->pNext;
		    }
		    goto timerFunc_skip;
		}
		else
		    pRetCtx->RETfrom = ARR_RET_LAST;
	    }
/*-----------------------------------------------------------------------------
*	if retrieval is for a time period, add period to first time stamp,
*	then change to ARR_RET_THROUGH mode.
*----------------------------------------------------------------------------*/
	    if (pRetCtx->RETthru == ARR_RET_PERIOD) {
		TsAddStamps(&pRetCtx->RETthruTS,
			&SydInputTs(pSspec), &pRetCtx->RETthruPerTS);
		pRetCtx->RETthru = ARR_RET_THROUGH;
	    }
	    if (pRetCtx->RETthru == ARR_RET_THROUGH) {
		if (TsCmpStampsGT(&SydInputTs(pSspec), &pRetCtx->RETthruTS))
			goto timerFunc_done;
	    }
	    ignorePartial = 1+(int)xv_get(pRetCtx->RETpartial_PCB,PANEL_VALUE);
	    sydInputStoreInSet(pSspec, ignorePartial);
	    if (pSspec->sampleCount >= pSspec->reqCount) {
		goto timerFunc_done;
	    }
timerFunc_skip:
	    ;
	}
	else
	    break;
    }
    return;
timerFunc_done:
    pRetCtx->RETcaSampling = 0;
    if (pSspec->sampleCount > 0) {
	tsStampToText(&pSspec->pTimeStamp[pSspec->firstData],
			    TS_TEXT_MMDDYY, fromTxt);
	tsStampToText(&pSspec->pTimeStamp[pSspec->lastData],
			    TS_TEXT_MMDDYY, toTxt);
	sprintf(message, "retrieved %d samples, from %s to %s",
			    pSspec->sampleCount, fromTxt, toTxt);
    }
    else
	sprintf(message, "no samples retrieved");
    xv_set(pRetCtx->RETstatus_PM, PANEL_LABEL_STRING, message, NULL);
}

/*+/subr**********************************************************************
* NAME	RETstop_mi - handle the Stop menu item
*
*-*/
static void RETstop_mi(menu, item)
Menu	menu;
Menu_item item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    SYD_SPEC	*pSspec=pCtx->pSspec;
    char	message[200];
    char	fromTxt[28], toTxt[28];

    if (pSspec == NULL)
	return;
    if (pSspec->type != SYD_TY_CA) {
	guiNotice(&pCtx->guiCtx,
		"Stop can be used only with Channel Access retrievals");
	return;
    }
    if (pRetCtx->RETcaSampling == 0)
	return;
    pRetCtx->RETcaSampling = 0;
    if (pSspec->sampleCount > 0) {
	tsStampToText(&pSspec->pTimeStamp[pSspec->firstData],
			    TS_TEXT_MMDDYY, fromTxt);
	tsStampToText(&pSspec->pTimeStamp[pSspec->lastData],
			    TS_TEXT_MMDDYY, toTxt);
	sprintf(message, "retrieved %d samples, from %s to %s",
			    pSspec->sampleCount, fromTxt, toTxt);
    }
    else
	sprintf(message, "no samples retrieved");
    xv_set(pRetCtx->RETstatus_PM, PANEL_LABEL_STRING, message, NULL);
}

/*+/subr**********************************************************************
* NAME	RETssf - install a sample set file for retrieval
*
*	o   all channels are deleted and the retrieval source is closed
*	    prior to opening the `sample set' file.
*	o   if the file name isn't blank,
*	    -   display the file name following the RETRIEVE FROM button
*	    -   make the Retrieve menu item ACTIVE and default
*	    -   get the file information into the text subwindow in CH CF
*	o   if the file name is blank,
*	    -   write blanks following the RETRIEVE FROM button
*
*-*/
static void RETssf(pCtx, path, dir, file)
ARR_CTX	*pCtx;
char	*path, *dir, *file;
{   ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    char	message[400];
    long	stat;
    FILE	*f;

    char	buf[100];
    int		bufLen, i;
    char	*pSel;

    guiTextswReset(pRetCtx->RETwinTextsw);
    if (pCtx->pSspec != NULL) {
	RETchan_DelAll(pCtx);
	stat = sydClose(pCtx->pSspec);
	assertAlways(stat == S_syd_OK);
	pCtx->pSspec = NULL;
    }

    if (file[0] != '\0') {
	stat = sydOpenSSF(&pCtx->pSspec, path);
	if (stat != S_syd_OK) {
	    guiNoticeFile(&pCtx->guiCtx,
			"couldn't initialize for `sample set' file", path);
	    return;
	}
	strcpy(pRetCtx->ssfFile, file);
	strcpy(pRetCtx->ssfDir, dir);
	strcpy(pRetCtx->ssfPath, path);
	sprintf(pRetCtx->RETwinName, "/tmp/ARR%d.inf.tmp", getpid());
	if ((f = fopen(pRetCtx->RETwinName, "w")) == NULL) {
	    guiNoticeFile(&pCtx->guiCtx,
			"Unable to open the file for AR file info",
			pRetCtx->RETwinName);
	    return;
	}

	stat = sydFileInfo(pCtx->pSspec, f);
	assertAlways(stat == S_syd_OK);
	fclose(f);

	xv_set(pRetCtx->RETwinTextsw, TEXTSW_FILE, pRetCtx->RETwinName, NULL);

	sprintf(message, "`sample set' file: %s", pRetCtx->ssfPath);
	xv_set(pRetCtx->ssfStatus_PM, PANEL_LABEL_STRING, message, NULL);
	xv_set(pRetCtx->RETuse_PCB, PANEL_VALUE, ARR_RET_SSF, NULL);
    }
    else {
	xv_set(pRetCtx->ssfStatus_PM, PANEL_LABEL_STRING, "", NULL);
	guiNotice(&pCtx->guiCtx, "no `sample set' file selected");
	xv_set(pRetCtx->RETuse_PCB, PANEL_VALUE, -1, NULL);
    }
}

/*+/subr**********************************************************************
* NAME	RETssfPick_xvo - handle Pick file button for `sample set' file
*
*-*/
RETssfPick_xvo(item)
Xv_opaque item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;

    guiFileSelect(&pCtx->guiCtx, "ARR: Pick `sample set' file",
		pRetCtx->ssfDir, pRetCtx->ssfFile, GUI_TDIM, RETssf, pCtx);
}

/*+/subr**********************************************************************
* NAME	RETwinChanButton_pb - handle the config channel buttons
*
* DESCRIPTION
*	Handles the Add and Delete buttons for channels.  If there is
*	selected in the text subwindow, it is copied into the channel
*	name PT item prior to processing.
*
*	If the channel name contains wildcard characters, they are
*	expanded as processing proceeds.
*
*	For adding a channel:
*	o   frees any existing data
*	o   makes sure new channel isn't a duplicate
*	o   adds the channel to the sync data spec
*
*-*/
static void
RETwinChanButton_pb(item)
Panel_item item;
{   ARR_CTX	*pCtx=(ARR_CTX *)xv_get(item, XV_KEY_DATA, ARRCTX);
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    char	*name, line[500];
    ARR_RET_CH	func;		/* channel function */
    char	*pSel;
    char	pattern[100];

    if (pCtx->pSspec == NULL) {
	guiNotice(&pCtx->guiCtx,
			"no retrieval source selected; use Retrieve from");
	return;
    }

/*-----------------------------------------------------------------------------
*	if there's selected text, and if it's in the window listing the
*	channels, get the channel name from the begin of the line containing
*	the selected text.
*----------------------------------------------------------------------------*/
    pSel = guiGetNameFromSeln(&pCtx->guiCtx, pRetCtx->RETwinTextsw);
    if (pSel != NULL)
	xv_set(pRetCtx->CHname_PT, PANEL_VALUE, pSel, NULL);
    name = (char *)xv_get(pRetCtx->CHname_PT, PANEL_VALUE);
    if (strlen(name) <= (size_t)0) {
	guiNoticeName(&pCtx->guiCtx, "illegal channel name", name);
	return;
    }
    if ((int)xv_get(item, XV_KEY_DATA, CH_FUNC) == ARR_RET_DEL) {
	ARR_CHAN	*pChan, *pChan1;
	int		count=0;

	if (pCtx->pChanHead == NULL) {
	    guiNotice(&pCtx->guiCtx, "no channels selected; can't delete");
	    return;
	}
	strcpy(pattern, name);
	for (pChan=pCtx->pChanHead; pChan!=NULL; pChan=pChan1) {
	    pChan1 = pChan->pNext;
	    if (wildMatch(pChan->pSChan->name, pattern, 0)) {
		RETchan_Del(pCtx, pChan);
		guiMessagePrintf(pRetCtx->CHname_PM,
				"deleted %s", pChan->pSChan->name);
		count++;
	    }
	}
	if (count == 0)
	    guiNoticePrintf(&pCtx->guiCtx, "couldn't find %s", pattern);
    }
    else {
	enum sydChanSync sync;
	char	syncText[7];
	if ((int)xv_get(item, XV_KEY_DATA, CH_FUNC) == ARR_RET_SYNC) {
	    sync = SYD_SY_NONF;
	    strcpy(syncText, "sync");
	}
	else {
	    sync = SYD_SY_FILLED;
	    strcpy(syncText, "filled");
	}
	if (strchr(name, '*') == NULL && strchr(name, '?') == NULL) {
	    RETwinAddChan(pCtx, name, sync, 0);
	    guiMessagePrintf(pRetCtx->CHname_PM,
				"added (%s) %s", syncText, name);
	}
	else {
	    int	charPos=0;
	    char	buf[200], *pC;
	    strcpy(pattern, name);
	    while (guiTextswGets(pRetCtx->RETwinTextsw,buf,200,&charPos)!=NULL){
/*-----------------------------------------------------------------------------
*	search textsw for channel names, one per line; ignore the heading
*	line for sample set files (beginning with "data from")
*----------------------------------------------------------------------------*/
		if (strncmp(buf, "data from", 9) != 0) {
		    if ((pC = strchr(buf, ' ')) != NULL)
			*pC = '\0';
		    if (wildMatch(buf, pattern, 0)) {
			RETwinAddChan(pCtx, buf, sync, 1);
			guiMessagePrintf(pRetCtx->CHname_PM,
				"added (%s) %s", syncText, buf);
		    }
		}
	    }
	}
    }
}

/*+/subr**********************************************************************
* NAME	RETwinAddChan - do actual work of adding a channel
*
*-*/
RETwinAddChan(pCtx, name, sync, quiet)
ARR_CTX	*pCtx;
char	*name;
enum sydChanSync sync;
int	quiet;
{
    ARR_CHAN	*pChan;
    SYD_CHAN	*pSChan;
    long	stat;

    if ((pSChan=sydChanFind(pCtx->pSspec, name)) != NULL && pSChan->dataChan) {
	if (quiet == 0)
	    guiNoticeName(&pCtx->guiCtx, "channel is already selected", name);
	return;
    }
    if ((pChan = (ARR_CHAN *)malloc(sizeof(ARR_CHAN))) == NULL) {
	(void)printf("couldn't malloc for new channel\n");
	exit(1);
    }
    if (sync == SYD_SY_NONF)
	pChan->name[0] = ' ';
    else
	pChan->name[0] = 'F';
    pChan->name[1] = ' ';
    strcpy(&pChan->name[2], name);
    stat = sydChanOpen(pCtx->pSspec, &pSChan, name, sync, NULL, 0);
    if (stat != S_syd_OK) {
	if (stat == S_syd_chanNotConn) {
	    int		n=0;
	    while (n++<3 && pSChan->discon)
		ca_pend_event(.05);
	    if (pSChan->discon) {
		guiNoticeName(&pCtx->guiCtx,
"connection not complete; use `Configure channels' if name is incorrect",
		name);
	    }
	}
	else {
	    guiNoticeName(&pCtx->guiCtx, "couldn't open channel", name);
	    free((char *)pChan);
	    return;
	}
    }
    pChan->pSChan = pCtx->pSspec->pChanTail;
    stat = sydSampleSetFree(pCtx->pSspec);
    assertAlways(stat == S_syd_OK);
    xv_set(pCtx->pRetCtx->RETstatus_PM, PANEL_LABEL_STRING, "", NULL);
    DoubleListAppend(pChan, pCtx->pChanHead, pCtx->pChanTail);
}

/*+/subr**********************************************************************
* NAME	RETchan_Del - delete a channel from the retrieval list
*
* DESCRIPTION
*	o   removes channel from retrieval list
*	o   puts channel's name in the "Channel name" PANEL_TEXT item
*	o   removes channel from delete menu
*
*	RETchan_DelAll deletes all the channels
*
* RETURNS
*
*-*/
RETchan_Del(pCtx, pChan)
ARR_CTX	*pCtx;
ARR_CHAN *pChan;
{
    long	stat;

    DoubleListRemove(pChan, pCtx->pChanHead, pCtx->pChanTail);
    stat = sydChanClose(pCtx->pSspec, pChan->pSChan);
    assertAlways(stat == S_syd_OK);
    free((char *)pChan);
}
RETchan_DelAll(pCtx)
ARR_CTX	*pCtx;
{
    while (pCtx->pChanHead != NULL)
	RETchan_Del(pCtx, pCtx->pChanHead);
}

/*+/subr**********************************************************************
* NAME	RETsetupInitCF - initialize the retrieval setup command frame
*
*-*/
static void RETsetupInitCF(pCtx)
ARR_CTX	*pCtx;
{
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    Panel	panel;
    Panel_item	ckbx;
    int		x=5, y=5, ht=0;
    int		x1, x2, y1, y2;

/*-----------------------------------------------------------------------------
* use archive request to determine source  Pick file...
* Retrieve from:
* [] `by channel' file  Pick file...  ........status message........
* [] `sample set' file  Pick file...  ........status message........
* [] Channel Access
* Channel name _________________  ........status message........
* Add channel, sync  Add channel, filled  Configure channels...
*----------------------------------------------------------------------------*/
    pRetCtx->RETsetup_CF = guiFrameCmd("ARR: retrieval setup",
				pCtx->guiCtx.baseFrame, &panel, NULL);

    guiMessage("use archive request to determine source", panel, &x,&y,&ht);
    guiButton("Pick file...", panel, &x, &y, &ht, guiEditorShowCF_xvo, NULL,
				GUI_PEDIT, pRetCtx->pConfigEdit, 0, NULL);
    y += ht + 5; x = 5; ht = 0;
    pRetCtx->RETuse_PCB = ckbx = guiCheckbox("Retrieve from:",
	panel, 3, &x, &y, &ht, RETchoose_xvo, ARRCTX, pCtx, 0, NULL);
    y += ht + 5; x = 5; ht = 0;
    guiCheckboxItem("`by channel' file", ckbx, ARR_RET_BCF, 0, &x, &y, &ht);
    x1 = x; y1 =y;
    y += ht + 5; x = 5; ht = 0;
    guiCheckboxItem("`sample set' file", ckbx, ARR_RET_SSF, 0, &x, &y, &ht);
    x2 = x; y2 =y;
    y += ht + 5; x = 5; ht = 0;
    guiCheckboxItem("Channel Access", ckbx, ARR_RET_CA, 0, &x, &y, &ht);
    y += ht + 5; x = 5; ht = 0;
    if (x1 < x2)	x1 = x2;
    else		x2 = x1;
    guiButton("Pick file...", panel, &x1, &y1, &ht,
				RETbcfPick_xvo, NULL, ARRCTX, pCtx, 0, NULL);
    pRetCtx->bcfStatus_PM = guiMessage("", panel, &x1, &y1, &ht);
    guiButton("Pick file...", panel, &x2, &y2, &ht,
				RETssfPick_xvo, NULL, ARRCTX, pCtx, 0, NULL);
    pRetCtx->ssfStatus_PM = guiMessage("", panel, &x2, &y2, &ht);


    pRetCtx->CHname_PT = guiTextField("Channel name", panel, &x, &y, &ht,
					    NULL, 35, 35, 0, NULL, 0, NULL);
    pRetCtx->CHname_PM = guiMessage("", panel, &x, &y, &ht);
    y += ht + 10; x = 5; ht = 0;

    guiButton("Add channel, sync", panel, &x, &y, &ht,
	    RETwinChanButton_pb, NULL, ARRCTX, pCtx, CH_FUNC, ARR_RET_SYNC);
    guiButton("Add channel, filled", panel, &x, &y, &ht,
	    RETwinChanButton_pb, NULL, ARRCTX, pCtx, CH_FUNC, ARR_RET_FILL);
    guiButton("Delete channel", panel, &x, &y, &ht,
	    RETwinChanButton_pb, NULL, ARRCTX, pCtx, CH_FUNC, ARR_RET_DEL);
    guiButton("Configure channels...", panel, &x, &y, &ht,
	    guiEditorShowCF_xvo, NULL,
	    GUI_PEDIT, pRetCtx->pConfigEdit, 0, NULL);
    y += ht + 5; x = 5; ht = 0;

    window_fit_height(panel);
    pRetCtx->RETwinTextsw = guiBrowser(pRetCtx->RETsetup_CF,panel,110,30,NULL);
    window_fit(pRetCtx->RETsetup_CF);
}

/*+/subr**********************************************************************
* NAME	RETinit - initialize for retrieval
*
* DESCRIPTION
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
RETinit(pCtx, panel, pX, pY, pHt)
ARR_CTX	*pCtx;
Panel	panel;
int	*pX, *pY, *pHt;
{
    Menu	menu;
    Xv_opaque	item;
    Panel_item	ckbx;

    ARR_RET_CTX	*pRetCtx;

    if ((pRetCtx = (ARR_RET_CTX *)malloc(sizeof(ARR_RET_CTX))) == NULL)
	assertAlways(0);
    pCtx->pRetCtx = pRetCtx;
    pRetCtx->pCtx = pCtx;

    pCtx->pRetCtx->RETcaSampling = 0;

    sprintf(pRetCtx->bcfDir, "%s/%s", pCtx->guiCtx.pwd, pCtx->dfltBCF_dir);
    strcpy(pRetCtx->bcfFile, "");
    sprintf(pRetCtx->ssfDir, "%s/%s", pCtx->guiCtx.pwd, pCtx->dfltSSF_dir);
    strcpy(pRetCtx->ssfFile, "");
    sprintf(pRetCtx->configFile, "config%d.tmp", getpid());
    sprintf(pRetCtx->configDir, "%s/%s", pCtx->guiCtx.pwd, pCtx->dfltRet_dir);
    sprintf(pRetCtx->configPath,"%s/%s",pRetCtx->configDir,pRetCtx->configFile);
    RETconfigInitEditor(pCtx);
    RETsetupInitCF(pCtx);

/*-----------------------------------------------------------------------------
* Pick data file...  Pick channels...  Chan config file...
* [] discard partial samples
* [] reset filled channels if more than _____ seconds between samples
* [] round time stamps to nearest ___ milli-seconds
* [] keep samples if ________________________
*
* start retrieving:
* [] following last retrieval
* [] at ____________________  (mm/dd/yy hh:mm:ss.msc)
* [] beginning of file
* retrieve:
* [] ______ samples (or max # samples for timed retrievals)
* [] through ____________________  (mm/dd/yy hh:mm:ss.msc)
* [] ______ hh:mm:ss.msc of data
*
* Retrieve v    status message...
*      Retrieve
*      Resync
*      Stop
*----------------------------------------------------------------------------*/
    guiButton("Pick data file...", panel, pX,pY,pHt, guiCFshowPin_xvo, NULL,
			GUI_PCTX, &pCtx->guiCtx, GUI_CF, pRetCtx->RETsetup_CF);
    guiButton("Pick channels...", panel, pX,pY,pHt, guiCFshowPin_xvo, NULL,
			GUI_PCTX, &pCtx->guiCtx, GUI_CF, pRetCtx->RETsetup_CF);
    guiButton("Chan config File...", panel, pX,pY,pHt, guiEditorShowCF_xvo,
			NULL, GUI_PEDIT, pRetCtx->pConfigEdit, 0, NULL);
    *pY += 15 + *pHt; *pX = 5; *pHt = 0;

    pRetCtx->RETpartial_PCB = ckbx = guiCheckbox(" ",
		panel, 1, pX,pY,pHt, NULL, 0, NULL, 0, NULL);
    *pX = 5;
    guiCheckboxItem("discard partial samples", ckbx, 0, 0, pX,pY,pHt);
    *pY += *pHt; *pX = 5; *pHt = 0;

    pRetCtx->RETdeadRst_PCB = ckbx = guiCheckbox(" ",
		panel, 1, pX,pY,pHt, NULL, 0, NULL, 0, NULL);
    *pX = 5;
    guiCheckboxItem("reset filled channels if more than", ckbx, 0,0,pX,pY,pHt);
    pRetCtx->RETdeadRst_PT = guiTextField("", panel, pX,pY,pHt, NULL,
					    4, 4, ARRCTX, pCtx, 0, NULL);
    guiMessage("seconds between samples", panel, pX,pY,pHt);
    *pY += *pHt; *pX = 5; *pHt = 0;

    pRetCtx->RETroundTs_PCB = ckbx = guiCheckbox(" ",
		panel, 1, pX,pY,pHt, NULL, 0, NULL, 0, NULL);
    *pX = 5;
    guiCheckboxItem("round time stamps to nearest", ckbx, 0, 0, pX,pY,pHt);
    pRetCtx->RETroundTs_PT = guiTextField("", panel, pX,pY,pHt, NULL,
					    4, 4, ARRCTX, pCtx, 0, NULL);
    guiMessage("milli-seconds", panel, pX,pY,pHt);
    *pY += *pHt; *pX = 5; *pHt = 0;

    pRetCtx->RETaccept_PCB = ckbx = guiCheckbox(" ",
		panel, 1, pX,pY,pHt, NULL, 0, NULL, 0, NULL);
    *pX = 5;
    guiCheckboxItem("accept samples if", ckbx, 0,0,pX,pY,pHt);
    pRetCtx->RETaccept_PT = guiTextField("", panel, pX,pY,pHt, NULL,
					    80, 60, ARRCTX, pCtx, 0, NULL);

    *pY += *pHt + 15; *pX = 5; *pHt = 0;
    pRetCtx->RETfrom_PCB = ckbx = guiCheckbox("start retrieving:",
		panel, 3, pX,pY,pHt, NULL, 0, NULL, 0, NULL);
    *pY += *pHt; *pX = 5; *pHt = 0;
    guiCheckboxItem("beginning of file", ckbx, ARR_RET_BOF, 1, pX,pY,pHt);
    *pY += *pHt; *pX = 5; *pHt = 0;
    guiCheckboxItem("following last retrieval",
					ckbx, ARR_RET_LAST, 0, pX,pY,pHt);
    *pY += *pHt; *pX = 5; *pHt = 0;
    guiCheckboxItem("at", ckbx, ARR_RET_AT, 0, pX,pY,pHt);
    pRetCtx->RETfromStamp_PT = guiTextField("", panel, pX,pY,pHt, NULL,
					    32, 32, ARRCTX, pCtx, 0, NULL);
    guiMessage("(mm/dd/yy hh:mm:ss.msc)", panel, pX,pY,pHt);
    *pY += *pHt + 5; *pX = 5; *pHt = 0;

    pRetCtx->RETthru_PCB = ckbx = guiCheckbox("retrieve:",
		panel, 3, pX,pY,pHt, NULL, 0, NULL, 0, NULL);
    *pY += *pHt; *pX = 5; *pHt = 0;
    guiCheckboxItem("", ckbx, ARR_RET_NSAMP, 1, pX,pY,pHt);
    pRetCtx->RETthruNsamp_PT = guiTextField("", panel, pX,pY,pHt, NULL,
					    6, 6, ARRCTX, pCtx, 0, NULL);
    guiMessage("samples (or max # samples for timed retrievals)",
					    panel, pX,pY,pHt);
    *pY += *pHt; *pX = 5; *pHt = 0;
    guiCheckboxItem("through", ckbx, ARR_RET_THROUGH, 0, pX,pY,pHt);
    pRetCtx->RETthruStamp_PT = guiTextField("", panel, pX,pY,pHt, NULL,
					    32, 32, ARRCTX, pCtx, 0, NULL);
    guiMessage("(mm/dd/yy hh:mm:ss.msc)", panel, pX,pY,pHt);
    *pY += *pHt; *pX = 5; *pHt = 0;
    guiCheckboxItem("", ckbx, ARR_RET_PERIOD, 0, pX,pY,pHt);
    pRetCtx->RETthruPer_PT = guiTextField("", panel, pX,pY,pHt, NULL,
					    21, 21, ARRCTX, pCtx, 0, NULL);
    guiMessage("hh:mm:ss.msc of data", panel, pX,pY,pHt);
    *pY += *pHt + 5; *pX = 5; *pHt = 0;

    menu = guiMenu(NULL, 0, NULL, 0, NULL);
    guiMenuItem("Retrieve", menu, RETbutton_mi, 0, 1, ARRCTX, pCtx, 0, NULL);
    guiMenuItem("Resync", menu, RETbcfResync_mi, 0, 0, ARRCTX, pCtx, 0, NULL);
    guiMenuItem("Stop", menu, RETstop_mi, 0, 0, ARRCTX, pCtx, 0, NULL);
    item = guiButton("Retrieve", panel, pX,pY,pHt, NULL, menu, 0,NULL,0,NULL);
    pRetCtx->RETstatus_PM = guiMessage("", panel, pX,pY,pHt);
    *pX += 600;
    guiMessage("", panel, pX,pY,pHt);    /* kludge to force panel width */
    *pY += 15 + (int)xv_get(item, XV_HEIGHT); /* use height of btn, not msg */
    *pX = 5; *pHt = 0;
}

/*+/subr**********************************************************************
* NAME	RETquit - wrapup processing
*
*-*/
void
RETquit(pRetCtx, destroyFlag)
ARR_RET_CTX *pRetCtx;
int	destroyFlag;
{
    ARR_CTX	*pCtx=pRetCtx->pCtx;
    long	stat;

    pRetCtx->RETcaSampling = 0;
    if (pCtx->pSspec != NULL) {
	RETchan_DelAll(pCtx);
	stat = sydClose(pCtx->pSspec);
	assertAlways(stat == S_syd_OK);
	pCtx->pSspec = NULL;
    }

    if (destroyFlag) {
	guiEditorReset(pRetCtx->pConfigEdit);
	guiTextswReset(pRetCtx->RETwinTextsw);
	xv_destroy_safe(pRetCtx->RETsetup_CF);
    }

    unlink(pRetCtx->RETwinName);
    unlink(pRetCtx->configPath);
    free(pRetCtx);
}

/*+/subr**********************************************************************
* NAME	RETquitCheck
*
* RETURNS
*	0	if it's OK to quit, else
*	-1
*
*-*/
int
RETquitCheck(pCtx)
ARR_CTX	*pCtx;
{
    ARR_RET_CTX	*pRetCtx=pCtx->pRetCtx;
    GUI_EDIT	*pEdit=pRetCtx->pConfigEdit;

    if (pEdit != NULL && pEdit->updFlag) {
	if (strcmp(pEdit->file, pRetCtx->configFile) == 0)
	    guiEditorReset(pEdit);
	else {
	    guiNotice(&pCtx->guiCtx, "can't quit; edit mode is set");
	    guiEditorShowCF(pEdit);
	    return -1;
	}
    }
    return 0;
}
