/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/*	@(#)EX.c	1.3 2/23/93
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 */
/*+/mod***********************************************************************
* TITLE	EX.c - export routines for ARR and AR
*
* DESCRIPTION
*
*-***************************************************************************/
#include <stdio.h>

#define XVIEW
#if defined XVIEW
#   include <xview/xview.h>
#   include <xview/frame.h>
#   include <xview/panel.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <guiSubr.h>
#include <sydDefs.h>

#include <enumShared.h>
#include "EX.h"


/*+/subr**********************************************************************
* NAME	EXport_xvo - handle the "Export" button push
*
*	o   decode the export option
*	o   do the export
*
*-*/
void EXport_xvo();
static void EXport_mi(menu,item) Menu menu;Menu_item item;{EXport_xvo(item);}
static void EXport_xvo(item)
Xv_opaque item;
{   EX_CTX	*pExCtx=(EX_CTX *)xv_get(item, XV_KEY_DATA, EXCTX);
    EX_TYPE	cmdType=(EX_TYPE)xv_get(item, XV_KEY_DATA, EXTYPE);
    SYD_SPEC    *pSspec=*pExCtx->ppSspec;
    int		option, show;
    FILE	*f;
    char	path[2*GUI_TDIM];
    int		modeCode;

    guiCFshowPin(pExCtx->EXwin_CF);
    if (pSspec == NULL || pSspec->chanCount <= 0) {
	if (cmdType == EX_NULL)
	    return;
        guiNotice(pExCtx->pGuiCtx, "no channels selected to export");
	guiMessagePrintf(pExCtx->message_PM, "export aborted");
        return;
    }
    if (pSspec->sampleCount <= 0) {
	if (cmdType == EX_NULL)
	    return;
        guiNotice(pExCtx->pGuiCtx, "no samples to export");
	guiMessagePrintf(pExCtx->message_PM, "export aborted");
        return;
    }

    show = (int)xv_get(pExCtx->EXstat_PCS, PANEL_VALUE);
    if (pExCtx->useStats == 0 && pExCtx->useMeans == 0)
	sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 0, NULL);
    else {
	if (show == 1) {
	    guiNotice(pExCtx->pGuiCtx,
				"can't show status with mean and/or stdDev");
	    guiMessagePrintf(pExCtx->message_PM, "export aborted");
	    return;
	}
	if (pExCtx->useStats == 1)
	    sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 1, NULL);
	else
	    sydSetAttr(pSspec, SYD_ATTR_USE_MEANS, 1, NULL);
	sydSampleSetStats(pSspec);
    }

    strcpy(pExCtx->dir, (char *)xv_get(pExCtx->dir_PT, PANEL_VALUE));
    strcpy(pExCtx->file, (char *)xv_get(pExCtx->file_PT, PANEL_VALUE));
    modeCode = (int)xv_get(pExCtx->fileMode_PCS, PANEL_VALUE);
    if (strlen(pExCtx->file) == 0) {
	guiNotice(pExCtx->pGuiCtx, "no file selected");
	guiMessagePrintf(pExCtx->message_PM, "export aborted");
	return;
    }
    sprintf(path, "%s/%s", pExCtx->dir, pExCtx->file);
    if (modeCode == 0) {	/* over-write mode */
	if ((f = fopen(path, "w")) == NULL) {
	    guiNoticeFile(pExCtx->pGuiCtx, "Unable to open for output", path);
	    guiMessagePrintf(pExCtx->message_PM, "export aborted");
	    return;
	}
    }
    else {			/* append mode */
	if ((f = fopen(path, "a")) == NULL) {
	    guiNoticeFile(pExCtx->pGuiCtx, "Unable to open for output", path);
	    guiMessagePrintf(pExCtx->message_PM, "export aborted");
	    return;
	}
    }

    if ((int)xv_get(pExCtx->EXtype_PCS, PANEL_VALUE) == 0) {
	if (show == 0)
	    option = 1;
	else
	    option = 2;
	guiMessagePrintf(pExCtx->message_PM, "exporting...");
	sydSampleSetExport(pSspec, f, option);
    }
    else if ((int)xv_get(pExCtx->EXtype_PCS, PANEL_VALUE) == 2) {
	if (pExCtx->useStats == 1) {
	    guiNotice(pExCtx->pGuiCtx,
		"can't export mean and stdDev to spreadsheet for arrays");
	    guiMessagePrintf(pExCtx->message_PM, "export aborted");
	    fclose(f);
	    return;
	}
	if (show == 0)
	    option = 3;
	else
	    option = 4;
	guiMessagePrintf(pExCtx->message_PM, "exporting...");
	sydSampleSetExport(pSspec, f, option);
    }
    else {
	if (pExCtx->useStats == 1) {
	    guiNotice(pExCtx->pGuiCtx,
		"can't export mean and stdDev to sample set file");
	    guiMessagePrintf(pExCtx->message_PM, "export aborted");
	    fclose(f);
	    return;
	}
	guiMessagePrintf(pExCtx->message_PM, "exporting...");
	sydSampleSetWriteSSF(pSspec, f, pExCtx->SSFtitle, "");
    }
    guiMessagePrintf(pExCtx->message_PM, "export complete");
    fclose(f);
}

/*+/subr**********************************************************************
* NAME	EXwinInitCF - initialize the export command frame
*
*-*/
static void EXwinInitCF(pExCtx, title)
EX_CTX	*pExCtx;
char	*title;
{
    Menu	menu;
    Panel	panel;
    Xv_opaque	item;
    int		y=5, x=5, ht=0, x1;

/*-----------------------------------------------------------------------------
* Export v    status...
*    to file
* convert for:  spreadsheet  sample set file  spreadsheet for arrays
* value status: suppress show
* Directory: _______________________
* File: _______________________  v over-write
*                                  append
*----------------------------------------------------------------------------*/

    pExCtx->EXwin_CF =
	    guiFrameCmd(title, pExCtx->pGuiCtx->baseFrame,&panel,NULL);
    menu = guiMenu(NULL, 0,NULL,0,NULL);
    guiMenuItem("to file", menu,EXport_mi,0,0,EXCTX,pExCtx,EXTYPE,EX_DO);
    guiButton("Export", panel, &x, &y, &ht, NULL, menu, 0,NULL,0,NULL);
    pExCtx->message_PM = guiMessage("", panel, &x,&y,&ht);
    y += ht + 5; x = 5; ht = 0;
    x1 = 80;
    guiMessage("convert for:", panel, &x,&y,&ht);
    pExCtx->EXtype_PCS = guiChoice("",
	"spreadsheet", "sample set file", "spreadsheet for arrays", NULL,
	panel, 1, 0, 3, &x1,&y,&ht, NULL, 0,NULL,0,NULL);
    y += ht + 5; x = 5; ht = 0;
    x1 = 80;
    guiMessage("value status:", panel, &x,&y,&ht);
    pExCtx->EXstat_PCS = guiChoice("", "suppress", "show", NULL,
	panel, 1, 1, 0, &x1,&y,&ht, NULL, 0,NULL,0,NULL);
    y += ht + 5; x = 5; ht = 0;

    pExCtx->dir_PT = guiTextField("Directory:", panel, &x,&y,&ht, NULL,
		GUI_TDIM-1, 40, 0, NULL, 0, NULL);
    xv_set(pExCtx->dir_PT, PANEL_VALUE, pExCtx->dir, NULL);
    y += ht + 5; x = 5; ht = 0;

    pExCtx->file_PT = guiTextField("File:", panel, &x,&y,&ht, NULL,
		GUI_TDIM-1, 40, 0, NULL, 0, NULL);
    strcpy(pExCtx->file, "");
    xv_set(pExCtx->file_PT, PANEL_VALUE, pExCtx->file, NULL);
    pExCtx->fileMode_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_CHOICE_STRINGS,	"over-write","append",NULL,
	PANEL_VALUE,	0,
	XV_X, x, XV_Y, y-5, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;

    window_fit(panel);
    window_fit(pExCtx->EXwin_CF);
}

/*+/subr**********************************************************************
* NAME	EXinitButton - initialize export window, tied to a button
*
* DESCRIPTION
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
EXinitButton(ppExCtx, pGuiCtx, progName,title,dfltDir,panel, pX,pY,pHt,ppSspec)
EX_CTX	**ppExCtx;
GUI_CTX	*pGuiCtx;
char	*progName;
char	*title;
char	*dfltDir;
Panel	panel;
int	*pX, *pY, *pHt;
SYD_SPEC **ppSspec;
{
    EX_CTX *pExCtx;

    if ((pExCtx = (EX_CTX *)malloc(sizeof(EX_CTX))) == NULL)
	assertAlways(0);
    *ppExCtx = pExCtx;
    pExCtx->pGuiCtx = pGuiCtx;
    pExCtx->ppSspec = ppSspec;
    pExCtx->useStats = 0;
    sprintf(pExCtx->dir, "%s/%s", pGuiCtx->pwd, dfltDir);
    sprintf(pExCtx->SSFtitle, "EXPORTED FROM: %s", progName);
    EXwinInitCF(pExCtx, title);

    guiButton("exporting...", panel, pX, pY, pHt, guiCFshowPin_xvo, NULL,
			GUI_PCTX, pGuiCtx, GUI_CF, pExCtx->EXwin_CF);
}

/*+/subr**********************************************************************
* NAME	EXinitMenu - initialize export window, tied to a menu item
*
*-*/
void
EXinitMenu(ppExCtx, pGuiCtx, progName, title, dfltDir, menu, dflt, ppSspec)
EX_CTX	**ppExCtx;
GUI_CTX	*pGuiCtx;
char	*progName;
char	*title;
char	*dfltDir;
Menu	menu;
int	dflt;
SYD_SPEC **ppSspec;
{
    EX_CTX *pExCtx;

    if ((pExCtx = (EX_CTX *)malloc(sizeof(EX_CTX))) == NULL)
	assertAlways(0);
    *ppExCtx = pExCtx;
    pExCtx->pGuiCtx = pGuiCtx;
    pExCtx->ppSspec = ppSspec;
    pExCtx->useStats = 0;
    sprintf(pExCtx->dir, "%s/%s", pGuiCtx->pwd, dfltDir);
    sprintf(pExCtx->SSFtitle, "EXPORTED FROM: %s", progName);
    EXwinInitCF(pExCtx, title);

    guiMenuItem("exporting...", menu, guiCFshowPin_mi, 0, dflt,
			GUI_PCTX, pGuiCtx, GUI_CF, pExCtx->EXwin_CF);
}

/*+/subr**********************************************************************
* NAME	EXquit
*
*-*/
void
EXquit(pExCtx, destroyFlag)
EX_CTX	*pExCtx;
int	destroyFlag;
{
    if (destroyFlag)
	xv_destroy_safe(pExCtx->EXwin_CF);
    free(pExCtx);
}
