/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/****************************************************************************
			GTA PROJECT   AT division

	Copyright, 1990, The Regents of the University of California
		      Los Alamos National Laboratory

	FILE PATH:	~ar/bfTest.c
	ENVIRONMENT:	SunOS, VxWorks
	MAKE OPTIONS:
	SCCS VERSION:	@(#)bfTest.c	1.3
*+/mod***********************************************************************
* TITLE	bfTest.c - tester for the 'block file' subroutines
*****************************************************************************/

#include "bfDefs.h"
#include "genTasks.h"

#ifdef vxWorks
#   include <vxWorks.h>
#   include <ioLib.h>		/* for O_RDWR and O_RDONLY definitions */
#   include <stdioLib.h>
#   define MAXPRIO 200
#else
#   include <stdio.h>
#   include <sys/fcntl.h>	/* for O_RDWR and O_RDONLY definitions */
#   define MAXPRIO 50
#endif


#ifndef vxWorks
    main(argc, argv)
    int     argc;		/* number of command line args */
    char   *argv[];		/* command line args */
    {
	(void)pod_setmaxpri(MAXPRIO);
	lwp_setstkcache(100000, 9);

	bfTest();
	exit(0);
    }
#endif

/*+/subr**********************************************************************
* NAME	bfTest - driver for block file test routine
*
* DESCRIPTION
*
* RETURNS
*
* BUGS
* o	text
*
* SEE ALSO
*
* EXAMPLE
*
*-*/
int
bfTest()
{
    int		stat;		/* status from calls */
    int		retStat=OK;	/* return status to caller */
    TASK_ID	bfTest1Tid;	/* task ID of bfTest1 */
    int		bfTest1();
    SEM_ID	semWorkDone;	/* semaphore for 'work completed' */
    char	bfTestCmd[80];	/* buffer for command data */

    (void)clearerr(stdin);

    semInit(semWorkDone);

/* give ourselves a lower priority, then start the bfTest1 task */
    stat = taskPrioritySet(SELF, GenLowerPriority(MAXPRIO, 1));
    bfTest1Tid = taskSpawn("bfTest1", MAXPRIO, VX_STDIO | VX_FP_TASK, 20000,
					bfTest1, semWorkDone, bfTestCmd);
    if (GenTaskNull(bfTest1Tid)) {
	(void)printf("bfTest: couldn't spawn bfTest1\n");
	return 1;
    }

    while ((printf("bfTest: "), fgets(bfTestCmd, 80, stdin)) != NULL) {
	if (strncmp(bfTestCmd, "quit", 4) == 0)
	    goto done;
	semGive(semWorkDone);
	taskResume(bfTest1Tid);
	semTake(semWorkDone, WAIT_FOREVER);
    }
    (void)printf("\n");
done:
#ifdef vxWorks
    checkStack(bfTest1Tid);	/* find out stack usage for bfTest1 */
#endif

    (void)strcpy(bfTestCmd, "quit");
    semGive(semWorkDone);
    taskResume(bfTest1Tid);

    (void)clearerr(stdin);

    return OK;
}

/*+/subr**********************************************************************
* NAME	bfTest1 - test block file subroutines
*
* DESCRIPTION
*	Provide a 'shell' for exercising the block file routines.
*
* RETURNS
*
*-*/
int
bfTest1(semWorkDone, cmdBuf)
SEM_ID	semWorkDone;	/* semaphore for 'work completed' */
char	*cmdBuf;	/* buffer for command data */
{
    char        command[10];    /* command string */
    char        message[80];    /* temp for status message */
    char        cmdArg1[40];    /* command option */
    char        cmdArg2[40];    /* command value */
    char	cmdLine[80];	/* command line from input */
    int         stat;           /* status return */

    BF_DESC	*pDesc=NULL;	/* pointer to block file descriptor */

    char	buffer[BF_MAX_BLOCK_SIZE];	/* buffer */
    BF_BLOCK	*pBuf;		/* buffer pointer */
    long	blockNum;	/* blockNum of last read or acquired block */

    pBuf = (BF_BLOCK *)buffer;
    command[0] = '\0';

    while (strcmp(command, "quit") != 0) {
	taskSuspend(SELF);			/* wait for input */
	semTake(semWorkDone, WAIT_FOREVER);
	(void)strcpy(cmdLine, cmdBuf);
	command[0] = '\0';
	cmdArg1[0] = '\0';
	cmdArg2[0] = '\0';
	sscanf(cmdLine, "%10s %40s %40s", command, cmdArg1, cmdArg2);
	if (strlen(command) == 0)
/*----------------------------------------------------------------------------
* blank line - ignore
*---------------------------------------------------------------------------*/
	    ;               /* no action */
	else if (strcmp(command, "acquire") == 0) {
/*----------------------------------------------------------------------------
* acquire - acquire a block from the free list of the 'current file' and make it
*    the 'current block'
*---------------------------------------------------------------------------*/
	    blockNum = bfAcquire(pDesc, pBuf);
	    (void)printf("%d\n", blockNum);
	}
	else if (strcmp(command, "close") == 0) {
/*----------------------------------------------------------------------------
* close - close the 'current file'
*---------------------------------------------------------------------------*/
	    stat = bfClose(&pDesc);
	    (void)printf("%d\n", stat);
	}
	else if (strcmp(command, "create") == 0) {
/*----------------------------------------------------------------------------
* create - create and initialize a named block file; this doesn't affect
*    the 'current file'
*---------------------------------------------------------------------------*/
	    int	i;
	    long	nBlocks;	/* number of blocks to create */
	    long	blockSize;	/* size of blocks */
	    long	fileFlags;	/* flags */
	    long	maxBlocks;	/* size limit */
	    char	name[40];	/* name of file */

	    i = sscanf(cmdLine, "%*s %40s %ld %ld %ld %ld", name, &nBlocks,
			&blockSize, &fileFlags, &maxBlocks);
	    if (i != 5) {
		(void)printf("syntax error; ");
		(void)printf(
			"usage: create name nBlks blkSize flags maxBlks\n");
	    }
	    else {
		pDesc = bfCreate(name, BF_TYPE_UNDEF, nBlocks, blockSize,
				fileFlags, maxBlocks);
		(void)printf("%x\n", pDesc);
	    }
	}
	else if (strcmp(command, "info") == 0) {
/*----------------------------------------------------------------------------
* info - print out some information on the named file; this doesn't affect
*    the 'current file'
*---------------------------------------------------------------------------*/
	    stat = bfInfo(cmdArg1);
	    (void)printf("%d\n", stat);
	}
	else if (strcmp(command, "open") == 0) {
/*----------------------------------------------------------------------------
* open - open the named file and make it the 'current file'
*---------------------------------------------------------------------------*/
	    int		mode;		/* mode for open */

	    if (strcmp(cmdArg2, "O_RDWR") == 0)
		mode = O_RDWR;
	    else if (strcmp(cmdArg2, "O_RDONLY") == 0)
		mode = O_RDONLY;
	    else
		mode = ERROR;
	    if (mode != ERROR) {
		pDesc = bfOpen(cmdArg1, mode, BF_TYPE_UNDEF, 0);
		(void)printf("%x\n", pDesc);
	    }
	    else {
		(void)printf("syntax error; ");
		(void)printf("usage: open name mode\n");
		(void)printf("	mode can be either O_RDWR or O_RDONLY\n");
	    }
	}
	else if (strcmp(command, "quit") == 0) {
/*----------------------------------------------------------------------------
* quit
*---------------------------------------------------------------------------*/
	    ;			/* no action */
	}
	else if (strcmp(command, "read") == 0) {
/*----------------------------------------------------------------------------
* read - read a block from the 'current file' and make it the 'current block'
*
* BUGS
* o	assumes information is printable text; should check BF_FLAG_BIN
*---------------------------------------------------------------------------*/
	    if (sscanf(cmdArg1, "%d", &blockNum) != 1) {
		(void)printf("syntax error; ");
		(void)printf("usage: read blockNum\n");
	    }
	    else {
		stat = bfRead(pDesc, pBuf, blockNum);
		(void)printf("%d\n", stat);
		if (stat == OK) {
		    (void)printf("flink, fByte, lByte: %d %d %d\n",
			pBuf->flink, pBuf->firstByte, pBuf->lastByte);
		    if (pBuf->lastByte < BF_BLOCK_DATA)
			;	/* no action, empty block */
		    else {
			int	i;
			char c;

			i = BF_BLOCK_DATA;
			if (  pBuf->firstByte != BF_BLOCK_DATA &&
			      pBuf->firstByte != 0  ) {
			    (void)printf("*** overflow info: ***\n");
			    if (pBuf->firstByte < BF_BLOCK_DATA) {
				/* block contains only overflow; print all */
				for (; i<=pBuf->lastByte; i++) {
				    if ((c = ((char *)pBuf)[i]) > 0)
					putc(c, stdout);
				}
			    }
			    else {
				/* block has overflow plus other; print part*/
				for (; i<pBuf->firstByte; i++) {
				    if ((c = ((char *)pBuf)[i]) > 0)
					putc(c, stdout);
				}
			    }
			    (void)printf("*** end of overflow info ***\n");
			}
			for (; i<=pBuf->lastByte; i++) {
			    /* print rest of block, if any */
			    if ((c = ((char *)pBuf)[i]) > 0)
				putc(c, stdout);
			}
		    }
		}
	    }
	}
	else if (strcmp(command, "release") == 0) {
/*----------------------------------------------------------------------------
* release - release 'current block' back to free block list for the 'current
*    file'
*---------------------------------------------------------------------------*/
	    stat = bfRelease(pDesc, pBuf, blockNum);
	    (void)printf("%d\n", stat);
	}
	else if (strcmp(command, "store") == 0) {
/*----------------------------------------------------------------------------
* store - store information into the 'current block'
*	store flink blockNum stores the specified block number as the flink
*			in the 'current block'.  No checking of blockNum is
*			done.
*	store data characters... stores the string of characters into the
*			'current block', starting at the byte position
*			following the byte pointed to by lastByte; lastByte
*			is updated following the operation.  If there isn't
*			room enough in the block to completely contain the
*			string of characters, no store is done.  If this is
*			the first string stored into the block, then also
*			set firstByte to beginning of string in block.
*
* BUGS
* o	a store can be done immediately after open or create, without having
*	done a read or acquire.
*---------------------------------------------------------------------------*/
	    if (!BfWriteable(pDesc)) {
		(void)printf("file opened O_RDONLY--can't store\n");
	    }
	    else if (strcmp(cmdArg1, "data") == 0) {
		int i,n;

		n = strlen(cmdLine) - 11;	/* data length, excluding \0 */
		if (n >= 2) {
		    if ((pBuf->lastByte + n) < BfB0BlockSize(pDesc)) {
			if (pBuf->firstByte < BF_BLOCK_DATA)
			    pBuf->firstByte = pBuf->lastByte+1;
			strncpy(&((char*)pBuf)[pBuf->lastByte+1],
					&cmdLine[11], n);
			pBuf->lastByte += n;
		    }
		    else
			(void)printf("not enough room in block for data\n");
		}
		else {
		    (void)printf("syntax error; ");
		    (void)printf("usage: store data string...\n");
		}
	    }
	    else if (strcmp(cmdArg1, "flink") == 0) {
		int i;
		long flink;

		if ((i=sscanf(cmdArg2, "%d", &flink)) == 1)
		    pBuf->flink = flink;
		else {
		    (void)printf("syntax error; ");
		    (void)printf("usage: store flink blockNum\n");
		}
	    }
	    else {
		(void)printf("syntax error; ");
		(void)printf("usage: store flink blockNum		or\n");
		(void)printf("              ");
		(void)printf("       store data string...\n");
	    }
	}
	else if (strcmp(command, "unlock") == 0) {
/*----------------------------------------------------------------------------
* unlock - remove the file write lock for the named file; this doesn't affect
*    the 'current file'
*---------------------------------------------------------------------------*/
	    if (strlen(cmdArg1) > 0) {
		char lockName[GEN_FNAME_DIM+5]; /* name of lock file */

		(void)sprintf(lockName, "%s.lock", cmdArg1);
		if ((stat = unlink(lockName)) != 0)
		    perror("bfTest: removing lock file");
	    }
	    else {
		(void)printf("syntax error; ");
		(void)printf("usage: unlock name\n");
	    }
	}
	else if (strcmp(command, "write") == 0) {
/*----------------------------------------------------------------------------
* write - write the 'current block' into the 'current file'
*---------------------------------------------------------------------------*/
	    stat = bfWrite(pDesc, pBuf, blockNum, 0);
	    (void)printf("%d\n", stat);
        }
/*----------------------------------------------------------------------------
*
*---------------------------------------------------------------------------*/
	else {
	    (void)printf("Legal commands are:\n");
	    (void)printf("	acquire\n");
	    (void)printf("	close\n");
	    (void)printf("	create name nBlks blkSize flags maxBlks\n");
	    (void)printf("	info name\n");
	    (void)printf("	open name mode\n");
	    (void)printf("	quit\n");
	    (void)printf("	read blockNum\n");
	    (void)printf("	release\n");
	    (void)printf("	store data string\n");
	    (void)printf("	store flink blockNum\n");
	    (void)printf("	unlock name\n");
	    (void)printf("	write\n");
	}
	semGive(semWorkDone);
    }

    return;
}
