/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef INC_guiSubr_h
#define INC_guiSubr_h
/*	guiSubr.h,v 1.3 2002/08/02 15:37:55 jba Exp
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 */
/*+/mod***********************************************************************
* TITLE	guiSubr.h - definitions for guiSubr.c
*
* DESCRIPTION
*
*-***************************************************************************/

#define GUI_TDIM 120

typedef struct {
    Frame	baseFrame;	/* base frame for this whole shootin' match */
    Display	*pDisp;		/* X display pointer */
    Xv_server	server;		/* xview server handle */
    Frame	fsFrame;	/* command frame for file selector */
    Textsw	fsTextsw;	/* text subwindow for file selection */
    Panel_item	fsDir_PT;	/* directory panel text for file sel */
    Panel_item	fsFile_PT;	/* file panel text for file sel */
    Panel_item	fsStatus_PM;	/* panel message for file sel status */
    void	(*fsCallFn)();	/* user's callback function for file sel */
    void	*fsCallArg;	/* first arg for user's callback function */
    char	fsLsFile[GUI_TDIM];/* file to contain file sel listing */
    char	host[GUI_TDIM];	/* name of host machine, or X display */
    char	user[GUI_TDIM];	/* name of user */
    char	pwd[GUI_TDIM];	/* present working directory */
} GUI_CTX;

typedef struct {
    GUI_CTX	*pGuiCtx;	/* pointer to gui context */
    Frame	print_CF;	/* command frame for printer setup */
    Panel_item	printer_PT;	/* printer name panel text item */
    Panel_item	landscape_PCS;	/* PANEL_CHOICE_STACK item for printer mode */
    Panel_item	prog_PCS;	/* PANEL_CHOICE_STACK item for print program */
    Panel_item	point_PT;	/* PANEL_TEXT item for point size */
    int		pointSize;
} GUI_PRT;

typedef struct {
    GUI_CTX	*pGuiCtx;	/* pointer to gui context */
    GUI_PRT	*pPrtCtx;	/* pointer to printer context */
    Frame	edit_CF;	/* the actual command frame */
    char	title[GUI_TDIM];/* title for command frames */
    char	path[GUI_TDIM*2];/* full path name */
    Panel_item	dir_PT;		/* directory panel text item */
    char	dir[GUI_TDIM];	/* directory */
    Panel_item	file_PT;	/* file panel text item */
    char	file[GUI_TDIM];	/* file name */
    Textsw	text_TSW;	/* text subwindow for browsing/editing */
    int		updFlag;	/* 0,1 for read-only/update */
    char	lockFile[GUI_TDIM*2+5];/* path of lockfile */
    int		loaded;		/* 0,1 for not loaded, loaded */
    Panel_item	status_PM;	/* status panel message item */
} GUI_EDIT;

/*-----------------------------------------------------------------------------
* XV_KEY_DATA keys
*----------------------------------------------------------------------------*/
enum guiDataKeys {
    GUI_PCTX=1,		/* pointer to gui context */
    GUI_CF,		/* command frame for guiCFshow_mi */
    GUI_PEDIT,		/* pointer to gui editor context */
    GUI_PPRT,		/* pointer to gui printer context */
    GUI_INT,		/* integer value */
};
/*-----------------------------------------------------------------------------
* notifier client numbers; user client numbers are expected to be
* in the range 100 to 200
*----------------------------------------------------------------------------*/
enum guiClientNums {
    GUI_FILE_SEL_CLIENT=1,
    GUI_HOSTNAME_CLIENT,
    GUI_PRINT_CLIENT,
};


Textsw guiBrowser();
Panel_item guiButton();
void guiButtonCenter();
Canvas guiCanvas();
void guiCFdismiss_xvo();
void guiCFshow(), guiCFshowPin();
void guiCFshow_mi(), guiCFshowPin_mi(), guiCFshow_xvo(), guiCFshowPin_xvo();
Panel_item guiCheckbox();
void guiCheckboxItem();
#ifdef __STDC__
Panel_item guiChoice(char *label, ...);
void guiEditorPrintf(GUI_EDIT *pEdit, char *fmt, ...);
void guiEditorStatusPrintf(GUI_EDIT *pEdit, char *fmt, ...);
void guiMessagePrintf(Xv_opaque item, char *fmt, ...);
void guiNoticePrintf(GUI_CTX *pCtx, char *fmt, ...);
void guiTextFieldPrintf(Xv_opaque item, char *fmt, ...);
void guiTextswPrintf(Textsw textsw, char *fmt, ...);

#else
Panel_item guiChoice();
void guiEditorPrintf();
void guiEditorStatusPrintf();
void guiMessagePrintf();
void guiNoticePrintf();
void guiTextFieldPrintf();
void guiTextswPrintf();
#endif
GUI_EDIT *guiEditor();
void guiEditorLoadFile();
void guiEditorNewEntry_pb();
void guiEditorShowCF(), guiEditorShowCF_xvo(), guiEditorShowCF_mi();
char *guiEditorGets();
int guiEditorPuts();
void guiEditorReset();

void guiFileSelect();
char *guiGetNameFromSeln();
char *guiGetSelnInTextsw();
Frame guiFrame(), guiFrameCmd();
Icon guiIconCreate();
Panel guiInit();
int guiLock();
Menu guiMenu();
Menu_item guiMenuItem();
Panel_item guiMessage();
void guiNotice(), guiNoticeFile(), guiNoticeName();
GUI_PRT *guiPrinter();
void guiPrinterShowCF_mi(), guiPrinterShowCF_xvo(), guiPrintFile();
void guiPrintGetCmd();
void guiShellCmd();
void guiShellCmd_execv_argv();
Panel_item guiTextField();
char *guiTextswGets();
int guiTextswPuts();
void guiTextswReset();
void guiTimer();

#endif /* INC_guiSubr_h */
