/*	@(#)arChanIOTest.c	1.2 9/20/91
 *	Author:	Roger A. Cole
 *	Date:	03-09-90
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	03-09-90	rac	initial version
 * .02	06-19-91	rac	installed in SCCS
 * .03	09-19-91	rac	make semWorkDone semaphore global
 * arChanIOTest.c,v
 * Revision 1.2  1994/05/06 19:06:02  mrk
 * conversion from sccs to cvs
 *
 *
 * make options
 *	-DvxWorks	makes a version for VxWorks
 *	-DNDEBUG	don't compile assert() checking
 *      -DDEBUG         compile various debug code, including checks on
 *                      malloc'd memory
 */
/*+/mod***********************************************************************
* TITLE	arChanIOTest.c - AR channel data file test bed
*
* DESCRIPTION
*	
*-***************************************************************************/

#include <arAccessLib.h>
#ifndef INC_genDefs_h
#   include <genDefs.h>
#endif
#ifndef INC_tsDefs_h
#   include <tsDefs.h>
#endif
#ifndef INC_bfDefs_h
#   include <bfDefs.h>
#endif
#ifndef INC_genTasks
#   include <genTasks.h>
#endif
#include <cadef.h>

#ifdef vxWorks
#   include <vxWorks.h>
#   include <ioLib.h>		/* for O_RDWR and O_RDONLY definitions */
#   include <stdioLib.h>
#   define MAXPRIO 200
#else
#   include <stdio.h>
#   include <sys/file.h>	/* for O_RDWR and O_RDONLY definitions */
#   define MAXPRIO 50
#endif

SEM_ID      semWorkDone;    /* semaphore for 'work completed' */


#ifndef vxWorks
    main()
    {
	int	stat=OK;	/* status */

        (void)pod_setmaxpri(MAXPRIO);
        lwp_setstkcache(100000, 9);

	stat = arChanIO();

	if (stat == OK)
	    exit(0);
	else
	    exit(1);
    }
#endif

/*+/subr**********************************************************************
* NAME	arChanIO - driver for channel data file test routine
*
* DESCRIPTION
*
* RETURNS
*
* BUGS
* o	text
*
* SEE ALSO
*
* EXAMPLE
*
*-*/
int
arChanIO()
{
    int         stat;           /* status from calls */
    int         retStat=OK;     /* return status to caller */
    TASK_ID     arChanIO1Tid;	/* task ID of arChanIO1 */
    int         arChanIO1();
    char        arChanIOCmd[80];/* buffer for command data */

    (void)clearerr(stdin);

    semInit(semWorkDone);

    stat = taskPrioritySet(SELF, GenLowerPriority(MAXPRIO, 1));
    arChanIO1Tid = taskSpawn("arChanIO1", MAXPRIO, VX_STDIO | VX_FP_TASK, 15000,
                                        arChanIO1, arChanIOCmd);
    if (GenTaskNull(arChanIO1Tid)) {
        (void)printf("arChanIO: couldn't spawn arChanIO1\n");
        return 1;
    }

    while ((printf("arChanIO: "), fgets(arChanIOCmd, 80, stdin)) != NULL) {
        if (strncmp(arChanIOCmd, "quit", 4) == 0)
            goto done;
        semGive(semWorkDone);
        taskResume(arChanIO1Tid);
        semTake(semWorkDone);
    }
    (void)printf("\n");
done:
#ifdef vxWorks
    checkStack(arChanIO1Tid);     /* find out stack usage for arChanIO1 */
#endif

    (void)strcpy(arChanIOCmd, "quit");
    semGive(semWorkDone);
    taskResume(arChanIO1Tid);

    (void)clearerr(stdin);

    return OK;
}

/*+/subr**********************************************************************
* NAME	arChanIO1 - test AR channel data files
*
* DESCRIPTION
*	Provide a 'shell' for testing AR channel data files
*
* RETURNS
*	OK, or
*	ERROR
*
*-*/
int
arChanIO1(cmdBuf)
char    *cmdBuf;        /* buffer for command data */
{
    char        cmdLine[80];    /* read buf: "command option value" */
    char	*pLine;
    char	*pCommand;
    char	delim;
    char        command[10];    /* command string */
    char        cmdArg1[40];    /* command option */
    char        cmdArg2[40];    /* command value */
    char	name[20];	/* file name */
    AR_CHAN_DESC *pArChanDesc;	/* pointer to channel descriptor */
    BF_DESC	*pBfDesc;	/* pointer to block file descriptor */
    int		(*pDumpFunc)();	/* pointer to bfDumpXxx() function */

    int         stat;           /* status return */
    int		i;
    int		openMode;	/* O_RDWR or O_RDONLY for opens */

    AR_CF_DESC	*pArCfDesc=NULL;/* pointer to channel file descriptor */
#define CD_DIM 200
    AR_CHAN_DESC *ppArChanDesc[CD_DIM];/* pointers to channel descriptors */
    int		chan= -1;	/* channel index into above array */

    BF_BLKNUM	blockNum;	/* blockNum of last read or acquired block */

    for (i=0; i<CD_DIM; i++)
	ppArChanDesc[i] = NULL;

    command[0] = '\0';

    while (strcmp(command, "quit") != 0) {
        taskSuspend(SELF);                      /* wait for input */
        semTake(semWorkDone);
        (void)strcpy(cmdLine, cmdBuf);
	pLine = cmdLine;

	stat = OK;

	command[0] = '\0';
	cmdArg1[0] = '\0';
	cmdArg2[0] = '\0';
	(void)sscanf(cmdLine, "%10s %40s %40s", command, cmdArg1, cmdArg2);
	nextANField(&pLine, &pCommand, &delim);
	if (strlen(command) == 0)
/*----------------------------------------------------------------------------
* blank line - ignore
*---------------------------------------------------------------------------*/
	    ;               /* no action */
/*----------------------------------------------------------------------------
* chanAdd - add a new channel to the file
*---------------------------------------------------------------------------*/
	else if (strcmp(command, "chanAdd") == 0) {
	    if (sscanf(cmdArg1, "%d", &chan) != 1) {
		(void)printf("arChanIO: illegal channel index\n");
		stat = ERROR;
	    }
	    if (stat == OK && (chan<0 || chan>=CD_DIM) ) {
		(void)printf("arChanIO: illegal channel index\n");
		stat = ERROR;
	    }
	    if (stat == OK && ppArChanDesc[chan] != NULL) {
		(void)printf("arChanIO: %d already open to %s\n",
			chan, ArCDChanHdr(ppArChanDesc[chan]).name);
		stat = ERROR;
	    }
	    if (strlen(cmdArg2) <= 0)
		stat = ERROR;
	    else if (strlen(cmdArg2) >= AR_NAME_DIM) {
		(void)printf(
			"arChanIO: channel name too long; %d is max length\n",
			AR_NAME_DIM-1);
		stat = ERROR;
	    }
	    if (stat == OK) {
		if (pArCfDesc == NULL) {
		    (void)printf("arChanIO: file not open\n");
		    stat = ERROR;
		}
	    }
	    if (stat == OK) {
		stat = arCFChanFind(pArCfDesc, cmdArg2);
		if (stat == OK) {
		    (void)printf("arChanIO: channel already exists\n");
		    stat = ERROR;
		}
		else
		    stat = OK;
	    }
	    if (stat == OK) {
		ppArChanDesc[chan] = arCFChanOpen(pArCfDesc, cmdArg2, O_RDWR);
		if (ppArChanDesc[chan] == NULL) {
		    (void)printf("arChanIO: error creating %s\n", cmdArg2);
		    stat = ERROR;
		}
	    }
	    if (stat == OK) {
		union db_access_val accVal;
		struct event_handler_args arg;

		pArChanDesc = ppArChanDesc[chan];
		arg.chid = NULL;
		arg.type = DBR_GR_FLOAT;
		arg.count = 1;
		arg.dbr = (void *)&accVal;

		accVal.gfltval.status			= 0;
		accVal.gfltval.severity			= 0;
		accVal.gfltval.precision		= 0;
		(void)strcpy(accVal.gfltval.units, "furlong");
		accVal.gfltval.upper_disp_limit		= 0.;
		accVal.gfltval.lower_disp_limit		= 0.;
		accVal.gfltval.upper_alarm_limit	= 0.;
		accVal.gfltval.upper_warning_limit	= 0.;
		accVal.gfltval.lower_warning_limit	= 0.;
		accVal.gfltval.lower_alarm_limit	= 0.;
		accVal.gfltval.value			= 0.;

		stat = arCFChanWriteGR(pArChanDesc, &arg);

	    }
	    if (stat != OK) {
		(void)printf("arChanIO: error adding channel\n");
		chan = -1;
	    }
	    else
		(void)printf("	chanStore value [repCount]\n");
	}
/*----------------------------------------------------------------------------
* chanClose - close a channel
*---------------------------------------------------------------------------*/
	else if (strcmp(command, "chanClose") == 0) {
	    if (sscanf(cmdArg1, "%d", &chan) != 1) {
		(void)printf("arChanIO: illegal channel index\n");
		stat = ERROR;
	    }
	    if (stat == OK && (chan<0 || chan>=CD_DIM) ) {
		(void)printf("arChanIO: illegal channel index\n");
		stat = ERROR;
	    }
	    if (stat == OK && ppArChanDesc[chan] == NULL) {
		(void)printf("arChanIO: %d not open\n", chan);
		stat = ERROR;
	    }
	    if (stat == OK) {
		if (pArCfDesc == NULL) {
		    (void)printf("arChanIO: file not open\n");
		    stat = ERROR;
		}
	    }
	    if (stat == OK) {
		if (arCFChanClose(&ppArChanDesc[chan]) != OK) {
		    (void)printf("arChanIO: error closing %s\n",
					ArCDChanHdr(ppArChanDesc[chan]).name);
		    stat = ERROR;
		}
	    }
	    chan = -1;
	}
/*----------------------------------------------------------------------------
* chanDel - delete channel from MI and close the channel
*---------------------------------------------------------------------------*/
	else if (strcmp(command, "chanDel") == 0) {
	    if (sscanf(cmdArg1, "%d", &chan) != 1) {
		(void)printf("arChanIO: illegal channel index\n");
		stat = ERROR;
	    }
	    if (stat == OK && (chan<0 || chan>=CD_DIM) ) {
		(void)printf("arChanIO: illegal channel index\n");
		stat = ERROR;
	    }
	    if (stat == OK && ppArChanDesc[chan] == NULL) {
		(void)printf("arChanIO: %d not open\n", chan);
		stat = ERROR;
	    }
	    if (stat == OK) {
		if (pArCfDesc == NULL) {
		    (void)printf("arChanIO: file not open\n");
		    stat = ERROR;
		}
	    }
	    if (stat == OK) {
		if (arCFChanDel(&ppArChanDesc[chan]) != OK) {
		    (void)printf("arChanIO: error deleting %s\n",
					ArCDChanHdr(ppArChanDesc[chan]).name);
		    stat = ERROR;
		}
	    }
	    chan = -1;
	}
	else if (strcmp(command, "chanChar") == 0) {
/*----------------------------------------------------------------------------
* chanChar - dump a block as text, or print indexHead and indexTail of
*	currently selected channel
*
*    chanChar [blockNum]
*
*---------------------------------------------------------------------------*/
	    pDumpFunc = bfDumpChar;
	    goto chanDump;
	}
	else if (strcmp(command, "chanHex") == 0) {
/*----------------------------------------------------------------------------
* chanHex - dump a block in hex, or print indexHead and indexTail of
*	currently selected channel
*
*    chanHex [blockNum]
*
*---------------------------------------------------------------------------*/
	    pDumpFunc = bfDumpHex;

chanDump:
	    pBfDesc = pArCfDesc->pBfDesc;

	    if (sscanf(cmdArg1, "%d", &blockNum) != 1)
		blockNum = 0;
	    else if (blockNum == 0) {
		(void)printf("arChanIO: blockNum must be > 0\n");
		stat = ERROR;
	    }
	    if (stat == OK) {
		if (pArCfDesc == NULL) {
		    (void)printf("arChanIO: file not open\n");
		    stat = ERROR;
		}
	    }
	    if (stat == OK) {
		if (blockNum == 0) {
		    if (chan < 0) {
			(void)printf("arChanIO: channel index not selected\n");
			stat = ERROR;
		    }
		    if (stat == OK) {
			if ((pArChanDesc = ppArChanDesc[chan]) == NULL) {
			    (void)printf("arChanIO: %d not open\n", chan);
			    stat = ERROR;
			}
			else {
			    (void)printf("indexHead=%d indexTail=%d\n",
				ArCDChanHdr(pArChanDesc).indexHead,
				ArCDChanHdr(pArChanDesc).indexTail);
			}
		    }
		}
		else
		    stat = (*pDumpFunc)(pBfDesc, blockNum);
	    }
	}
	else if (strcmp(command, "chanOpenR") == 0) {
/*----------------------------------------------------------------------------
* chanOpenR - open (for reading) a channel to the file
*---------------------------------------------------------------------------*/
	    openMode = O_RDONLY;
	    goto chanOpen;
	}
	else if (strcmp(command, "chanOpenW") == 0) {
/*----------------------------------------------------------------------------
* chanOpenW - open (for writing) a channel to the file
*---------------------------------------------------------------------------*/
	    openMode = O_RDWR;
chanOpen:
	    if (sscanf(cmdArg1, "%d", &chan) != 1) {
		(void)printf("arChanIO: illegal channel index\n");
		stat = ERROR;
	    }
	    if (stat == OK && (chan<0 || chan>=CD_DIM) ) {
		(void)printf("arChanIO: illegal channel index\n");
		stat = ERROR;
	    }
	    if (stat == OK && ppArChanDesc[chan] != NULL) {
		(void)printf("arChanIO: %d already open to %s\n",
			chan, ArCDChanHdr(ppArChanDesc[chan]).name);
		stat = ERROR;
	    }
	    if (strlen(cmdArg2) <= 0)
		stat = ERROR;
	    else if (strlen(cmdArg2) >= AR_NAME_DIM) {
		(void)printf(
			"arChanIO: channel name too long; %d is max length\n",
			AR_NAME_DIM-1);
		stat = ERROR;
	    }
	    if (stat == OK) {
		if (pArCfDesc == NULL) {
		    (void)printf("arChanIO: file not open\n");
		    stat = ERROR;
		}
	    }
	    if (stat == OK) {
		if (arCFChanFind(pArCfDesc, cmdArg2) != OK) {
		    (void)printf("arChanIO: channel doesn't exist\n");
		    stat = ERROR;
		}
	    }
	    if (stat == OK) {
		ppArChanDesc[chan] = arCFChanOpen(pArCfDesc, cmdArg2, openMode);
		if (ppArChanDesc[chan] == NULL) {
		    (void)printf("arChanIO: error opening %s\n", cmdArg2);
		    stat = ERROR;
		}
	    }
	    if (stat != OK) {
		chan = -1;
	    }
	    else {
		if (openMode == O_RDONLY)
		    (void)printf("	chanRead [repCount]\n");
		else
		    (void)printf("	chanStore value [repCount]\n");
	    }
	}
	else if (strcmp(pCommand, "chanDescDump") == 0) {
/*-----------------------------------------------------------------------------
* chanDescDump - dump some information about the channel descriptor
*----------------------------------------------------------------------------*/
	    AR_CHAN_DESC *pArChanDesc;	/* pointer to channel descriptor */
	    char	stampText[28];

	    if (chan < 0) {
		(void)printf("arChanIO: channel index not selected\n");
		stat = ERROR;
	    }
	    else if (stat == OK && ppArChanDesc[chan] == NULL) {
		(void)printf("arChanIO: %d not open\n");
		stat = ERROR;
	    }
	    if (stat == OK)
		pArChanDesc = ppArChanDesc[chan];
	    if (stat == OK) {
		(void)printf("chanHdr=blk%d[%d]  curIndx=blk%d[%d]\n",
		    pArChanDesc->pMIBuf->blkNum, pArChanDesc->hdrNum,
		    pArChanDesc->pIndexBuf->blkNum, pArChanDesc->datInfNum);
		(void)printf("contents of curIndx=0x%08x, 0x%08x, 0x%08x\n",
		    ArCDDatInfo(pArChanDesc).dataBlock,
		    ArCDDatInfo(pArChanDesc).stamp.secPastEpoch,
		    ArCDDatInfo(pArChanDesc).stamp.nsec);
		(void)printf("stamp of curIndx=%s\n",
		    tsStampToText(&ArCDDatInfo(pArChanDesc).stamp,
						TS_TEXT_MMDDYY, stampText));
		(void)printf("curData=blk%d at 0x%x; pData(relative)=%d\n",
		    pArChanDesc->pDataBuf->blkNum, pArChanDesc->pDataBuf,
		    pArChanDesc->pData-(char *)pArChanDesc->pDataBuf);
		(void)printf("stamp of last data transferred=%s\n",
		    tsStampToText(&pArChanDesc->timeStamp, TS_TEXT_MMDDYY,
							stampText));
	    }
	}
	else if (strcmp(command, "chanPos") == 0) {
/*-----------------------------------------------------------------------------
* chanPos - position by time within a channel stream
*
*    chanPos  timeStamp
*----------------------------------------------------------------------------*/
	    AR_CHAN_DESC *pArChanDesc;	/* pointer to channel descriptor */
	    TS_STAMP	posStamp;	/* desired time stamp, if pos==1 */

	    if (chan < 0) {
		(void)printf("arChanIO: channel index not selected\n");
		stat = ERROR;
	    }
	    else if (stat == OK && ppArChanDesc[chan] == NULL) {
		(void)printf("arChanIO: %d not open\n");
		stat = ERROR;
	    }
	    if (stat == OK)
		pArChanDesc = ppArChanDesc[chan];

	    if (tsTextToStamp(&posStamp, &pLine) != S_ts_OK) {
		(void)printf("arChanIO: error reading desired time\n");
		stat = ERROR;
	    }
	    if (stat == OK) {
		if ((stat = arCFChanPosition(pArChanDesc, &posStamp)) != OK) {
		    (void)printf("arChanIO: error positioning to time\n");
		    stat = ERROR;
		}
	    }
	}
	else if (strcmp(command, "chanRead") == 0 ||
	         strcmp(command, "chanSkip") == 0) {
/*----------------------------------------------------------------------------
* chanRead - read a value from the channel stream
*
*    chanRead [repCount]
*    chanSkip [repCount]
*
*    the next value is (or several values are) read from the channel,
*    depending on whether repCount is specified.
*---------------------------------------------------------------------------*/
	    AR_CHAN_DESC *pArChanDesc;	/* pointer to channel descriptor */
	    int		count;
	    char	stampText[28];	/* time stamp */
	    int		skip=0;

	    if (chan < 0) {
		(void)printf("arChanIO: channel index not selected\n");
		stat = ERROR;
	    }
	    else if (stat == OK && ppArChanDesc[chan] == NULL) {
		(void)printf("arChanIO: %d not open\n");
		stat = ERROR;
	    }
	    if (stat == OK)
		pArChanDesc = ppArChanDesc[chan];

	    if (strcmp(command, "chanSkip") == 0) {
		skip = 1;
		if (sscanf(cmdArg1, "%d", &count) != 1) {
		    (void)printf("arChanIO: error reading count\n");
		    stat = ERROR;
		}
		if (count <= 0) {
		    (void)printf("arChanIO: count must be greater than 0\n");
		    stat = ERROR;
		}
	    }
	    else {
		if (sscanf(cmdArg1, "%d", &count) != 1) {
		    (void)printf("arChanIO: error reading count\n");
		    stat = ERROR;
		}
		if (count <= 0) {
		    (void)printf("arChanIO: count must be greater than 0\n");
		    stat = ERROR;
		}
	    }

	    if (stat != OK)
		;	/* no action */
	    else {
		union db_access_val *pAccVal;
		chtype	type;
		int	elCount;
		int	i;

		type = dbf_type_to_DBR_TIME(ArCFChanFieldType(pArChanDesc));
		elCount = ArCFChanElementCount(pArChanDesc);
		pAccVal =
		    (union db_access_val *)malloc(dbr_size_n(type, elCount));
		assert(pAccVal != NULL);
		for (i=0; i<count; i++) {
		    stat = arCFChanRead(pArChanDesc, type, pAccVal, elCount);
		    if (stat != OK)
			break;

		    if (!skip || i>=count-1) {
			tsStampToText(&pAccVal->tstrval.stamp,
						    TS_TEXT_MMDDYY, stampText);
		    }
		    if (skip && i<count-1)
			;	/* no action */
		    else if (type == DBR_TIME_STRING) {
			assert(elCount == 1);
			(void)printf("%s %s\n", stampText,
			    				pAccVal->tstrval.value);
		    }
		    else if (type == DBR_TIME_SHORT && elCount > 1) {
			short	*pShort;

			pShort = &pAccVal->tshrtval.value;
			(void)printf("%s\n", stampText);
			for (i=0; i<elCount;) {
			    if (i+9 < elCount) {
                    (void)printf("%  5d:%6d%6d%6d%6d%6d%6d%6d%6d%6d%6d\n", i,
                        *pShort++, *pShort++, *pShort++, *pShort++, *pShort++,
                        *pShort++, *pShort++,*pShort++,  *pShort++, *pShort++);
				i += 10;
			    }
			    else
				(void)printf("  %5d:%6d\n", i++, *pShort++);
			}
		    }
		    else if (type == DBR_TIME_SHORT && elCount == 1) {
			(void)printf("%s %d\n", stampText,
			    pAccVal->tshrtval.value);
		    }
		    else if (type == DBR_TIME_FLOAT) {
			assert(elCount == 1);
			(void)printf("%s %f\n", stampText,
			    pAccVal->tfltval.value);
		    }
		}
		free(pAccVal);
		if ((pArChanDesc->flags & AR_CDESC_EOF) != 0) {
		    (void)printf("EOF\n");
		    stat = OK;
		}
		else if (stat != OK)
		    (void)printf("arChanIO: error on read\n");
	    }
	}
	else if (strcmp(command, "chanSelect") == 0) {
/*----------------------------------------------------------------------------
* chanSelect - select a channel for store, read, etc.
*
*    chanSelect [index]
*
*    if index isn't specified, a list of indexes is printed, indicating
*    which ones are in use.  The 'current index', if any, is flagged with
*    a * in the list.
*---------------------------------------------------------------------------*/
	    if (cmdArg1[0] != '\0') {
		if (sscanf(cmdArg1, "%d", &chan) != 1) {
		    (void)printf("arChanIO: illegal channel index\n");
		    stat = ERROR;
		}
		if (stat == OK && (chan<0 || chan>=CD_DIM) ) {
		    (void)printf("arChanIO: illegal channel index\n");
		    stat = ERROR;
		}
		if (stat == OK && ppArChanDesc[chan] == NULL) {
		    (void)printf("arChanIO: %d not open\n");
		    stat = ERROR;
		}
		else {
		    (void)printf("%d is now selected, \"%s\"\n", chan,
			    ArCDChanHdr(ppArChanDesc[chan]).name);
		}
		if (stat != OK)
		    chan = -1;
	    }
	    else {
		(void)printf("open channels (index, mode, chanName) are:\n");
		for (i=0; i<CD_DIM; i++) {
		    if (ppArChanDesc[i] != NULL) {
			if (i == chan)
			    (void)printf("*");
			else
			    (void)printf(" ");
			if ((ppArChanDesc[i]->flags & AR_CDESC_WRITE) != 0)
			    (void)printf("W");
			else
			    (void)printf("R");

			(void)printf("%d: \"%s\"\n", i,
				    ArCDChanHdr(ppArChanDesc[i]).name);
		    }
		}
	    }
	}
	else if (strcmp(command, "chanStore") == 0) {
/*----------------------------------------------------------------------------
* chanStore - store a value into the channel stream
*
*    chanStore value [repCount]
*
*    the specified value is stored for the channel, either once, or to
*    simulate as many samples as indicated by repCount.  For repeated
*    stores, the simulated samples are 1 ms apart, and the value is
*    incremented by 1. for each simulated sample.
*
*    Only scalar DBF_FLOAT channels are presently handled.
*---------------------------------------------------------------------------*/
	    AR_CHAN_DESC *pArChanDesc;	/* pointer to channel descriptor */
	    int		count;
	    TS_STAMP	delta;		/* delta time between samples */
	    union db_access_val unionBuf;
	    struct event_handler_args arg;

	    if (chan < 0) {
		(void)printf("arChanIO: channel index not selected\n");
		stat = ERROR;
	    }
	    if (stat == OK && ppArChanDesc[chan] == NULL) {
		(void)printf("arChanIO: %d not open\n");
		stat = ERROR;
	    }
	    pArChanDesc = ppArChanDesc[chan];

	    if (sscanf(cmdArg2, "%d", &count) != 1)
		count = 1;
	    if (count <= 0 || count > 100) {
		(void)printf("arChanIO: count must be between 1 and 100\n");
		stat = ERROR;
	    }

	    (void)tsLocalTime(&(unionBuf.tstrval.stamp));
	    (void)tsLocalTime(&delta);
	    delta.secPastEpoch = 0;
	    delta.nsec = 1000000;	/* 1 milli-sec worth of nsec */

	    if (stat != OK)
		;	/* no action */
	    else if (1 || ArCDChanHdr(pArChanDesc).chanType == DBF_FLOAT) {
#define FLOAT_BUF unionBuf.tfltval

		if (sscanf(cmdArg1, "%f", &FLOAT_BUF.value) != 1) {
		    (void)printf("arChanIO: illegal value\n");
		    stat = ERROR;
		}
		if (stat == OK) {

		    arg.type = DBR_TIME_FLOAT;
		    arg.count = 1;
		    arg.dbr = (void *)&FLOAT_BUF;

		    FLOAT_BUF.status = 0;
		    FLOAT_BUF.severity = 0;
		    for (i=0; i<count; i++) {
			if (arCFChanWrite(pArChanDesc, &arg) != OK)
			    break;

			FLOAT_BUF.value += 1.;
			TsAddStamps(&FLOAT_BUF.stamp, &FLOAT_BUF.stamp, &delta);
		    }
		}
	    }
	}
	else if (strcmp(command, "close") == 0) {
/*----------------------------------------------------------------------------
* close - close the file
*---------------------------------------------------------------------------*/
	    if (pArCfDesc == NULL) {
		(void)printf("arChanIO: file not open\n");
		stat = ERROR;
	    }
	    else {
		stat = arCFClose(&pArCfDesc);
		for (i=0; i<CD_DIM; i++) {
		    ppArChanDesc[i] = NULL;
		}
	    }
	}
	else if (strcmp(command, "create") == 0) {
/*----------------------------------------------------------------------------
* create - create and initialize an empty AR channel data file, with maximum
*	size constrained by the specified number of bytes.
*---------------------------------------------------------------------------*/
	    int		nBytes;		/* number of bytes, for 'create' */

	    if (pArCfDesc != NULL) {
		(void)printf("arChanIO: a file is already open\n");
		stat = ERROR;
	    }
	    if (strlen(cmdArg1) >= GEN_FNAME_DIM) {
		(void)printf("arChanIO: file name too long\n");
		stat = ERROR;
	    }
	    else if (strlen(cmdArg1) <= 0)
		stat = ERROR;

	    (void)sscanf(cmdArg2, "%d", &nBytes);
	    if (stat == OK) {
		if ((pArCfDesc = arCFCreate(cmdArg1, nBytes)) == NULL) {
		    (void)printf("arChanIO: can't create %s\n", cmdArg1);
		    stat = ERROR;
		}
	    }
	}
	else if (strcmp(command, "info") == 0) {
/*----------------------------------------------------------------------------
* info - print some information about the file
*---------------------------------------------------------------------------*/
	    if (pArCfDesc == NULL) {
		(void)printf("arChanIO: file not open\n");
		stat = ERROR;
	    }
	    else {
		(void)printf("arChanIO: MIhead, MItail, MINblk = %d %d %d\n",
		    ArB0MIHead(pArCfDesc),
		    ArB0MITail(pArCfDesc), ArB0MINblk(pArCfDesc));
		(void)printf("arChanIO: MIFree_blkNum, _hdrNum = %d %d\n",
		    ArB0MIFree_blkNum(pArCfDesc),
		    ArB0MIFree_hdrNum(pArCfDesc));
		if (ArB0MIHead(pArCfDesc) != 0) {
		    AR_MI_BUF *pMICurrent;
		    AR_CHAN_HDR *pChanHdr;

		    assert(ArB0MIHead(pArCfDesc) > 0);
		    assert(ArB0MITail(pArCfDesc) > 0);
		    assert(ArB0MINblk(pArCfDesc) > 0);
		    assert(pArCfDesc->pMIHead != NULL);
		    assert(pArCfDesc->pMITail != NULL);

		    pMICurrent = pArCfDesc->pMIHead;
		    while (pMICurrent != NULL) {
			int i;

			(void)printf("%4d %4d\n",
			    pMICurrent->blkNum, pMICurrent->bfInfo.flink);

			for (i=0; i<AR_CF_NHDR; i++) {
			    pChanHdr = &pMICurrent->chanHdr[i];
			    if (pChanHdr->name[0] == '\0') {
				(void)printf("\t%2d  free\n", i);
			    }
			    else {
				(void)printf("\t%2d  %s\n", i,
					pChanHdr->name);
			    }
			}
			pMICurrent = pMICurrent->pNextMI;
		    }
		}
	    }
	}
	else if (strcmp(command, "open") == 0) {
/*----------------------------------------------------------------------------
* open - open a file in O_RDWR mode
*---------------------------------------------------------------------------*/
	    if (pArCfDesc != NULL) {
		(void)printf("arChanIO: a file is already open\n");
		stat = ERROR;
	    }
	    if (strlen(cmdArg1) >= GEN_FNAME_DIM) {
		(void)printf("arChanIO: file name too long\n");
		stat = ERROR;
	    }
	    else if (strlen(cmdArg1) <= 0)
		stat = ERROR;

	    if (stat == OK) {
		if ((pArCfDesc = arCFOpen(cmdArg1, O_RDWR)) == NULL) {
		    (void)printf("arChanIO: can't open %s\n", cmdArg1);
		    stat = ERROR;
		}
	    }
	}
	else if (strcmp(command, "quit") == 0) {
/*----------------------------------------------------------------------------
* quit
*---------------------------------------------------------------------------*/
	    ;			/* no action */
	}
	else if (strcmp(command, "showMI") == 0) {
/*----------------------------------------------------------------------------
* showMI - print some information about the master index
*---------------------------------------------------------------------------*/
	    TS_STAMP	oldestStamp;
	    char	oldestStampText[28];
	    TS_STAMP	newestStamp;
	    char	newestStampText[28];

	    if (pArCfDesc == NULL) {
		(void)printf("arChanIO: file not open\n");
		stat = ERROR;
	    }
	    else {
		if (ArB0MIHead(pArCfDesc) != 0) {
		    AR_MI_BUF *pMICurrent;
		    AR_CHAN_HDR *pChanHdr;

		    assert(ArB0MIHead(pArCfDesc) > 0);
		    assert(ArB0MITail(pArCfDesc) > 0);
		    assert(ArB0MINblk(pArCfDesc) > 0);
		    assert(pArCfDesc->pMIHead != NULL);
		    assert(pArCfDesc->pMITail != NULL);

		    pMICurrent = pArCfDesc->pMIHead;
		    while (pMICurrent != NULL) {
			int i;


			(void)printf("%4d %4d\n",
			    pMICurrent->blkNum, pMICurrent->bfInfo.flink);
			for (i=0; i<AR_CF_NHDR; i++) {
			    pChanHdr = &pMICurrent->chanHdr[i];

			    if (pChanHdr->name[0] == '\0') {
				(void)printf("%2d  free\n", i);
			    }
			    else {
				oldestStamp.secPastEpoch =
						pChanHdr->oldestSecPastEpoch,
				oldestStamp.nsec = 0;
				newestStamp.secPastEpoch =
						pChanHdr->newestSecPastEpoch,
				newestStamp.nsec = 0;
				(void)printf(
				    "%2d \"%s\" %d %s %s  %s\n", i,
				   pChanHdr->name,
				   pChanHdr->elCount,
				   dbf_type_to_text(pChanHdr->chanType),
				   tsStampToText(&oldestStamp, TS_TEXT_MMDDYY,
							oldestStampText),
				   tsStampToText(&newestStamp, TS_TEXT_MMDDYY,
							newestStampText) );
			    }
			}
			pMICurrent = pMICurrent->pNextMI;
		    }
		}
		else
		    (void)printf("no MI blocks\n");
	    }
	}
	else if (strcmp(command, "test") == 0) {
/*----------------------------------------------------------------------------
* test
*---------------------------------------------------------------------------*/
	    AR_CHAN_DESC *pArChanDesc;

	    pArChanDesc = ppArChanDesc[chan];
	    if (--pArChanDesc->remainCount >= 0)
		*(char *)(++(pArChanDesc->pData)) = 'a';
	    else {
		stat = arCF_DFlushWbufAndPutc('a', pArChanDesc);
	    }
	    ArCFModifySet(pArChanDesc->pArCfDesc, pArChanDesc->pDataBuf);
	}
	else if (strcmp(command, "testOpen") == 0) {
/*----------------------------------------------------------------------------
* testOpen
*---------------------------------------------------------------------------*/
	    char	chName[AR_NAME_DIM];

	    for (i=0; i<CD_DIM; i++) {
		(void)sprintf(chName, "%04d", i);
		if ((ppArChanDesc[i] = arCFChanOpen(pArCfDesc, chName, O_RDWR))
								== NULL) {
		    (void)fprintf(stderr,
				"arChanIO: error opening: %s\n", chName);
		}
	    }
	}
	else if (strcmp(command, "testWrite") == 0) {
/*----------------------------------------------------------------------------
* testWrite
*---------------------------------------------------------------------------*/
	    int nSamples;
	    TS_STAMP	timeStamp;	/* time stamp */
	    TS_STAMP	oldStamp;	/* old time stamp */
	    TS_STAMP	deltaStamp;	/* delta time between samples */
            union db_access_val unionBuf;
            struct event_handler_args arg;

	    if (sscanf(cmdArg1, "%d", &nSamples) != 1) {
		(void)printf("syntax error; ");
		stat = ERROR;
	    }
	    else {
		int j;

#define TEST_FLOAT unionBuf.tfltval

		arg.type = DBR_TIME_FLOAT;
		arg.count = 1;
		arg.dbr = (void *)&TEST_FLOAT;

		for (i=0; i<CD_DIM; i++) {
		    ArCDChanHdr(ppArChanDesc[i]).chanType = DBR_FLOAT;
		    ArCDChanHdr(ppArChanDesc[i]).elCount = 1;
		}

		TEST_FLOAT.value = 1.2;
		TEST_FLOAT.status = 0;
		TEST_FLOAT.severity = 0;
		deltaStamp.secPastEpoch = 0;
		deltaStamp.nsec = 10000000;	/* 10 milli-sec */
		oldStamp.secPastEpoch = 0;
		oldStamp.nsec = 0;

		for (j=0; j<nSamples; j++) {
		    (void)tsLocalTime(&timeStamp);
		    if (oldStamp.secPastEpoch == 0)
			oldStamp = timeStamp;
		    else if (TsCmpStampsGE(&oldStamp, &timeStamp))
			TsAddStamps(&timeStamp, &oldStamp, &deltaStamp);

		    TEST_FLOAT.stamp = timeStamp;

		    for (i=0; i<CD_DIM; i++) {

			stat = arCFChanWrite(ppArChanDesc[i], &arg);
			assert(stat == OK);
			/*  (void)testWrite(ppArChanDesc[i]);  */
		    }
		}
	    }
	}
	else if (strcmp(command, "tsFromText") == 0) {
/*----------------------------------------------------------------------------
* tsFromText
*---------------------------------------------------------------------------*/
	    TS_STAMP	stamp;
	    char	message[80];
	    char	*pMsg;

	    while (1) {
		(void)printf("type stamp (or RETURN): ");
		fflush(stdout);
		fgets(message, 80, stdin);
		if (strlen(message) <= 1)
		    break;
		pMsg = message;
		tsTextToStamp(&stamp, &pMsg);
		(void)printf("%12d %12d   0x%8x 0x%8x\n",
			stamp.secPastEpoch, stamp.nsec,
			stamp.secPastEpoch, stamp.nsec);
	    }
	}
	else if (strcmp(command, "tsToText") == 0) {
/*----------------------------------------------------------------------------
* tsToText
*---------------------------------------------------------------------------*/
	    TS_STAMP	stamp;
	    char	message[80];

	    while (1) {
		(void)printf("type sec and nsec in hex (or 0 0): ");
		fflush(stdout);
		if (fscanf(stdin,"%x", &stamp.secPastEpoch) != 1)
		    break;
		if (fscanf(stdin,"%x", &stamp.nsec) != 1)
		    break;
		if (stamp.secPastEpoch == 0 && stamp.nsec == 0)
		    break;
		tsStampToText(&stamp, TS_TEXT_MMDDYY, message);
		(void)printf("%s\n", message);
	    }
	}
	else if (strcmp(command, "unlock") == 0) {
/*----------------------------------------------------------------------------
* unlock - remove the file write lock for the file
*---------------------------------------------------------------------------*/
	    if (strlen(cmdArg1) > 0) {
		char lockName[GEN_FNAME_DIM+5]; /* name of lock file */

		(void)sprintf(lockName, "%s.lock", cmdArg1);
		if ((stat = unlink(lockName)) != 0)
		    perror("arChanIO: removing lock file");
	    }
	    else {
		(void)printf("syntax error; ");
		stat = ERROR;
	    }
	}
/*----------------------------------------------------------------------------
*
*---------------------------------------------------------------------------*/
	else {
	    stat = ERROR;
	}
	if (stat == ERROR) {
	    (void)printf("Legal commands are:\n");
	    (void)printf("	chanAdd index chanName\n");
	    (void)printf("	chanChar [blockNum]\n");
	    (void)printf("	chanClose index\n");
	    (void)printf("	chanDel index\n");
	    (void)printf("	chanDescDump\n");
	    (void)printf("	chanHex [blockNum]\n");
	    (void)printf("	chanOpenR index chanName\n");
	    (void)printf("	chanOpenW index chanName\n");
	    (void)printf("	chanPos timeStampText\n");
	    (void)printf("	chanRead [repCount]\n");
	    (void)printf("	chanSkip [repCount]\n");
	    (void)printf("	chanSelect [index]\n");
	    (void)printf("	chanStore value [repCount]\n");
	    (void)printf("	close\n");
	    (void)printf("	create fileName maxBytes\n");
	    (void)printf("	info\n");
	    (void)printf("	open fileName\n");
	    (void)printf("	quit\n");
	    (void)printf("	retrieve\n");
	    (void)printf("	showMI\n");
	    (void)printf("	store <file\n");
	    (void)printf("	testOpen\n");
	    (void)printf("	testWrite nSamples\n");
	    (void)printf("	tsFromText\n");
	    (void)printf("	tsToText\n");
	    (void)printf("	unlock fileName\n");
	}
        semGive(semWorkDone);
    }

    if (pArCfDesc != NULL)
	stat = arCFClose(&pArCfDesc);

    return OK;
}
#if 0
testWrite(pArChanDesc)
AR_CHAN_DESC *pArChanDesc;
{
    static char	data[]="abcdefghi";
    int i, stat;

    for (i=0; i<9; i++) {
	if (--pArChanDesc->remainCount >= 0)
	    *(char *)(++(pArChanDesc->pData)) = data[i];
	else {
	    stat = arCF_DFlushWbufAndPutc(data[i], pArChanDesc);
	    assert(stat == OK);
	}
    }
    ArCFModifySet(pArChanDesc->pArCfDesc, pArChanDesc->pDataBuf);

    return OK;
}
#endif
