/*	@(#)arLib.c	1.4 1/7/92
 *	Author:	Roger A. Cole
 *	Date:	12-06-89
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	12-06-89	rac	initial version
 * .02	07-31-91	rac	installed in SCCS
 * .03	10-21-91	rac	split into smaller files; handle REQ_STATUS:
 *				and SAMP_STATUS: commands; don't use LWP
 *
 * make options
 *	-DNDEBUG	don't compile assert checking
 *      -DDEBUG         compile various debug code, including checks on
 *                      malloc'd memory
 */
/*+/mod***********************************************************************
* TITLE	arLib - general utility routines for AR code
*
* DESCRIPTION
*
*-***************************************************************************/

#include <stdio.h>
#include <sys/file.h>	/* O_RDWR definition */

#define AR_TEXT_GLBLSOURCE	/* this compiles arDefs.h text when compiling
				this source file.  Other source files which
				use arDefs.h will get `extern' declarations
				for these text fields. */
#include <tsDefs.h>
#include <arCS.h>
#include <arDefs.h>
#include <arAccessLib.h>


/*----------------------------------------------------------------------------
*    synopis summaries
*
*	int arCaStatusMessage(     status,       message,  pCh		)
*
*	int arConnBreak(           pArReq				)
*      void arConnHandler(         arg					)
*	int arConnMake(            pArReq				)
*	int arConnQCheck(                				)
*	int arConnQPrintInfo(      abbrev      				)
*	int arElCreateAndAppend(   pArReq,      chanName		)
*	int arElCreateWrapup(      ppArEl                		)
*      void arElEventHandler(	   arg					)
*    AR_EL *arElFind(              pArReq,      chanName		)
*	int arElGetValue(          pArEl				)
*	int arElGetNativeValue(    pArEl				)
*	int arElLogStatus(	   pArEl,       status  		)
*	int arElPrintInfo(         pArEl				)
*	int arElPrintValue(        pArEl				)
*	int arElPutStringValue(    pArEl,       pValue			)
*	int arMainQCheck(         					)
*	int arMainQPrintInfo(	   abbrev				)
*	int arLogMessage(          progName,    message,  status	)
*       int arMethActionAct(       pArMeth				)
*       int arMethActionInact(     pArMeth				)
*       int arMethActionSample(    pArMeth				)
*	int arMethAdd(		   pArMeth,     actionType		)
*	int arMethBuild(	   pArMeth,     string			)
*	int arMethBuildCheck(	   pArMeth            			)
*	int arMethDel(		   pArMeth				)
*      void arMethEventHandlerCa(  arg					)
*	int arMethGetInfoAsText(   desc,        pArMeth, >buf,  bufSize	)
*	int arMethPrintInfo(	   pArMeth,     desc			)
*	int arMethSetCheck(	 					)
*	int arMethSetPrintInfo(   					)
*	int arReadEnabled(	 					)
*	int arReadReq(		   pArReq				)
*	int arReadReqWrapup(	   pArReq				)
*	int arReadQAddReq(	   reqName				)
*	int arReadQCheck(                 				)
*	int arReadQPrintInfo(	   abbrev				)
*	int arReqCreate(          >ppArReq,     reqName	     		)
*	int arReqDelete(           pArReq				)
*	int arReqDisable(          pArReq				)
*   AR_REQ *arReqFind(	           reqName				)
*	int arReqGetValues(        pArReq				)
*	int arSampleChanAddMon(    pArReq				)
*	int arReqPrintInfo(	   pArReq,      abbrev			)
*	int arReqPrintValues(      pArReq				)
*---------------------------------------------------------------------------*/

/*+/subr***********************************************************************
* NAME	arCaStatusMessage - print message for channel access status code
*
* DESCRIPTION
*	Prints the text for a channel access status code.  If an additional
*	message is specified by the caller, it is also printed, along with
*	the channel name, if the caller specified an archive element and
*	if that element has a channel name associated with it.
*
*	Printing is directed to stderr.
*
* RETURNS
*	AR_S_OK
*
* BUGS
* o	this routine depends heavily on knowing how ca_signal works; if
*	ca_signal were to change, this routine might not work.
*
*-*/
int
arCaStatusMessage(status, argMsg, pCh)
int	status;		/* I status code from channel access */
char	*argMsg;	/* I additional message to print, or NULL */
AR_EL	*pCh;		/* I channel ptr associated with status, or NULL */
{
    char	message[80];	/* temp for message */
    int		msgNo;		/* CA message number */

/*----------------------------------------------------------------------------
*    print caller's message and/or channel name, depending on what
*    was specified by caller
*---------------------------------------------------------------------------*/
    if (argMsg != NULL && pCh != NULL) {
	(void)sprintf(message, "%s %s", argMsg, ca_name(pCh));
	(void)arLogMessage(" ", message, NULL);
    }
    else if (argMsg != NULL)
	arLogMessage(" ", argMsg, NULL);
    else {
	(void)sprintf(message, "(no name)");
	(void)arLogMessage(" ", message, NULL);
    }

/*----------------------------------------------------------------------------
*    print message for channel access status code
*---------------------------------------------------------------------------*/
    arLogMessage(" ", ca_message(status), NULL);

    return AR_S_OK;
}

/*+/subr***********************************************************************
* NAME	arLogMessage - print a message and/or status on stderr
*
* DESCRIPTION
*	Prints a message and/or a the text for a status code on stderr,
*	with a prefix of date and time and program name.  If both a
*	message and a status are specified, they will be printed on
*	two separate lines.
*
* RETURNS
*	AR_S_OK
*
* BUGS
* o	need to make sure status code is a valid AR status code
* o	check for status code != NULL depends on no status codes being zero
*
*-*/
int
arLogMessage(progName, message, status)
char	*progName;	/* I program name (or " "; pglArCtx->progname
				assumed) */
char	*message;	/* I message to print, or NULL; don't end with \n */
int	status;		/* I status code to print, or NULL */
{
    char	textTemp[80];	/* temp for text */
    TS_STAMP	timeTemp;	/* temp for present time */

    if (*progName == ' ')
	progName = pglArCtx->progName;
    (void)tsLocalTime(&timeTemp);
    (void)tsStampToText(&timeTemp, TS_TEXT_MMDDYY, textTemp);
    if (message != NULL)
	(void)fprintf(stderr, "%.17s %-6s: %s\n", textTemp, progName, message);
    if (status != NULL)
	(void)fprintf(stderr, "%.17s %-6s: %s\n", textTemp, progName,
						ArStatusToText(status));

    return AR_S_OK;
}

/*+/subr***********************************************************************
* NAME	arMainQCheck - check requests on main request list
*
* DESCRIPTION
*	Checks the requests on the main archive request list to see if
*	any processing is needed.  Processing depends on request status:
*
*	AR_REQ_CONN_CA_BAD -- the request is removed from the list and added
*	to the connect list.  The request is removed from any method
*	queues it happens to be on, and any channels which have monitors
*	have their monitors cleared.  In order to stimulate connections to
*	channels which may have been added to the request, the request
*	status is changed to AR_REQ_CONN_NONE.
*
*	AR_REQ_CONN_OK -- the request status is changed to AR_REQ_INACTIVE
*	and it is added to the activation method queue.
*
*	If the .disable flag is set in the request, and if the .busy flag
*	isn't set, the request is removed from the list and deleted.
*
* RETURNS
*	AR_S_OK, or
*	status code from other routines
*
*-*/
int
arMainQCheck()
{
    int		retStat=AR_S_OK;/* return status to caller */
    int		stat;		/* status return from calls */
    int		statTemp;	/* temporary for cumulative status */
    char	message[80];	/* temporary for error messages */
    AR_EL	*pArEl;		/* pointer to next archive element */
    AR_REQ	*pArReq;	/* pointer to next archive request */
    AR_REQ	*pArReqNext;	/* pointer to following archive request */

    pArReq = pglArCtx->pReqMainHead;

    while (pArReq != NULL) {
	pArReqNext = pArReq->pNext;
	if (pArReq->disable && pArReq->busy == 0) {
	    DoubleListRemove(pArReq, pglArCtx->pReqMainHead,
						pglArCtx->pReqMainTail);
	    if ((stat = arReqDelete(pArReq)) != AR_S_OK) {
		(void)sprintf(message,
			"errors while disabling request: %s", pArReq->name);
		arLogMessage(" ", message, stat);
	    }
	}
	else if (pArReq->status == AR_REQ_CONN_CA_BAD) {
	    DoubleListRemove(pArReq, pglArCtx->pReqMainHead,
						pglArCtx->pReqMainTail);
	    (void)sprintf(message, "%s inactivated to (re)connect channel(s)",
							pArReq->name);
	    (void)arLogMessage(" ", message, NULL);
	    pArReq->status = AR_REQ_CONN_NONE;
	    pArEl = pArReq->pElListHead;
	    while (pArEl != NULL) {
		if (pArEl->pEv != NULL) {
		    stat = ca_clear_event(pArEl->pEv);
		    if (stat != ECA_NORMAL)
			(void)arCaStatusMessage(stat, NULL, pArEl->pCh);
		    else
			pArEl->pEv = NULL;
		}
		pArEl = pArEl->pNext;
	    }
	    statTemp = AR_S_OK;
	    if (pArReq->act.state != AR_METH_ST_UNDEF) {
		stat = arMethDel(&pArReq->act);
		if (stat != AR_S_OK) {
		    statTemp = stat;
		    (void)sprintf(message,
			"error deleting activation method for: %s",
								pArReq->name);
		    arLogMessage(" ", message, stat);
		}
	    }
	    if (pArReq->inact.state != AR_METH_ST_UNDEF) {
		stat = arMethDel(&pArReq->inact);
		if (stat != AR_S_OK) {
		    statTemp = stat;
		    (void)sprintf(message,
			"error deleting inactivation method for: %s",
								pArReq->name);
		    arLogMessage(" ", message, stat);
		}
	    }
	    if (pArReq->samp.state != AR_METH_ST_UNDEF) {
		stat = arMethDel(&pArReq->samp);
		if (stat != AR_S_OK) {
		    statTemp = stat;
		    (void)sprintf(message,
			"error deleting sample method for: %s", pArReq->name);
		    arLogMessage(" ", message, stat);
		}
	    }
	    if (statTemp == AR_S_OK) {
		DoubleListAppend(pArReq, pglArCtx->pReqConnHead,
						pglArCtx->pReqConnTail);
	    }
	    else {
		retStat = statTemp;
		arLogMessage(" ", "request being deleted", NULL);
		stat = arReqDelete(pArReq);
		if (stat != AR_S_OK) {
		    retStat = stat;
		    (void)sprintf(message, "error deleting request: %s",
								pArReq->name);
		}
	    }
        }
	else if (pArReq->status == AR_REQ_CONN_OK) {
	    pArReq->status = AR_REQ_INACTIVE;
	    stat = arMethAdd(&pArReq->act, AR_METH_AC_ACT);
	    if (stat != AR_S_OK) {
		retStat = stat;
		(void)sprintf(message,
                            "error queueing for activation: %s", pArReq->name);
		arLogMessage(" ", message, stat);
		arLogMessage(" ", "request being deleted", NULL);
		DoubleListRemove(pArReq, pglArCtx->pReqMainHead,
						pglArCtx->pReqMainTail);
		stat = arReqDelete(pArReq);
	    }
	}
	pArReq = pArReqNext;
    }

    return retStat;
}

/*+/subr***********************************************************************
* NAME	arMainQPrintInfo - print information for all archive requests
*
* DESCRIPTION
*	Prints information for each archive request in the archive request
*	list.
*
* RETURNS
*	AR_S_OK, or
*	status code from arReqPrintInfo
*
*-*/
int
arMainQPrintInfo(abbrev)
int	abbrev;		/* I 0,1 for full,short print */
{
    int		retStat=AR_S_OK;/* return status to caller */
    int		stat;		/* status return from calls */
    AR_REQ	*pArReq;	/* pointer to next request */

    if (!abbrev)
	(void)arLogMessage(" ", "**** MAIN REQUEST LIST ****", NULL);

    pArReq = pglArCtx->pReqMainHead;
    while (pArReq != NULL) {
	stat = arReqPrintInfo(pArReq, abbrev);
	if (stat != AR_S_OK)
	    retStat = stat;
	pArReq = pArReq->pNext;
    }

    return retStat;
}
