/*	@(#)reqSubr.h	1.5 11/17/92
 *	Author:	Roger A. Cole
 *	Date:	03-08-92
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1992, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 *  .01 03-08-92 rac	initial version
 *  .02 03-15-92 rac	continued development
 *  .03 08-28-92 rac	add new archive request features
 *  .04 09-30-92 rac	print stamps for interval test
 *  .05 11-17-92 rac	allow printing to status channels even if they
 *			aren't connected
 * reqSubr.h,v
 * Revision 1.2  1994/05/06 19:06:18  mrk
 * conversion from sccs to cvs
 *
 *
 * make options
 *	-DNDEBUG	don't compile assert() checking
 *      -DDEBUG         compile various debug code, including checks on
 *                      malloc'd memory
 */
/*+/mod***********************************************************************
* TITLE	reqSubr.h - structures and definitions for reqSubr.c
*
* DESCRIPTION
*
*-***************************************************************************/

/*-----------------------------------------------------------------------------
*    raw archive request structure
*----------------------------------------------------------------------------*/
#ifndef REQ_PRIVATE
    extern char *glReqOutText[];
#else
    char *glReqOutText[] = {
        /* REQ_OUT_UNDEF     */ "undefined",
        /* REQ_OUT_BCF       */ "chan:bin",
        /* REQ_OUT_SSF       */ "set:txt",
        /* REQ_OUT_PAST      */ "illegal"
    };
#endif

typedef enum {
    REQ_OUT_UNDEF,	/* MUST BE ZERO!!! */
    REQ_OUT_BCF,
    REQ_OUT_SSF,

    REQ_OUT_PAST	/* last legal + 1 */
} REQ_OUT_TYPE;


#ifndef REQ_PRIVATE
    extern char *glReqTypeText[];
#else
    char *glReqTypeText[] = {
        /* REQ_TYPE_UNDEF     */ "undefined",
        /* REQ_TYPE_AFTER     */ "after",
        /* REQ_TYPE_AFTER_STEP*/ "after step",
        /* REQ_TYPE_ALWAYS    */ "always",
        /* REQ_TYPE_BEFORE    */ "before",
        /* REQ_TYPE_CONTIN    */ "continuous",
        /* REQ_TYPE_DELAY     */ "delay",
        /* REQ_TYPE_DEMAND    */ "demand",
        /* REQ_TYPE_DONE      */ "done",
        /* REQ_TYPE_EVENT     */ "event",
        /* REQ_TYPE_EVERY     */ "every",
        /* REQ_TYPE_FOR       */ "for",
        /* REQ_TYPE_FOREVER   */ "forever",
        /* REQ_TYPE_IF        */ "if",
        /* REQ_TYPE_MONITOR   */ "monitor",
        /* REQ_TYPE_NEVER     */ "never",
        /* REQ_TYPE_NTH       */ "nth",
        /* REQ_TYPE_PERIODIC  */ "periodic",
        /* REQ_TYPE_ROUND     */ "round",
        /* REQ_TYPE_SAMPLES   */ "samples",
        /* REQ_TYPE_SCHED     */ "scheduled",
        /* REQ_TYPE_SNAP      */ "snapshot",
        /* REQ_TYPE_START     */ "start",
        /* REQ_TYPE_STEP      */ "step",
        /* REQ_TYPE_TIMED     */ "timed",
        /* REQ_TYPE_UNTIL     */ "until",
        /* REQ_TYPE_WHEN      */ "when",
        /* REQ_TYPE_PAST      */ "illegal"
    };
#endif

typedef enum {
    REQ_TYPE_UNDEF,	/* MUST BE ZERO!!! */
    REQ_TYPE_AFTER,
    REQ_TYPE_AFTER_STEP,
    REQ_TYPE_ALWAYS,
    REQ_TYPE_BEFORE,
    REQ_TYPE_CONTIN,
    REQ_TYPE_DELAY,
    REQ_TYPE_DEMAND,
    REQ_TYPE_DONE,
    REQ_TYPE_EVENT,
    REQ_TYPE_EVERY,
    REQ_TYPE_FOR,	/* need number of samples or hh:mm:ss */
    REQ_TYPE_FOREVER,
    REQ_TYPE_IF,
    REQ_TYPE_MONITOR,
    REQ_TYPE_NEVER,
    REQ_TYPE_NTH,
    REQ_TYPE_PERIODIC,	/* hh:mm:ss */
    REQ_TYPE_ROUND,	/* need number of milli-sec */
    REQ_TYPE_SAMPLES,
    REQ_TYPE_SCHED,	/* need date stamp and time stamp */
    REQ_TYPE_SNAP,
    REQ_TYPE_START,
    REQ_TYPE_STEP,
    REQ_TYPE_TIMED,	/* hh:mm:ss */
    REQ_TYPE_UNTIL,	/* chanName condition ... */
    REQ_TYPE_WHEN,	/* chanName condition ... */

    REQ_TYPE_PAST	/* last legal + 1 */
} REQ_TYPE;

#ifndef REQ_PRIVATE
    extern char *glReqEventText[];
#else
    char *glReqEventText[] = {
        /* REQ_EV_UNDEF     */   "undefined",
        /* REQ_EV_CHANGE    */   "change",
        /* REQ_EV_EQ        */   "==",
        /* REQ_EV_NEQ       */   "!=",
        /* REQ_EV_GT        */   ">",
        /* REQ_EV_GE        */   ">=",
        /* REQ_EV_LT        */   "<",
        /* REQ_EV_LE        */   "<=",
        /* REQ_EV_PAST      */   "illegal"
    };
#endif

typedef enum {
    REQ_EV_UNDEF,		/* MUST BE ZERO!!! */
    REQ_EV_CHANGE,
    REQ_EV_EQ,
    REQ_EV_NEQ,
    REQ_EV_OK_NON_NUM = REQ_EV_NEQ,  /* event types above this are OK for ENUM,
				STRING, and CHAR (i.e., non-numeric) channels */
    REQ_EV_GT,
    REQ_EV_GE,
    REQ_EV_LT,
    REQ_EV_LE,

    REQ_EV_PAST		/* last legal + 1 */
} REQ_EV_COND;

/*/subhead RAW_CHAN------------------------------------------------------------
* data channel structure
*	These channels are the ones following the %endHeader% line.
*
*----------------------------------------------------------------------------*/
typedef struct rawChan {
    struct rawReq *pRaw;
    struct rawChan *pNext;
    struct rawChan *pPrev;
    int		filled;
    char	name[db_name_dim];
    chid	pCh;
    evid	pEv;
    chid	pChADEL;
    short	isAField;	/* 1 if this is a field, but not .VAL */
    float	deadband;
    long	nSamp;		/* number of samples obtained */
    long	nSampOK;	/* number of samples with OK interval */
    TS_STAMP	prevTs;		/* stamp of previous sample */
    TS_STAMP	minTs;		/* minimum stamp encountered */
    TS_STAMP	maxTs;		/* maximum stamp encountered */
    double	maxInterval;	/* largest interval between samples */
    double	minInterval;	/* smalles interval between samples */
} RAW_CHAN;

/*/subhead REQ_STAT------------------------------------------------------------
*
*----------------------------------------------------------------------------*/
typedef struct reqStat {
    char	name[db_name_dim];
    chid	pCh;
    char	msg[20];
} REQ_STAT;

/*/subhead REQ_TRIG------------------------------------------------------------
* trigger condition
*	A trigger is used to initiate an activity.  Typically, a trigger
*	will be a simple type, with possible minor modifications.
*
*	Trigger conditions are checked by reqTrigTest, which returns 0 or
*	1 depending on whether the condition is false or true.
*
*	The main types are:
*	    REQ_TYPE_AFTER_STEP	reqTrigTest always returns 0
*	    REQ_TYPE_ALWAYS	reqTrigTest always returns 1
*	    REQ_TYPE_DEMAND	reqTrigTest always returns 1; the fact that
*				a demand was made is noted by reqTrigTest
*	    REQ_TYPE_DONE	reqTrigTest returns 1 if the `done' flag
*				has been set
*	    REQ_TYPE_EVENT	reqTrigTest returns 1 if an event condition
*				has been satisfied; actual testing occurs
*				in reqTrigMonHandler and reqTrigEventTest
*	    REQ_TYPE_IF		reqTrigTest returns 1 if an condition is
*				satisfied at the time of a test; actual
*				testing occurs in reqTrigEventTest
*	    REQ_TYPE_NEVER	reqTrigTest always returns 0
*	    REQ_TYPE_PERIODIC	reqTrigTest returns 1 if timer has expired,
*				and the first time the condition is tested
*	    REQ_TYPE_SCHED	reqTrigTest returns 1 if the date and time
*				has arrived
*	    REQ_TYPE_TIMED	reqTrigTest returns 1 if timer has expired
*
*	If the "at least every hh:mm:ss" condition has been specified, then
*	reqTrigTest will keep track of when 1 was last returned, and return
*	1 if hh:mm:ss has elapsed, even if the actual trigger condition
*	isn't satisfied.  The "at least" timer will then be restarted.
*
*	eventCond:  (default REQ_EV_UNDEF)
*	    REQ_EV_xxx	specifies the condition to check
*	atleastType:  (default REQ_TYPE_NEVER)
*	    REQ_TYPE_TIMED  specifies that elapsed time is to be checked
*
*	When a trigger for REQ_TYPE_EVENT  or _IF is created, the comparison
*	value is stored as a string in strVal.  When the trigger is actually
*	used, the code (reqTrigConnect and friends) is expected to determine
*	the intrinsic type of the channel, convert the string value into
*	the intrinsic type for comparisons, and store the resultant value
*	back into the union.
*
*	For events, it is possible to specify to trigger every nth time
*	the condition is met.  The everyXxx fields are used for this.  When
*	the trigger is armed, the counter is set to zero and the "primed"
*	flag is set.  If the event condition is satisfied at the time of
*	arming, then "primed" is set to 0; otherwise to 1.  Whenever
*	subsequent testing of the event condition is satisfied, and "primed"
*	is 1, then the counter is incremented and "primed" is set to 0.
*	When the event condition isn't satisfied, "primed" is set back to
*	1.  Thus, only on the transition from non-satisfied to satisfied
*	will the counter be bumped.
*
*----------------------------------------------------------------------------*/
typedef struct {
    REQ_TYPE	type;		/* trigger type */
    short	armed;		/* 1 if trigger is armed */
    short	demand;		/* 1 if a demand has been received */
    short	notYetInit;	/* 1 if trigger not yet fully initialized */
    short	notYetTested;	/* 1 if trigger never tested */
    short	doneFlag;	/* 1 if user has set it; 0 at trigArm time */
    chid	event_pCh;	/* chid for event */
    evid	event_pEv;	/* CA event id for efent */
    char	eventName[db_name_dim]; /* name of channel */
    short	eventHappened;	/* 1 if the event occurred */
    TS_STAMP	dataTs;		/* time stamp of most recent data */
    short	eventNoValYet;	/* 1 if the event hasn't yet got a value */
    REQ_EV_COND	eventCond;	/* event condition */
    int		everyComp;	/* count from "every n times", for comparison */
    short	everyCount;	/* count of times condition has been met */
    short	everyPrimed;	/* 1 if condition has been in unmet state */
    short	notYetPrimed;	/* 1 if never have determined primed state */
    union {
	short	shtVal;
	float	fltVal;
	double	dblVal;
	long	lngVal;
	char	strVal[db_strval_dim];
	char	chrVal;
    }		eventValCmp,	/* comparison value for event */
		eventValCurr,	/* current value for channel */
		eventValPrev;	/* previous value for channel */
    struct dbr_gr_enum enumGr;	/* graphics info for enum channels */
    short	enumGetGR;	/* 1 if graphics info is needed;
				   2 if received but not checked;
				   0 otherwise */
    REQ_TYPE	atleastType;	/* type from "atleast" keyword */
    TS_STAMP	atleastStamp;	/* time stamp from "atleast" keyword */
    TS_STAMP	atleastLast;	/* last time atleast condition was satisfied */
    TS_STAMP	atleastNext;	/* next time to trigger atleast activity */
    TS_STAMP	timeStamp;	/* time stamp for timed activity */
    TS_STAMP	timeLast;	/* last time timed condition was satisfied */
    TS_STAMP	timeNext;	/* next time to trigger timed activity */
} REQ_TRIG;

/*/subhead REQ_SAMPLE----------------------------------------------------------
* sample method
*	The sample method controls sampling activities.  In addition to the
*	sample trigger, the sample method also contains parameters to control
*	sampling once the trigger has occurred.
*
*	The sample type will be one of:
*	    REQ_TYPE_MONITOR	specifies monitoring individual channels
*				for all changes
*	    REQ_TYPE_SNAP	specifies sampling with synchronous snapshots
*
*    MONITOR
*	Monitor sampling saves each change in every channel.
*
*	Once sampling starts (because of the trigger), it will continue
*	as controlled by the "for" parameters.  "forType" will be one of:
*	    REQ_TYPE_FOREVER	sample forever (this is the default)
*	    REQ_TYPE_EVENT	sample until the forEvent condition is true
*				(from the "until" keyword)
*	    REQ_TYPE_TIMED	sample for hh:mm:ss in forStamp.  bufNsamples
*				isn't used.
*
*    SNAP
*	Snapshot sampling takes synchronous samples.  All or only a few of
*	the samples are included in the snapshot.  This is controlled by
*	"everyType":
*	    REQ_TYPE_CONTIN	specifies that all samples are saved (this
*				is the default)
*	    REQ_TYPE_NTH	specifies that only every nth sample will
*				be saved, as indicated by everyCmp
*	    REQ_TYPE_TIMED	specifies that a sample will be saved only
*				periodically, as indicated by everyStamp
*
*	Once sampling starts (because of the trigger), it will continue as
*	controlled by the "for" parameters.  "forType" will be one of:
*	    REQ_TYPE_EVENT	sample until the forEvent condition is true
*				(from the "until" keyword)
*	    REQ_TYPE_SAMPLES	sample for forCount samples
*	    REQ_TYPE_TIMED	sample for hh:mm:ss in forStamp.  bufNsamples
*				is used for sydSampleSetAlloc to know how
*				much space to allocate.
*
*	The "start" parameters control the relationship between the start
*	of the snapshot and the occurrence of the trigger.  It is possible
*	to have sampling start with samples either prior to or following
*	the trigger.  (This is possible because of buffering and continuous
*	monitoring.)  "startType" will be one of:
*	    REQ_TYPE_NTH	specifies to start sampling with the nth
*				sample before or after the trigger (startCount)
*	    REQ_TYPE_TIMED	specifies to start sampling hh:mm:ss before
*				or after the trigger (startStamp)
*	startWhen controls whether the sampling starts before or after the
*	trigger.  It will be either REQ_TYPE_BEFORE or REQ_TYPE_AFTER.
*----------------------------------------------------------------------------*/
typedef struct {
    REQ_TYPE	type;		/* sample type */
    short	halt;		/* 1 indicates sampling should stop */
    REQ_TRIG	trigger;	/* trigger for sampling */
    REQ_TYPE	everyType;	/* type from "every" keyword */
    int		everyCmp;	/* sample count from "every" keyword */
    int		everyCount;	/* count of skipped samples */
    TS_STAMP	everyStamp;	/* time stamp from "every" keyword */
    TS_STAMP	everyLast;	/* time stamp of last sample */
    TS_STAMP	everyNext;	/* time stamp for next sample */
    REQ_TYPE	forType;	/* type from "for" keyword */
    int		forCmp;		/* sample count from "for" keyword */
    REQ_TRIG	forEvent;	/* trigger condition from "until" keyword */
    TS_STAMP	forStamp;	/* time stamp from "for" keyword */
    TS_STAMP	forStop;	/* time stamp to stop timed sampling */
    REQ_TYPE	startType;	/* type from "start" keyword */
    int		startCount;	/* sample count from "start" keyword */
    TS_STAMP	startStamp;	/* time stamp from "start" keyword */
    REQ_TYPE	startWhen;	/* before or after trigger */
    int		bufNsamples;	/* number of input samples to buffer ahead */
    int		roundMsec;	/* milli-second rounding factor */
    int		ignorePartial;	/* 1 if partial samples are to be ignored */
    REQ_OUT_TYPE outputType;	/* output type */
    char	outDir[120];	/* output file directory */
    char	outFile[120];	/* output file name */
    int		outSize;	/* output file size limit, in bytes */
} REQ_SAMPLE;

/*/subhead REQ_STEP------------------------------------------------------------
* step through a range of values
*	A step structure controls stepping a channel through a range of
*	values, with an optional time delay following each step.
*
*	If desired, a sample trigger can be initiated following each ca_put.
*
*
*----------------------------------------------------------------------------*/
typedef struct reqStep {
    REQ_TYPE	type;		/* type: REQ_TYPE_UNDEF or REQ_TYPE_STEP */
    chid	pCh;		/* channel pointer */
    char	name[db_name_dim];
    double	delay;		/* delay time, in seconds */
    double	beg;		/* beginning value for sequence */
    double	end;		/* ending value for sequence */
    double	present;	/* present value in sequence */
    double	step;		/* step size in sequence */
    short	active;		/* 1 if stepping is in progress */
    short	complete;	/* 1 if stepping has completed */
    short	delayPending;	/* 1 if waiting for delay */
    short	delayDone;	/* 1 if wait for delay is done */
    TS_STAMP	delayTs;	/* time delay is complete */
    short	getPending;	/* 1 if waiting for get after put */
    short	getDone;	/* 1 if get value has been received */
    short	samplePending;	/* 1 if waiting for a sample to start */
    short	sampleActive;	/* 1 if waiting for a sample to complete */
    short	sampleDone;	/* 1 if sample just completed */
} REQ_STEP;

typedef struct rawReq {
    RAW_CHAN	*pChanHead;	/* head of channel list for request */
    RAW_CHAN	*pChanTail;	/* tail of channel list for request */
    char	file[120];	/* file name for request */
    char	dir[120];	/* directory for request */
    char	name[db_name_dim];/* name of archive request */
    char	desc[120];	/* last DESC: item in request */
    short	totalErrors;	/* total errors */
    REQ_TRIG	actTrig;	/* activation trigger */
    short	allowDiscon;	/* allow activating even if some discon */
    RAW_CHAN	actChan;
    REQ_TRIG	inactTrig;	/* inactivation trigger */
    RAW_CHAN	inactChan;
    REQ_SAMPLE	sample;		/* sample method */
    RAW_CHAN	forChan;
    RAW_CHAN	triggerChan;
    char	exportFile[120];/* export file name */
    RAW_CHAN	sampleStatChan;	/* sample status channel */
    RAW_CHAN	reqStatChan;	/* request status channel */
    REQ_STEP	step;		/* structure for stepping a channel */
    RAW_CHAN	stepChan;	/* step channel */
    short	hadAct;		/* 1 if have had ACT: */
    short	hadDesc;	/* 1 if have had DESC: */
    short	hadEndHead;	/* 1 if have had %endHeader% */
    short	hadExport;	/* 1 if have had EXPORT: */
    short	hadFor;		/* 1 if have had SAMPLE: for */
    short	hadInact;	/* 1 if have had INACT: */
    short	hadMonitor;	/* 1 if have had MONITOR: */
    short	hadOutput;	/* 1 if have had OUTPUT: */
    short	hadReqStat;	/* 1 if have had REQ_STATUS: */
    short	hadSnapshot;	/* 1 if have had SNAPSHOT: */
    short	hadSampleStat;	/* 1 if have had SAMP_STATUS: */
    short	hadStep;	/* 1 if have had STEP: */
    short	hadTrigger;	/* 1 if have had TRIGGER: */
    double	jitterSec;	/* seconds of jitter allowed, to be OK */
    double	intervalSec;	/* seconds between samples, to be OK */
} RAW_REQ;

RAW_REQ *rawReqAlloc();
void rawReqCheckConnect_init();
void rawReqCheckConnect_wrap();
void rawReqCheckInterval_init();
void rawReqCheckInterval_wrap();
void rawReqCheckSemantics();
void rawReqFree();
int rawReqParseLine();
RAW_REQ *rawReqReadFile();
int reqEventTest(), reqEventTest_sm();
void reqTrigArm();
void reqTrigInit(), reqTrigWrap();
void reqTrig_setDone();
int reqTrigTest();

void reqStatInit();
void reqStatPrintf();
void reqStatWrap();
void reqStepInit();
void reqStepWrap();
int reqStep_seqDoneCheck();
void reqStep_seqStart();
int reqStep_stepDoneCheck();
void reqStep_stepNext();
