/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef INC_cmdDefs_h
#define INC_cmdDefs_h
/*	cmdDefs.h,v 1.2 2002/08/02 15:37:58 jba Exp
 *	Author:	Roger A. Cole
 *	Date:	10/24/90
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 */
/*+/mod***********************************************************************
* TITLE	cmdDefs.h - definitions for command processing routines
*
* DESCRIPTION
*	This file contains some definitions which are needed to use
*	cmdSubr.c, a set of routines which form a sort of "front end"
*	for processing text commands.
*
* REQUIRES:
*	<genDefs.h>
*
* SEE ALSO
*	cmdSubr.c
*
*
*-***************************************************************************/
/*/subhead CX_CMD--------------------------------------------------------------
* CX_CMD
*
*	The context block for command processing routines contains items for:
*	o   assisting in commands parsing
*	o   managing generic help information
*	o   directing input and output related to commands
*----------------------------------------------------------------------------*/
typedef struct cxCmd {
    HELP_LIST	helpList;	/* list of help topics */
    HELP_TOPIC	helpBg;		/* help info--bg command */
    HELP_TOPIC	helpCmds;	/* generic command information for help */
    HELP_TOPIC	helpCmdsSpec;	/* commands--program specific */
    HELP_TOPIC	helpUsage;	/* usage information for help */
    FILE	*input;		/* input stream */
    char	*inputName;	/* name of input stream (for file input) */
    int		inputEOF;	/* 1 indicates EOF on keyboard/socket input */
    FILE	*dataOut;	/* data output stream */
    int		dataOutRedir;	/* 1 indicates dataOut is re-directed */
    char	*prompt;	/* prompt string */
    int		promptFlag;	/* 1 indicates a prompt is needed */
    char	*pLine;		/* pointer into line, used by nextXxxField */
    char	*pCommand;	/* pointer to command field */
    char	*pField;	/* pointer to next field */
    struct cxCmd *pCxCmdRoot;	/* pointer to root command context */
    struct cxCmd *pPrev;	/* pointer to previous command context */
    char	line[80];	/* input line */
    int		fldLen;		/* length of next field, including delim */
    char	delim;		/* delimiter of next field */
} CX_CMD;

/*/subhead prototypes----------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
long cmdBgCheck();
char *cmdRead();
void cmdSource();
void cmdCloseContext();
void cmdInitContext();

#endif
