/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef INC_sydPlotDefs_h
#define INC_sydPlotDefs_h
/*	sydPlotDefs.h,v 1.2 2002/08/02 15:38:01 jba Exp
 *	Author:	Roger A. Cole
 *	Date:	12-04-90
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 * include options
 *	-DSYD_PLOT_PRIVATE compiles private structures
 *	-DXWINDOWS	compiles X11 structure items, etc.
 */
/*+/mod***********************************************************************
* TITLE	sydPlotDefs.h - definitions for synchronous data plotting routines
*
* DESCRIPTION
*
*-***************************************************************************/

#undef XWINDOWS
#define XWINDOWS

#include <genDefs.h>
#include <sydDefs.h>
#include <pprPlotDefs.h>
#ifdef XWINDOWS
#   include <X11/Xlib.h>
#endif


/*/subhead SYD_PL_xxx----------------------------------------------------------
* plot attributes
*	SYD_PLATTER_FG1 & 2 are used for Smith Chart axes
* plot type
*----------------------------------------------------------------------------*/

typedef enum sydPlAttr {
    SYD_PLATTR_LINE,	/* plot with lines */
    SYD_PLATTR_POINT,	/* plot with points */
    SYD_PLATTR_MARK,	/* plot with marks */
    SYD_PLATTR_SHOW,	/* show point status on plot */
    SYD_PLATTR_UNDER,	/* "fill under" plotting */
    SYD_PLATTR_XLAB,	/* do label for x axis */
    SYD_PLATTR_XANN,	/* do annotation for x axis */
    SYD_PLATTR_YLAB,	/* do label for y axis */
    SYD_PLATTR_YANN,	/* do annotation for y axis */
    SYD_PLATTR_MONO,	/* plotting medium is monochrome */
    SYD_PLATTR_BG,	/* set background pixel value */
    SYD_PLATTR_FG,	/* set foreground pixel value */
    SYD_PLATTR_FG1,	/* set first alternate fg pixel value */
    SYD_PLATTR_FG2,	/* set second alternate fg pixel value */
    SYD_PLATTR_XCHAN,	/* channel is an X-axis channel */
    SYD_PLATTR_WRAP,	/* wrap-around if value exceeds extent */
    SYD_PLATTR_BAR,	/* plot an error bar */
    SYD_PLATTR_STDDEV,	/* plot an error bar using standard deviation */
    SYD_PLATTR_INCR,	/* increment, in seconds, for strip charts */
} SYD_PLATTR;

#ifndef SYD_PLOT_PRIVATE
    extern char *glArPltyText;
#else
    char *glArPltyText[] = {
       /* SYD_PLAX_UNDEF     */	"undefined",
       /* SYD_PLAX_SMITH_IMP */	"Smith chart, impedance",
       /* SYD_PLAX_SMITH_ADM */	"Smith chart, admittance",
       /* SYD_PLAX_SMITH_IMM */	"Smith chart, immitance",
       /* SYD_PLAX_STRIP_Y   */	"strip chart, single y",
       /* SYD_PLAX_STRIP_YY  */	"strip chart, multiple y",
       /* SYD_PLAX_TY        */	"time vs y",
       /* SYD_PLAX_TYY       */	"time vs multiple y",
       /* SYD_PLAX_XY        */	"x vs y",
       /* SYD_PLAX_XYY       */	"x vs multiple y",
       /* SYD_PLAX_Y         */	"y, uniform spacing",
       /* SYD_PLAX_YY        */	"multiple y, uniform spacing",
       /* SYD_PLAX_PAST      */	"illegal"
    };
#endif

typedef enum sydPlType {
    SYD_PLAX_UNDEF,	/* MUST BE ZERO!! */
    SYD_PLAX_SMITH_IMP,
    SYD_PLAX_SMITH_ADM,
    SYD_PLAX_SMITH_IMM,
    SYD_PLAX_STRIP_Y,
    SYD_PLAX_STRIP_YY,
    SYD_PLAX_TY,
    SYD_PLAX_TYY,
    SYD_PLAX_XY,
    SYD_PLAX_XYY,
    SYD_PLAX_Y,
    SYD_PLAX_YY,
    SYD_PLAX_PAST	/* last legal + 1 */
} SYD_PLAX;

/*/subhead plot_slave_struct---------------------------------------------------
*
*----------------------------------------------------------------------------*/
typedef struct sydPlSlave {
    struct sydPlSlave *pPrev;   /* pointer to previous slave */
    struct sydPlSlave *pNext;   /* pointer to next slave */
    PPR_AREA	*pArea;		/* ptr to area structure for pprXxx routines */
    SYD_CHAN	*pSChan;	/* pointer to synchronous channel struct */
    double	extentVal;	/* value at end of axis */
    double	originVal;	/* value at origin of axis */
    int		nInt;		/* number of intervals for axis */
    int		nSubInt;	/* number of sub-intervals for axis */
    int		decPl;		/* number of decimal places for annotations */
    int		lineKey;	/* line key number for slave */
    int		markNum;	/* mark number for slave */
    short	xChan;		/* 1 indicates an x-axis channel */
    char	*pAnnot[db_state_dim];/* pointers to annotations (for ENUM) */
    char	**ppAnnot;	/* NULL or pointer to annotation array */
#ifdef XWINDOWS
    Pixmap	*pPix;		/* pointer to X11 pixmap for strip chart */
    unsigned long bg;		/* X11 background pixel value, or 0 */
    unsigned long fg;		/* X11 foreground pixel value, or 0 */
#endif
    double	xFracLeft;	/* window fraction at left x data value */
    double	xFracRight;	/* window fraction at right x data value */
    double	yFracBot;	/* window fraction at bottom y data value */
    double	yFracTop;	/* window fraction at top y data value */
    double	annotXFL;	/* win frac at left x of annot `rectangle' */
    double	annotXFR;	/* win frac at right x of annot `rectangle' */
    double	annotYFB;	/* win frac at bottom y of annot `rectangle' */
    double	annotYFT;	/* win frac at top y of annot `rectangle' */
    short	first;
    short	skip;
    double	oldX;
    double	oldY;
    double	errorHalfWidth;	/* half width for error bar */
} SYD_PL_SLAVE;

/*/subhead plot_master_struct--------------------------------------------------
* SYD_PL_MSTR
*
*	A plot master controls the plotting of data for one or more
*	channels.  It contains:
*
*	o   a list of plot slaves, one slave per channel
*	o   a pointer to the retrieval set master; all slaves share this
*	o   some overall information about the master plot, such as its
*	    text title (not to be confused with the _window_ title)
*	o   some x axis information for the case where time is used as the
*	    x axis
*	o   "hidden" information about the window in which plotting occurs
*----------------------------------------------------------------------------*/

typedef struct sydPlMstr {
    PPR_WIN_TY	winType;	/* type of "window" being used */
    SYD_PLAX	plotAxis;	/* type of plot axes--one of SYD_PLAX_xxx */
    short	linePlot;	/* 1 for line plot (can be used with marks) */
    short	pointPlot;	/* 1 for point plot (can be used with lines) */
    short	markPlot;	/* 1 for mark plot */
    short	showStat;	/* 1 for show "alarm" status */
    short	fillUnder;	/* 1 for "fill under" plot */
    short	noColor;	/* 1 for monochrome plotting */
    SYD_PLATTR	errBar;		/* 0 or SYD_PLATTR_{BAR or STDDEV} */
#ifdef XWINDOWS
    Display	*pDisp;		/* pointer to X11 display, or NULL */
    Window	window;		/* X11 window */
    unsigned long fg;		/* initial foreground pixel value for master */
    unsigned long bg;		/* initial background pixel value for master */
    unsigned long altPixel1;	/* alternate pixel value, or 0 */
    unsigned long altPixel2;	/* alternate pixel value, or 0 */
#endif
    short	useXlabel;	/* 1 to label x axis */
    short	useXannot;	/* 1 to annotate x axis */
    short	useYlabel;	/* 1 to label y axis */
    short	useYannot;	/* 1 to annotate y axis */
    SYD_SPEC	*pSspec;	/* pointer to synchronous set descriptor */
    PPR_WIN	*pWin;		/* ptr to window struct for pprXxx routines */
    char	title[120];	/* top title for plot */
    char	label[120];	/* label if time is used for x axis */
    char	lTitle[120];	/* left title for plot */
    char	bTitle[120];	/* bottom title for plot */
    char	rTitle[120];	/* right title for plot */
    SYD_PL_SLAVE *pHead;	/* pointer to head of list of slave plots */
    SYD_PL_SLAVE *pTail;	/* pointer to tail of list of slave plots */
    long        nSlaves;        /* number of slave plots */
    double	originVal;	/* elapsed time at origin of x axis */
    float	originFrac;	/* x window fraction at origin of x axis */
    double	extentVal;	/* elapsed time at end of x axis */
    float	extentFrac;	/* x window fraction at end of x axis */
    int		nInt;		/* number of intervals for x axis */
    int		nSubInt;	/* number of sub-intervals for axis */
    float	stripIncr;	/* increment for strip charts, in seconds */
    short	wrapX;		/* wrap around if time exceeds extentVal */
    int		x;		/* x pos of window on the display */
    int		y;		/* y pos of window on the display */
    int		width;		/* width of the window */
    int		height;		/* height of the window */
} SYD_PL_MSTR;

/*-----------------------------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
SYD_PL_SLAVE *sydPlotChanAdd();
SYD_PL_SLAVE *sydPlotChan_event();
void sydPlotAxisAutoRange();
void sydPlotAxisSetup();
long sydPlotAxisSetAttr();
long sydPlotDone();
long sydPlotInit();
long sydPlotInitUW();
long sydPlotSetAttr();
long sydPlotSetTitles();
long sydPlotWinLoop();
void sydPlot();
void sydPlotSamples();
long sydPlotTimeCursor();
void sydPlot_StripYShift();
void sydPlot_StripYYShift();

#endif
